#ifndef SINGLETOPDPDMAKER_TREEMAKER_H
#define SINGLETOPDPDMAKER_TREEMAKER_H

#include <string>
#include <vector>
#include <map>

#include "GaudiKernel/Algorithm.h"
#include "GaudiKernel/MsgStream.h"
#include "GaudiKernel/ITHistSvc.h"
#include "StoreGate/StoreGate.h"

#include "EventInfo/EventInfo.h"
#include "EventInfo/EventID.h"
#include "EventInfo/EventType.h"
#include "MissingETEvent/MissingET.h"
#include "egammaEvent/PhotonContainer.h"
#include "egammaEvent/Photon.h"
#include "egammaEvent/ElectronContainer.h"
#include "egammaEvent/Electron.h"
#include "egammaEvent/EMShower.h"
#include "egammaEvent/egammaParamDefs.h"
#include "egammaEvent/egammaPIDdefs.h"
#include "egammaEvent/EMTrackMatch.h"
#include "CaloEvent/CaloSampling.h"
#include "CaloEvent/CaloCluster.h"
#include "muonEvent/MuonContainer.h"
#include "muonEvent/Muon.h"
#include "muonEvent/MuonParamDefs.h"
#include "tauEvent/TauJetContainer.h"
#include "tauEvent/TauJet.h"
#include "tauEvent/TauJetParameters.h"
#include "JetEvent/JetCollection.h"
#include "JetEvent/Jet.h"
#include "JetTagEvent/TrackAssociation.h"
#include "JetTagInfo/SoftElectronInfo.h"
#include "JetTagInfo/SoftMuonInfo.h"
#include "JetUtils/JetCaloHelper.h"
#include "Particle/TrackParticleContainer.h"
#include "Particle/TrackParticle.h"

#include "GeneratorObjects/McEventCollection.h"
#include "HepMC/GenEvent.h"
#include "HepMC/WeightContainer.h"

#include "CLHEP/Vector/LorentzVector.h"

#include "TTree.h"

class TreeMaker : public Algorithm
{

public:

  TreeMaker(const std::string& name, ISvcLocator* pSvcLocator);
  virtual ~TreeMaker();
  
  StatusCode initialize();
  StatusCode finalize();
  StatusCode execute();
  
  bool TestContainerType();
  bool TestContainerLabel();
  
  void SetBranches();
  void DeleteBranches();
  void ClearBranches();
  void FillBranches();
    
// EventInfoTree.cxx
  
  void SetEventInfo();
  void ClearEventInfo();
  void FillEventInfo();
    
// MissingEtTree.cxx

  void SetMissingEt();
  void ClearMissingEt();
  void FillMissingEt();
    
// PhotonTree.cxx

  void SetPhotons();
  void DeletePhotons();
  void ClearPhotons();
  void FillPhotons();  
  
// ElectronTree.cxx
  
  void SetElectrons();
  void DeleteElectrons();
  void ClearElectrons();
  void FillElectrons();
    
// MuonTree.cxx
 
  void SetMuons();
  void DeleteMuons();
  void ClearMuons();
  void FillMuons();  
  
// TauJetTree.cxx
    
  void SetTauJets();
  void DeleteTauJets();
  void ClearTauJets();
  void FillTauJets();  
  
// JetTree.cxx
  
  void SetJets();
  void DeleteJets();
  void ClearJets();
  void FillJets();  

private:

  StoreGateSvc * aStoreGateSvc;
  ITHistSvc    * aTHistSvc;
  TTree        * aTree;
  
  int InputEventNumber;
  int OutputEventNumber;
    
// Input Containers
      
  std::string EventInfoName;
  std::string McEventContainerName;
  std::string TrackParticleContainerName; 
  
  std::vector<std::string> MissingEtName; 
  std::vector<std::string> MissingEtLabel;
  
  std::vector<std::string> ContainerName;
  std::vector<std::string> ContainerLabel;
  
  std::vector<double> PtCut;
    
  std::vector<std::string> PhotonContainerName;
  std::vector<std::string> ElectronContainerName;
  std::vector<std::string> MuonContainerName;
  std::vector<std::string> TauJetContainerName;
  std::vector<std::string> JetContainerName;
  
  std::vector<std::string> PhotonContainerLabel;
  std::vector<std::string> ElectronContainerLabel;
  std::vector<std::string> MuonContainerLabel;
  std::vector<std::string> TauJetContainerLabel;
  std::vector<std::string> JetContainerLabel;
              
  std::map<std::string, double> PhotonPtCut;
  std::map<std::string, double> ElectronPtCut;
  std::map<std::string, double> MuonPtCut;
  std::map<std::string, double> TauJetPtCut;
  std::map<std::string, double> JetPtCut;
    
  const EventInfo                   * aEventInfo;
  const McEventCollection           * aMcEventContainer;
  const Rec::TrackParticleContainer * aTrackParticleContainer;
  
  std::map<std::string, const MissingET*> aMissingEt;
  
  std::map<std::string, const PhotonContainer          *> aPhotonContainer;
  std::map<std::string, const ElectronContainer        *> aElectronContainer;
  std::map<std::string, const Analysis::MuonContainer  *> aMuonContainer;
  std::map<std::string, const Analysis::TauJetContainer*> aTauJetContainer;
  std::map<std::string, const JetCollection            *> aJetContainer;
      
// Output Tree
  
  std::string TreeName;
    
  double TrackMuonDRCutMin;
  double TrackMuonDRCutMax;
  double TrackMuonPtCut;
    
  #include "EventInfoTree.h"
  #include "MissingEtTree.h"
  #include "PhotonTree.h"
  #include "ElectronTree.h"
  #include "MuonTree.h"
  #include "TauJetTree.h"
  #include "JetTree.h"
            
};

#endif







