#include "SingleTopDPDMaker/ContainerThinning.h"

ContainerThinning::ContainerThinning(const std::string& name, ISvcLocator* pSvcLocator) : Algorithm(name, pSvcLocator),
aThinningSvc("ThinningSvc", name)
{

  declareProperty("ContainerName", ContainerName);
  
  declareProperty("SelectionInfoContainerName", SelectionInfoContainerName);
  declareProperty("RequiredSelectionLabel",     RequiredSelectionLabel    );  
            
}

ContainerThinning::~ContainerThinning(){}

StatusCode ContainerThinning::initialize()
{
  
  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Initializing " << name() << endreq;
                    
  bool Test1 = ContainerName.size() > 0;
  bool Test2 = ContainerName.size() == SelectionInfoContainerName.size();
  
  bool isTestContainerSize = Test1 && Test2;
  
  if(!isTestContainerSize)
  {
  
    MsgLog << MSG::FATAL << "Wrong Container Size Properties :" << endreq;
    
    if(!Test1) MsgLog << MSG::FATAL << "Wrong Container Name Size"                << endreq;
    if(!Test2) MsgLog << MSG::FATAL << "Wrong Selection Info Container Name Size" << endreq;
    
    return StatusCode::FAILURE;
    
  }
  
  bool isTestContainerType = this->TestContainerType();
      
  if(!isTestContainerType)
  {
    MsgLog << MSG::FATAL << "Wrong Container Type Properties" << endreq;
    return StatusCode::FAILURE;
  }
    
  for(unsigned int i=0; i<ContainerName.size(); i++)
  {
    
    MsgLog << MSG::INFO << "Container Name                = " << ContainerName[i]              << endreq;
    MsgLog << MSG::INFO << "Selection Info Container Name = " << SelectionInfoContainerName[i] << endreq;
    
    std::vector<std::string> SelectionLabel = RequiredSelectionLabel[i];
    
    for(unsigned int j=0; j<SelectionLabel.size(); j++)
    {
      MsgLog << MSG::INFO << "Required Selection Label = " << SelectionLabel[j] << endreq;
    }
    
  }
  
  StatusCode status = service("StoreGateSvc", aStoreGateSvc);

  if(status.isFailure()) 
  {
    MsgLog << MSG::FATAL << "StoreGateSvc Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  status = aThinningSvc.retrieve();
  
  if(status.isFailure()) 
  {
    MsgLog << MSG::FATAL << "ThinningSvc Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  InputEventNumber  = 0;
  OutputEventNumber = 0;
                
  return StatusCode::SUCCESS;
  
}

StatusCode ContainerThinning::finalize()
{

  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Finalizing " << name() << endreq;
  
  MsgLog << MSG::INFO << "Input  Event Number = " << InputEventNumber  << endreq;
  MsgLog << MSG::INFO << "Output Event Number = " << OutputEventNumber << endreq;
    
  return StatusCode::SUCCESS;
  
}

StatusCode ContainerThinning::execute()
{

  MsgStream MsgLog(msgSvc(), name());
  
  StatusCode status;
  
  InputEventNumber++;
            
  for(unsigned int i=0; i<ContainerName.size(); i++)
  {
  
    status = aStoreGateSvc->retrieve(aSelectionInfoContainer, SelectionInfoContainerName[i]);
  
    if(status.isFailure())
    {
      MsgLog << MSG::FATAL << SelectionInfoContainerName[i] << " Container Unavailable" << endreq;
      return StatusCode::FAILURE;
    }
  
    bool isThinned = false;
  
    if     (ContainerType[i] == "Photon")    isThinned = this->Thin(i, aPhotonContainer);
    else if(ContainerType[i] == "Electron")  isThinned = this->Thin(i, aElectronContainer);
    else if(ContainerType[i] == "Muon")      isThinned = this->Thin(i, aMuonContainer);
    else if(ContainerType[i] == "TauJet")    isThinned = this->Thin(i, aTauJetContainer);
    else if(ContainerType[i] == "Jet")       isThinned = this->Thin(i, aJetContainer);
    else if(ContainerType[i] == "Truth")     isThinned = this->Thin(i, aTruthParticleContainer);
    else if(ContainerType[i] == "Composite") isThinned = this->Thin(i, aCompositeParticleContainer);
    
    if(!isThinned) 
    {
      MsgLog << MSG::FATAL << ContainerName[i] << " Thinning Unavailable" << endreq;
      return StatusCode::FAILURE;
    }
              
  }
  
  setFilterPassed(true);
  
  OutputEventNumber++;
        
  return StatusCode::SUCCESS;
  
}

template <class CT> bool ContainerThinning::Thin(unsigned int i, const CT * aContainer)
{

  StatusCode status;
  
  status = aStoreGateSvc->retrieve(aContainer, ContainerName[i]);
  if(status.isFailure()) return false;
    
  SelectionInfoContainer::const_iterator InfoItr;
  
  for(InfoItr=aSelectionInfoContainer->begin(); InfoItr!=aSelectionInfoContainer->end(); InfoItr++)
  {
  
    SelectionInfo * aSelectionInfo = *InfoItr;
    
    std::string Name       = aSelectionInfo->GetName();
    std::string Label      = aSelectionInfo->GetLabel();
    std::vector<bool> Mask = aSelectionInfo->GetMask();
    
    if(Name != ContainerName[i]) continue;
    
    std::vector<std::string>::const_iterator LabelItr;
      
    for(LabelItr=RequiredSelectionLabel[i].begin(); LabelItr!=RequiredSelectionLabel[i].end(); LabelItr++)
    {
    
      std::string SelectionLabel = *LabelItr;
      
      if(Label == SelectionLabel)
      {
        status = aThinningSvc->filter(*aContainer, Mask, IThinningSvc::Operator::And);
        if(status.isFailure()) return false;
      }
      
    }
    
  }
  
  return true;

}

bool ContainerThinning::TestContainerType()
{

  for(unsigned int i=0; i<ContainerName.size(); i++)
  {
    
    std::string Name = ContainerName[i];
    
    bool Veto = Name.find("Truth", 0) == std::string::npos && Name.find("Composite", 0) == std::string::npos;
  
    if     (Name.find("Photon",    0) != std::string::npos && Veto) ContainerType.push_back("Photon");
    else if(Name.find("Electron",  0) != std::string::npos && Veto) ContainerType.push_back("Electron");
    else if(Name.find("Muon",      0) != std::string::npos && Veto) ContainerType.push_back("Muon");
    else if(Name.find("Tau",       0) != std::string::npos && Veto) ContainerType.push_back("TauJet");
    else if(Name.find("Jet",       0) != std::string::npos && Veto) ContainerType.push_back("Jet");
    else if(Name.find("Truth",     0) != std::string::npos)         ContainerType.push_back("Truth");
    else if(Name.find("Composite", 0) != std::string::npos)         ContainerType.push_back("Composite");
    else return false;
    
  }
    
  return true;

}
