#include "SingleTopDPDMaker/EFDecisionSelection.h"

EFDecisionSelection::EFDecisionSelection(const std::string& name, ISvcLocator* pSvcLocator) : Algorithm(name, pSvcLocator),
aTriggerDecisionTool("TrigDec::TrigDecisionTool")
{

  declareProperty("ElectronChainName", ElectronChainName);
  declareProperty("PhotonChainName",   PhotonChainName  );
  declareProperty("MuonChainName",     MuonChainName    );
  declareProperty("TauChainName",      TauChainName     );
  declareProperty("JetChainName",      JetChainName     );
  declareProperty("BJetChainName",     BJetChainName    );
  
  declareProperty("GetElectronPassedFeatures", GetElectronPassedFeatures = false);
  declareProperty("GetPhotonPassedFeatures",   GetPhotonPassedFeatures   = false);
  declareProperty("GetMuonPassedFeatures",     GetMuonPassedFeatures     = false);
  declareProperty("GetTauPassedFeatures",      GetTauPassedFeatures      = false);
  declareProperty("GetJetPassedFeatures",      GetJetPassedFeatures      = false);
  declareProperty("GetBJetPassedFeatures",     GetBJetPassedFeatures     = false);
  
  declareProperty("OutputContainerName", OutputContainerName );
  
  declareProperty("TriggerDecisionTool", aTriggerDecisionTool);
            
}

EFDecisionSelection::~EFDecisionSelection(){}

StatusCode EFDecisionSelection::initialize()
{
  
  MsgStream MsgLog(messageService(), name());
  
  MsgLog << MSG::INFO << "Initializing " << name() << endreq;
  
  bool isTestContainerSize = OutputContainerName.size() > 0;
  
  if(!isTestContainerSize)
  {
    MsgLog << MSG::FATAL << "Wrong Container Size Properties" << endreq;
    return StatusCode::FAILURE;
  }
  
  bool isTestContainerType = this->TestContainerType();
      
  if(!isTestContainerType)
  {
    MsgLog << MSG::FATAL << "Wrong Container Type Properties" << endreq;
    return StatusCode::FAILURE;
  }
  
  bool isTestChainSize = this->TestChainSize();
  
  if(!isTestChainSize)
  {
    MsgLog << MSG::FATAL << "Wrong Chain Size Properties" << endreq;
    return StatusCode::FAILURE;
  } 
              
  for(unsigned int i=0; i<OutputContainerName.size(); i++)
  {
    MsgLog << MSG::INFO << "Output Container Name = " << OutputContainerName[i] << endreq;
  }
  
  if(isElectron)
  {
    MsgLog << MSG::INFO << "Electron Chain Name          = " << ElectronChainName         << endreq;
    MsgLog << MSG::INFO << "Get Electron Passed Features = " << GetElectronPassedFeatures << endreq;
  }
  
  if(isPhoton)
  {
    MsgLog << MSG::INFO << "Photon Chain Name          = " << PhotonChainName         << endreq;
    MsgLog << MSG::INFO << "Get Photon Passed Features = " << GetPhotonPassedFeatures << endreq;
  }
  
  if(isMuon)
  {
    MsgLog << MSG::INFO << "Muon Chain Name          = " << MuonChainName         << endreq;
    MsgLog << MSG::INFO << "Get Muon Passed Features = " << GetMuonPassedFeatures << endreq;
  }
  
  if(isTau)
  {
    MsgLog << MSG::INFO << "Tau Chain Name          = " << TauChainName         << endreq;
    MsgLog << MSG::INFO << "Get Tau Passed Features = " << GetTauPassedFeatures << endreq;
  }
  
  if(isJet)
  {
    MsgLog << MSG::INFO << "Jet Chain Name          = " << JetChainName         << endreq;
    MsgLog << MSG::INFO << "Get Jet Passed Features = " << GetJetPassedFeatures << endreq;
  }
  
  if(isBJet)
  {
    MsgLog << MSG::INFO << "BJet Chain Name          = " << BJetChainName         << endreq;
    MsgLog << MSG::INFO << "Get BJet Passed Features = " << GetBJetPassedFeatures << endreq;
  }
    
  StatusCode status;
            
  status = service("StoreGateSvc", aStoreGateSvc);

  if(status.isFailure()) 
  {
    MsgLog << MSG::FATAL << "StoreGateSvc Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  status = aTriggerDecisionTool.retrieve();
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Trigger Decision Tool Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  InputEventNumber  = 0;
  OutputEventNumber = 0;
        
  return StatusCode::SUCCESS;
  
}

StatusCode EFDecisionSelection::finalize()
{

  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Finalizing " << name() << endreq;
  
  MsgLog << MSG::INFO << "Input  Event Number = " << InputEventNumber  << endreq;
  MsgLog << MSG::INFO << "Output Event Number = " << OutputEventNumber << endreq;
  
  return StatusCode::SUCCESS;
  
}

bool EFDecisionSelection::TestContainerType()
{
  
  for(unsigned int i=0; i<OutputContainerName.size(); i++)
  {
    
    std::string Name = OutputContainerName[i];
    
    if(Name.find("EF", 0) == std::string::npos) return false;
    
    if(Name.find("Electron", 0) != std::string::npos) 
    {
      isElectron = true;
      ElectronContainerName = Name;
    }
    
    else if(Name.find("Photon", 0) != std::string::npos) 
    {
      isPhoton = true;
      PhotonContainerName = Name;
    }
    
    else if(Name.find("Muon", 0) != std::string::npos) 
    {
      isMuon = true;
      MuonContainerName = Name;
    }
    
    else if(Name.find("Tau", 0) != std::string::npos) 
    {
      isTau = true;
      TauContainerName = Name;
    }
    
    else if(Name.find("BJet", 0) != std::string::npos) 
    {
      isBJet = true;
      BJetContainerName = Name;
    }
    else if(Name.find("Jet", 0) != std::string::npos) 
    {
      isJet = true;
      JetContainerName = Name;
    }
    
    else return false;
        
  }
    
  return true;

}

bool EFDecisionSelection::TestChainSize()
{

  if(isElectron && ElectronChainName.size() == 0) return false;
  if(isPhoton   && PhotonChainName.size()   == 0) return false; 
  if(isMuon     && MuonChainName.size()     == 0) return false;  
  if(isTau      && TauChainName.size()      == 0) return false; 
  if(isJet      && JetChainName.size()      == 0) return false; 
  if(isBJet     && BJetChainName.size()     == 0) return false;
    
  return true;

}

StatusCode EFDecisionSelection::execute()
{
  
  InputEventNumber++;
  
  StatusCode status;
  
  if(isElectron) 
  {
    status = this->executeElectron();
    if(status.isFailure()) return StatusCode::FAILURE;
  }
  
  if(isPhoton) 
  {
    status = this->executePhoton();
    if(status.isFailure()) return StatusCode::FAILURE;
  }
    
  if(isMuon)
  {
    status = this->executeMuon();
    if(status.isFailure()) return StatusCode::FAILURE;
  }
  
  if(isTau)
  {
    status = this->executeTau();
    if(status.isFailure()) return StatusCode::FAILURE;
  }
  
  if(isJet)
  {
    status = this->executeJet();
    if(status.isFailure()) return StatusCode::FAILURE;
  }
  
  if(isBJet)
  {
    status = this->executeBJet();
    if(status.isFailure()) return StatusCode::FAILURE;
  }
  
  setFilterPassed(true);
  
  OutputEventNumber++;
  
  return StatusCode::SUCCESS;
  
}

StatusCode EFDecisionSelection::executeElectron()
{

  MsgStream MsgLog(msgSvc(), name());
    
  std::vector<std::string>::const_iterator Itr;
  
  for(Itr=ElectronChainName.begin(); Itr!=ElectronChainName.end(); Itr++)
  {
  
    std::vector<const egammaContainer*> Containers;
  
    std::string Name = *Itr;
    
    HLT::ErrorCode status;
    
    if(GetElectronPassedFeatures) status = aTriggerDecisionTool->getPassFeatures(Name, Containers);
    else                          status = aTriggerDecisionTool->getAllFeatures (Name, Containers);
    
    if(status.code != 0)
    {
      MsgLog << MSG::FATAL << "No Input Electron Container Available" << endreq;
      return StatusCode::FAILURE;
    }
    
    std::vector<const egammaContainer*>::const_iterator ContainerItr;
    
    for(ContainerItr=Containers.begin(); ContainerItr!=Containers.end(); ContainerItr++)
    {
      ElectronContainers.insert(*ContainerItr);
    }
          
  }
      
  aElectronContainer = new INavigable4MomentumCollection(SG::VIEW_ELEMENTS);
  
  StatusCode status;
    
  status = aStoreGateSvc->record(aElectronContainer, ElectronContainerName);
    
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Output Electron Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
    
  this->SelectElectrons();
    
  status = aStoreGateSvc->setConst(aElectronContainer);
    
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Output Electron Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  return StatusCode::SUCCESS;
    
}

StatusCode EFDecisionSelection::executePhoton()
{

  
  return StatusCode::SUCCESS;
    
}

StatusCode EFDecisionSelection::executeMuon()
{

  return StatusCode::SUCCESS;
    
}

StatusCode EFDecisionSelection::executeTau()
{

  return StatusCode::SUCCESS;
    
}

StatusCode EFDecisionSelection::executeJet()
{

  return StatusCode::SUCCESS;
    
}

StatusCode EFDecisionSelection::executeBJet()
{
  
  return StatusCode::SUCCESS;
    
}
  
void EFDecisionSelection::SelectElectrons()
{

      
}

void EFDecisionSelection::SelectPhotons()
{

      
}

void EFDecisionSelection::SelectMuons()
{

      
}

void EFDecisionSelection::SelectTaus()
{

      
}

void EFDecisionSelection::SelectJets()
{

      
}

void EFDecisionSelection::SelectBJets()
{


}
