#include "SingleTopDPDMaker/ElectronSelection.h"

ElectronSelection::ElectronSelection(const std::string& name, ISvcLocator* pSvcLocator) : Algorithm(name, pSvcLocator)
{

  declareProperty("InputContainerName",  InputContainerName  = "ElectronAODCollection"    );
  declareProperty("OutputContainerName", OutputContainerName = "SelectedElectronContainer");
  declareProperty("JetContainerName",    JetContainerName    = "Cone4H1TowerJets"         );
  
  declareProperty("SelectionInfoContainerName", SelectionInfoContainerName = "None"             );
  declareProperty("SelectionLabel",             SelectionLabel             = "ElectronSelection");            
  
  declareProperty("ApplyEtConeCut",   ApplyEtConeCut   = false );
  declareProperty("ApplyJetDRCut",    ApplyJetDRCut    = false );
  declareProperty("ApplyEtaCrackCut", ApplyEtaCrackCut = false );
  
  declareProperty("Author",         Author         = "Electron");
  declareProperty("EmMask",         EmMask         = "Medium"  );
  declareProperty("PtCut",          PtCut          = 20.0*GeV  );
  declareProperty("EtaCut",         EtaCut         = 2.5       );
  declareProperty("ConeSize",       ConeSize       = 0.2       );
  declareProperty("EtConeCut",      EtConeCut      = 6.0*GeV   );
  declareProperty("JetDRCut",       JetDRCut       = 0.4       );
  declareProperty("MinEtaCrack",    MinEtaCrack    = 1.37      );
  declareProperty("MaxEtaCrack",    MaxEtaCrack    = 1.52      );
  
}

ElectronSelection::~ElectronSelection(){}

StatusCode ElectronSelection::initialize()
{
  
  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Initializing " << name() << endreq;
  
  MsgLog << MSG::INFO << "Input  Container Name = " << InputContainerName  << endreq;
  MsgLog << MSG::INFO << "Output Container Name = " << OutputContainerName << endreq;
  
  if(SelectionInfoContainerName != "None")
  {
    MsgLog << MSG::INFO << "Selection Info Container Name = " << SelectionInfoContainerName << endreq;
    MsgLog << MSG::INFO << "Selection Label               = " << SelectionLabel             << endreq;
  }

  MsgLog << MSG::INFO << "Apply Et Cone Cut   = " << ApplyEtConeCut   << endreq;  
  MsgLog << MSG::INFO << "Apply Jet DR Cut    = " << ApplyJetDRCut    << endreq; 
  MsgLog << MSG::INFO << "Apply Eta Crack Cut = " << ApplyEtaCrackCut << endreq; 
  
  MsgLog << MSG::INFO << "Author      = " << Author              << endreq;
  MsgLog << MSG::INFO << "Em Mask     = " << EmMask              << endreq;
  MsgLog << MSG::INFO << "Pt Cut      = " << PtCut/GeV << " GeV" << endreq;
  MsgLog << MSG::INFO << "Eta Cut     = " << EtaCut              << endreq;
  
  if(ApplyEtConeCut)
  {
    MsgLog << MSG::INFO << "Cone Size   = " << ConeSize                << endreq;
    MsgLog << MSG::INFO << "Et Cone Cut = " << EtConeCut/GeV << " GeV" << endreq;
  }
  
  if(ApplyJetDRCut)
  {
    MsgLog << MSG::INFO << "Jet Container Name = " << JetContainerName << endreq;
    MsgLog << MSG::INFO << "Jet DR Cut         = " << JetDRCut         << endreq;
  }
  
  if(ApplyEtaCrackCut)
  {
    MsgLog << MSG::INFO << "Min Eta Crack = " << MinEtaCrack << endreq;
    MsgLog << MSG::INFO << "Max Eta Crack = " << MaxEtaCrack << endreq;
  }
      
  bool isTestAuthorOption   = this->TestAuthorOption  ();
  bool isTestEmMaskOption   = this->TestEmMaskOption  ();
  bool isTestConeSizeOption = this->TestConeSizeOption();
  
  if(!isTestAuthorOption)
  {
    MsgLog << MSG::FATAL << "Wrong Author Property" << endreq;
    return StatusCode::FAILURE;
  }
  
  if(!isTestEmMaskOption)
  {
    MsgLog << MSG::FATAL << "Wrong Em Mask Property" << endreq;
    return StatusCode::FAILURE;
  }
  
  if(!isTestConeSizeOption)
  {
    MsgLog << MSG::FATAL << "Wrong Cone Size Property" << endreq;
    return StatusCode::FAILURE;
  }
  
  StatusCode status = service("StoreGateSvc", aStoreGateSvc);

  if(status.isFailure()) 
  {
    MsgLog << MSG::FATAL << "StoreGateSvc Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  InputEventNumber  = 0;
  OutputEventNumber = 0;
  
  InputObjectNumber  = 0;
  OutputObjectNumber = 0;
    
  return StatusCode::SUCCESS;
  
}

StatusCode ElectronSelection::finalize()
{
  
  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Finalizing " << name() << endreq;
  
  MsgLog << MSG::INFO << "Input  Event Number  = " << InputEventNumber  << endreq;
  MsgLog << MSG::INFO << "Output Event Number  = " << OutputEventNumber << endreq;
  
  MsgLog << MSG::INFO << "Input  Object Number = " << InputObjectNumber  << endreq;
  MsgLog << MSG::INFO << "Output Object Number = " << OutputObjectNumber << endreq;
  
  return StatusCode::SUCCESS;
  
}

StatusCode ElectronSelection::execute()
{
  
  MsgStream MsgLog(msgSvc(), name());
  
  StatusCode status;
  
  InputEventNumber++;
      
  status = aStoreGateSvc->retrieve(aInputContainer, InputContainerName);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Input Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  if(ApplyJetDRCut)
  {
  
    status = aStoreGateSvc->retrieve(aJetContainer, JetContainerName);
  
    if(status.isFailure())
    {
      MsgLog << MSG::FATAL << "Jet Container Unavailable" << endreq;
      return StatusCode::FAILURE;
    }
    
  }
  
  aOutputContainer = new ElectronContainer(SG::VIEW_ELEMENTS);
  
  status = aStoreGateSvc->record(aOutputContainer, OutputContainerName);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Output Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  status = aStoreGateSvc->retrieve(aSelectionInfoContainer, SelectionInfoContainerName);
  
  if(status.isFailure())
  {
    
    if(SelectionInfoContainerName != "None")
    {
    
      if(InputEventNumber == 1) MsgLog << MSG::INFO << "Creating Selection Info Container" << endreq;
    
      aSelectionInfoContainer = new SelectionInfoContainer(SG::VIEW_ELEMENTS);
    
      status = aStoreGateSvc->record(aSelectionInfoContainer, SelectionInfoContainerName);
    
      if(status.isFailure())
      {
        MsgLog << MSG::FATAL << "Selection Info Container Unavailable" << endreq;
        return StatusCode::FAILURE;
      }
    
    }
    
    else 
    {
      aSelectionInfoContainer = 0;
      if(InputEventNumber == 1) MsgLog << MSG::INFO << "No Selection Info Container" << endreq;
    }
        
  }
    
  Mask.clear();
  
  this->Select();
  
  if(aSelectionInfoContainer)
  {  
    SelectionInfo * aSelectionInfo = new SelectionInfo();
  
    aSelectionInfo->SetName(InputContainerName);
    aSelectionInfo->SetLabel(SelectionLabel);
    aSelectionInfo->SetMask(Mask);
    
    aSelectionInfoContainer->push_back(aSelectionInfo);
  }
      
  status = aStoreGateSvc->setConst(aOutputContainer);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Output Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
    
  InputObjectNumber  += static_cast<int>(aInputContainer->size());
  OutputObjectNumber += static_cast<int>(aOutputContainer->size());
  
  setFilterPassed(true);
  
  OutputEventNumber++;
    
  return StatusCode::SUCCESS;
  
}

void ElectronSelection::Select()
{

  ElectronContainer::const_iterator Itr;
  
  for(Itr=aInputContainer->begin(); Itr!= aInputContainer->end(); Itr++)
  {
    
    const Analysis::Electron * aElectron = *Itr;
    
    bool isSelected = this->SelectElectron(aElectron);
                
    if(isSelected)
    {           
      Analysis::Electron * aSelectedElectron = new Analysis::Electron(*aElectron);
      aOutputContainer->push_back(aSelectedElectron);
      
      Mask.push_back(true);
    }
    
    else 
    {
      Mask.push_back(false);
    }
               
  }
    
}

bool ElectronSelection::SelectElectron(const Analysis::Electron * aElectron)
{
  
  bool isPtCut       = aElectron->pt() > PtCut;
  bool isEtaCut      = fabs(aElectron->eta()) < EtaCut;
    
  bool isAuthor      = this->TestAuthor(aElectron);
  bool isEmCut       = this->TestEmMask(aElectron);
    
  bool isIsolated    = true;
  bool isJetIsolated = true;
    
  if(ApplyEtConeCut) isIsolated    = this->TestEtCone(aElectron);
  if(ApplyJetDRCut)  isJetIsolated = this->TestJetDeltaR(aElectron);
    
  bool isEtaCrack    = false;
  if(ApplyEtaCrackCut) isEtaCrack = fabs(aElectron->eta()) >= MinEtaCrack && fabs(aElectron->eta()) <= MaxEtaCrack;
    
  bool isSelected = isPtCut && isEtaCut && isAuthor && isEmCut && isIsolated && isJetIsolated && !isEtaCrack;
  
  return isSelected;

}

bool ElectronSelection::TestAuthorOption()
{

  bool Test = false;
  
  if     (Author == "All")            Test = true;
  else if(Author == "OnlyElectron")   Test = true;
  else if(Author == "OnlySofte")      Test = true;
  else if(Author == "Electron")       Test = true;
  else if(Author == "Softe")          Test = true;
  else if(Author == "EitherElectron") Test = true;
  
  return Test;

}

bool ElectronSelection::TestAuthor(const Analysis::Electron * aElectron)
{

  int author = aElectron->author();

  bool Test = false;
  
  if     (Author == "All")            Test = true;
  else if(Author == "OnlyElectron")   Test = (author == 1);
  else if(Author == "OnlySofte")      Test = (author == 2);
  else if(Author == "Electron")       Test = (author == 1) || (author == 3);
  else if(Author == "Softe")          Test = (author == 2) || (author == 3);
  else if(Author == "EitherElectron") Test = (author == 1) || (author == 2) || (author == 3);
    
  return Test; 
  
//  0x0 = egammaParameters::AuthorUnknown
//  0x1 = egammaParameters::AuthorElectron 
//  0x2 = egammaParameters::AuthorSofte
//  0x4 = egammaParameters::AuthorPhoton

}

bool ElectronSelection::TestEmMaskOption()
{

  bool Test = false;
  
  if     (EmMask == "None")             Test = true;
  else if(EmMask == "Medium")           Test = true;
  else if(EmMask == "Loose")            Test = true;
  else if(EmMask == "Tight")            Test = true;
  else if(EmMask == "TightNoIsolation") Test = true;
  
  return Test;  

}

bool ElectronSelection::TestEmMask(const Analysis::Electron * aElectron)
{

  bool Test = false;
  
  if     (EmMask == "None")              Test = true;
  else if(EmMask == "Medium")            Test = aElectron->isem(egammaPID::ElectronMedium)           == 0;
  else if(EmMask == "Loose")             Test = aElectron->isem(egammaPID::ElectronLoose)            == 0;
  else if(EmMask == "Tight")             Test = aElectron->isem(egammaPID::ElectronTight)            == 0;
  else if(EmMask == "TightNoIsolation")  Test = aElectron->isem(egammaPID::ElectronTightNoIsolation) == 0;
  
  return Test;  

}

bool ElectronSelection::TestConeSizeOption()
{

  bool Test = false;
  
  if(ConeSize == 0.2) 
  {
    Test = true;
    ConeSizeParameter = egammaParameters::etcone20;
  }
  
  return Test; 

}

bool ElectronSelection::TestEtCone(const Analysis::Electron * aElectron)
{

  bool Test = false;

  const EMShower * aEMShower = aElectron->detail<EMShower>("egDetailAOD");
    
  if(aEMShower) 
  {
    Test = aEMShower->parameter(ConeSizeParameter) < EtConeCut;
  }
  
  return Test; 

}

bool ElectronSelection::TestJetDeltaR(const Analysis::Electron * aElectron)
{

  bool Test = true;
  
  JetCollection::const_iterator Itr;
  
  for(Itr=aJetContainer->begin(); Itr!=aJetContainer->end(); Itr++)
  {
  
    double dR = P4Help::deltaR(aElectron, *Itr);
    Test = dR > JetDRCut;
    if(!Test) break;
  
  }
      
  return Test;

}
