#include "SingleTopDPDMaker/EventFilter.h"

EventFilter::EventFilter(const std::string& name, ISvcLocator* pSvcLocator) : Algorithm(name, pSvcLocator)
{

  declareProperty("MissingEtName",      MissingEtName      = "None" );
  declareProperty("MissingEtThreshold", MissingEtThreshold = 0.0*GeV);  
  
  declareProperty("ContainerName",       ContainerName      );
  declareProperty("FirstContainerName",  FirstContainerName );
  declareProperty("SecondContainerName", SecondContainerName);
  
  declareProperty("PtThreshold",        PtThreshold       );
  declareProperty("FirstPtThreshold",   FirstPtThreshold  );
  declareProperty("SecondPtThreshold",  SecondPtThreshold );
  
  declareProperty("EtaThreshold",       EtaThreshold      );
  declareProperty("FirstEtaThreshold",  FirstEtaThreshold );
  declareProperty("SecondEtaThreshold", SecondEtaThreshold);
         
  declareProperty("MinNumber",    MinNumber   );
  declareProperty("MaxNumber",    MaxNumber   );  
  declareProperty("MinSumNumber", MinSumNumber);
  declareProperty("MaxSumNumber", MaxSumNumber); 
             
}

EventFilter::~EventFilter(){}

StatusCode EventFilter::initialize()
{
  
  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Initializing " << name() << endreq;
  
  if(ContainerName.size() > 0)
  {
   
    bool Test1 = ContainerName.size() == PtThreshold.size();
    bool Test2 = ContainerName.size() == EtaThreshold.size();
    bool Test3 = ContainerName.size() == MinNumber.size();
    bool Test4 = ContainerName.size() == MaxNumber.size();
    
    bool isTestContainerSize = Test1 && Test2 && Test3 && Test4;
    
    if(!isTestContainerSize)
    {
      
      MsgLog << MSG::FATAL << "Wrong Single Container Size Properties :" << endreq;
      
      if(!Test1) MsgLog << MSG::FATAL << "Wrong Pt Threshold Size"  << endreq; 
      if(!Test2) MsgLog << MSG::FATAL << "Wrong Eta Threshold Size" << endreq; 
      if(!Test3) MsgLog << MSG::FATAL << "Wrong Min Number Size"    << endreq; 
      if(!Test4) MsgLog << MSG::FATAL << "Wrong Max Number Size"    << endreq;
      
      return StatusCode::FAILURE;
      
    }
    
  }
      
  if(FirstContainerName.size() > 0)
  {
    bool Test1 = FirstContainerName.size() == SecondContainerName.size();
    bool Test2 = FirstContainerName.size() == FirstPtThreshold.size();
    bool Test3 = FirstContainerName.size() == SecondPtThreshold.size();
    bool Test4 = FirstContainerName.size() == FirstEtaThreshold.size();
    bool Test5 = FirstContainerName.size() == SecondEtaThreshold.size();
    bool Test6 = FirstContainerName.size() == MinSumNumber.size();
    bool Test7 = FirstContainerName.size() == MaxSumNumber.size();
    
    bool isTestContainerSize = Test1 && Test2 && Test3 && Test4 && Test5 && Test6 && Test7;
    
    if(!isTestContainerSize)
    {
      
      MsgLog << MSG::FATAL << "Wrong Double Container Size Properties :" << endreq;
      
      if(!Test1) MsgLog << MSG::FATAL << "Wrong Second Container Name Size" << endreq; 
      if(!Test2) MsgLog << MSG::FATAL << "Wrong First Pt Threshold Size"    << endreq;
      if(!Test3) MsgLog << MSG::FATAL << "Wrong Second Pt Threshold Size"   << endreq; 
      if(!Test4) MsgLog << MSG::FATAL << "Wrong First Eta Threshold Size"   << endreq;
      if(!Test5) MsgLog << MSG::FATAL << "Wrong Second Eta Threshold Size"  << endreq;
      if(!Test6) MsgLog << MSG::FATAL << "Wrong Min Sum Number Size"        << endreq; 
      if(!Test7) MsgLog << MSG::FATAL << "Wrong Max Sum Number Size"        << endreq;
      
      return StatusCode::FAILURE;
      
    }
    
  }
  
  bool isTestContainerType = this->TestContainerType();
  
  if(!isTestContainerType)
  {
    MsgLog << MSG::FATAL << "Wrong Container Type Properties" << endreq;
    return StatusCode::FAILURE;
  }
  
  if(MissingEtName != "None")
  {  
    MsgLog << MSG::INFO << "Missing Et Name      = " << MissingEtName                    << endreq;
    MsgLog << MSG::INFO << "Missing Et Threshold = " << MissingEtThreshold/GeV << " GeV" << endreq;
  }
  
  for(unsigned int i=0; i<ContainerName.size(); i++)
  {
    MsgLog << MSG::INFO << "Single Container Name = " << ContainerName[i]             << endreq;
    MsgLog << MSG::INFO << "Single Pt Threshold   = " << PtThreshold[i]/GeV << " GeV" << endreq;
    MsgLog << MSG::INFO << "Single Eta Threshold  = " << EtaThreshold[i]              << endreq;
    MsgLog << MSG::INFO << "Min Single Number     = " << MinNumber[i]                 << endreq;
    MsgLog << MSG::INFO << "Max Single Number     = " << MaxNumber[i]                 << endreq;
  }
  
  for(unsigned int i=0; i<FirstContainerName.size(); i++)
  {
    MsgLog << MSG::INFO << "First Container Name  = " << FirstContainerName[i]              << endreq;
    MsgLog << MSG::INFO << "Second Container Name = " << SecondContainerName[i]             << endreq;
    MsgLog << MSG::INFO << "First Pt Threshold    = " << FirstPtThreshold[i]/GeV  << " GeV" << endreq;
    MsgLog << MSG::INFO << "Second Pt hreshold    = " << SecondPtThreshold[i]/GeV << " GeV" << endreq;
    MsgLog << MSG::INFO << "First Eta Threshold   = " << FirstEtaThreshold[i]/GeV           << endreq;
    MsgLog << MSG::INFO << "Second Eta hreshold   = " << SecondEtaThreshold[i]/GeV          << endreq;
    MsgLog << MSG::INFO << "Min Sum Number        = " << MinSumNumber[i]                    << endreq;
    MsgLog << MSG::INFO << "Max Sum Number        = " << MaxSumNumber[i]                    << endreq;
  }
  
  StatusCode status = service("StoreGateSvc", aStoreGateSvc);

  if(status.isFailure()) 
  {
    MsgLog << MSG::FATAL << "StoreGateSvc Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
                        
  InputEventNumber  = 0;
  OutputEventNumber = 0;
  
  return StatusCode::SUCCESS;
  
}

StatusCode EventFilter::finalize()
{

  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Finalizing " << name() << endreq;
  
  MsgLog << MSG::INFO << "Input  Event Number = " << InputEventNumber  << endreq;
  MsgLog << MSG::INFO << "Output Event Number = " << OutputEventNumber << endreq;
          
  return StatusCode::SUCCESS;
  
}

StatusCode EventFilter::execute()
{

  MsgStream MsgLog(msgSvc(), name());
  
  StatusCode status;
  
  InputEventNumber++;

  if(!(InputEventNumber%1000)) 
  {
    MsgLog << MSG::INFO << "Current Event Number = " << InputEventNumber << endreq;
  }
  
  if(MissingEtName != "None")
  {
  
    status = aStoreGateSvc->retrieve(aMissingEt, MissingEtName);
  
    if(status.isFailure())
    {
      MsgLog << MSG::FATAL << "Missing Et Unavailable" << endreq;
      return StatusCode::FAILURE;
    }
    
  }
  
  bool isMissingEt = true;
  if(MissingEtName != "None") isMissingEt = aMissingEt->et() > MissingEtThreshold;
  
  if(!isMissingEt)
  {
    setFilterPassed(false);
    return StatusCode::SUCCESS;
  }
              
  for(unsigned int i=0; i<ContainerName.size(); i++)
  {
  
    int Size = 0;
    
    if     (ContainerType[i] == "Photon")   Size = this->ContainerSize(aPhotonContainer,   ContainerName[i], 
                                                                       PtThreshold[i],     EtaThreshold[i]);
    else if(ContainerType[i] == "Electron") Size = this->ContainerSize(aElectronContainer, ContainerName[i], 
                                                                       PtThreshold[i],     EtaThreshold[i]);
    else if(ContainerType[i] == "Muon")     Size = this->ContainerSize(aMuonContainer,     ContainerName[i], 
                                                                       PtThreshold[i],     EtaThreshold[i]);
    else if(ContainerType[i] == "TauJet")   Size = this->ContainerSize(aTauJetContainer,   ContainerName[i], 
                                                                       PtThreshold[i],     EtaThreshold[i]);
    else if(ContainerType[i] == "Jet")      Size = this->ContainerSize(aJetContainer,      ContainerName[i], 
                                                                       PtThreshold[i],     EtaThreshold[i]);
    
    if(Size == -1)
    {
      MsgLog << MSG::FATAL << ContainerName[i] << " Unavailable" << endreq;
      return StatusCode::FAILURE;
    }
    
    bool isSize = (Size >= MinNumber[i]) && (Size <= MaxNumber[i]);
    
    if(!isSize)
    {
      setFilterPassed(false);
      return StatusCode::SUCCESS;
    }
    
  }
  
  for(unsigned int i=0; i<FirstContainerName.size(); i++)
  {
  
    int FirstSize = 0;
    
    if     (FirstContainerType[i] == "Photon")   FirstSize = this->ContainerSize(aPhotonContainer,    FirstContainerName[i], 
                                                                                 FirstPtThreshold[i], FirstEtaThreshold[i]);
    else if(FirstContainerType[i] == "Electron") FirstSize = this->ContainerSize(aElectronContainer,  FirstContainerName[i], 
                                                                                 FirstPtThreshold[i], FirstEtaThreshold[i]);
    else if(FirstContainerType[i] == "Muon")     FirstSize = this->ContainerSize(aMuonContainer,      FirstContainerName[i], 
                                                                                 FirstPtThreshold[i], FirstEtaThreshold[i]);
    else if(FirstContainerType[i] == "TauJet")   FirstSize = this->ContainerSize(aTauJetContainer,    FirstContainerName[i], 
                                                                                 FirstPtThreshold[i], FirstEtaThreshold[i]);
    else if(FirstContainerType[i] == "Jet")      FirstSize = this->ContainerSize(aJetContainer,       FirstContainerName[i], 
                                                                                 FirstPtThreshold[i], FirstEtaThreshold[i]);
    
    if(FirstSize == -1)
    {
      MsgLog << MSG::FATAL << FirstContainerName[i] << " Unavailable" << endreq;
      return StatusCode::FAILURE;
    }
    
    int SecondSize = 0;
    
    if     (SecondContainerType[i] == "Photon")   SecondSize = this->ContainerSize(aPhotonContainer,     SecondContainerName[i], 
                                                                                   SecondPtThreshold[i], SecondEtaThreshold[i]);
    else if(SecondContainerType[i] == "Electron") SecondSize = this->ContainerSize(aElectronContainer,   SecondContainerName[i], 
                                                                                   SecondPtThreshold[i], SecondEtaThreshold[i]);
    else if(SecondContainerType[i] == "Muon")     SecondSize = this->ContainerSize(aMuonContainer,       SecondContainerName[i], 
                                                                                   SecondPtThreshold[i], SecondEtaThreshold[i]);
    else if(SecondContainerType[i] == "TauJet")   SecondSize = this->ContainerSize(aTauJetContainer,     SecondContainerName[i], 
                                                                                   SecondPtThreshold[i], SecondEtaThreshold[i]);
    else if(SecondContainerType[i] == "Jet")      SecondSize = this->ContainerSize(aJetContainer,        SecondContainerName[i], 
                                                                                   SecondPtThreshold[i], SecondEtaThreshold[i]);
    
    if(SecondSize == -1)
    {
      MsgLog << MSG::FATAL << SecondContainerName[i] << " Unavailable" << endreq;
      return StatusCode::FAILURE;
    }
    
    int SumSize = FirstSize+SecondSize;
    
    bool isSumSize = (SumSize >= MinSumNumber[i]) && (SumSize <= MaxSumNumber[i]);
    
    if(!isSumSize)
    {
      setFilterPassed(false);
      return StatusCode::SUCCESS;
    }
    
  }
                
  OutputEventNumber++;
  
  setFilterPassed(true);
    
  return StatusCode::SUCCESS;
  
}

bool EventFilter::TestContainerType()
{

  for(unsigned int i=0; i<ContainerName.size(); i++)
  {
    
    std::string Name = ContainerName[i];
  
    if     (Name.find("Photon",   0) != std::string::npos) ContainerType.push_back("Photon");
    else if(Name.find("Electron", 0) != std::string::npos) ContainerType.push_back("Electron");
    else if(Name.find("Muon",     0) != std::string::npos) ContainerType.push_back("Muon");
    else if(Name.find("Tau",      0) != std::string::npos) ContainerType.push_back("TauJet");
    else if(Name.find("Jet",      0) != std::string::npos) ContainerType.push_back("Jet");
    else return false;
    
  }
        
  for(unsigned int i=0; i<FirstContainerName.size(); i++)
  {
    
    std::string Name = FirstContainerName[i];
  
    if     (Name.find("Photon",   0) != std::string::npos) FirstContainerType.push_back("Photon");
    else if(Name.find("Electron", 0) != std::string::npos) FirstContainerType.push_back("Electron");
    else if(Name.find("Muon",     0) != std::string::npos) FirstContainerType.push_back("Muon");
    else if(Name.find("Tau",      0) != std::string::npos) FirstContainerType.push_back("TauJet");
    else if(Name.find("Jet",      0) != std::string::npos) FirstContainerType.push_back("Jet");
    else return false;
    
  }
  
  for(unsigned int i=0; i<SecondContainerName.size(); i++)
  {
     
    std::string Name = SecondContainerName[i];
  
    if     (Name.find("Photon",   0) != std::string::npos) SecondContainerType.push_back("Photon");
    else if(Name.find("Electron", 0) != std::string::npos) SecondContainerType.push_back("Electron");
    else if(Name.find("Muon",     0) != std::string::npos) SecondContainerType.push_back("Muon");
    else if(Name.find("Tau",      0) != std::string::npos) SecondContainerType.push_back("TauJet");
    else if(Name.find("Jet",      0) != std::string::npos) SecondContainerType.push_back("Jet");
    else return false;
  
  }
  
  return true;

}

template <class CT> int EventFilter::ContainerSize(const CT * aContainer, std::string Name, double MinPt, double MinEta)
{
    
  StatusCode status = aStoreGateSvc->retrieve(aContainer, Name);
  if(status.isFailure()) return -1;
  
  int Size = 0;
  
  typename CT::const_iterator Itr;
            
  for(Itr=aContainer->begin(); Itr!=aContainer->end(); Itr++)
  {
      
    bool isPt  = (*Itr)->pt() > MinPt;
    bool isEta = fabs((*Itr)->eta()) >= MinEta;
    
    if(isPt && isEta) Size++;
    
  }
 
  return Size;
  
}
