#include "SingleTopDPDMaker/L1TriggerSelection.h"

L1TriggerSelection::L1TriggerSelection(const std::string& name, ISvcLocator* pSvcLocator) : Algorithm(name, pSvcLocator)
{

  declareProperty("Level1Name",          Level1Name          = "LVL1_ROI");
  declareProperty("OutputContainerName", OutputContainerName             );
          
}

L1TriggerSelection::~L1TriggerSelection(){}

StatusCode L1TriggerSelection::initialize()
{
  
  MsgStream MsgLog(messageService(), name());
  
  MsgLog << MSG::INFO << "Initializing " << name() << endreq;
      
  MsgLog << MSG::INFO << "Level1 Name = " << Level1Name << endreq;
  
  bool isTestContainerSize = OutputContainerName.size() > 0;
  
  if(!isTestContainerSize)
  {
    MsgLog << MSG::FATAL << "Wrong Container Size Properties" << endreq;
    return StatusCode::FAILURE;
  }
  
  bool isTestContainerType = this->TestContainerType();
      
  if(!isTestContainerType)
  {
    MsgLog << MSG::FATAL << "Wrong Container Type Properties" << endreq;
    return StatusCode::FAILURE;
  }
  
  for(unsigned int i=0; i<OutputContainerName.size(); i++)
  {
    MsgLog << MSG::INFO << "Output Container Name = " << OutputContainerName[i] << endreq;
  }
            
  StatusCode status = service("StoreGateSvc", aStoreGateSvc);

  if(status.isFailure()) 
  {
    MsgLog << MSG::FATAL << "StoreGateSvc Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  InputEventNumber  = 0;
  OutputEventNumber = 0;
        
  return StatusCode::SUCCESS;
  
}

StatusCode L1TriggerSelection::finalize()
{

  MsgStream MsgLog(messageService(), name());
  
  MsgLog << MSG::INFO << "Finalizing " << name() << endreq;
  
  MsgLog << MSG::INFO << "Input  Event Number = " << InputEventNumber  << endreq;
  MsgLog << MSG::INFO << "Output Event Number = " << OutputEventNumber << endreq;
  
  return StatusCode::SUCCESS;
  
}

bool L1TriggerSelection::TestContainerType()
{
  
  for(unsigned int i=0; i<OutputContainerName.size(); i++)
  {
    
    std::string Name = OutputContainerName[i];
    
    if(Name.find("L1", 0) == std::string::npos) return false;
    
    if(Name.find("Muon", 0) != std::string::npos) 
    {
      isMuon = true;
      MuonContainerName = Name;
    }
    
    else if(Name.find("Tau", 0) != std::string::npos) 
    {
      isEmTau = true;
      EmTauContainerName = Name;
    }
    
    else if(Name.find("Jet", 0) != std::string::npos) 
    {
      isJet = true;
      JetContainerName = Name;
    }
        
    else return false;
        
  }
    
  return true;

}

StatusCode L1TriggerSelection::execute()
{

  MsgStream MsgLog(msgSvc(), name());
  
  InputEventNumber++;
  
  StatusCode status;
  
  status = aStoreGateSvc->retrieve(aLevel1, Level1Name);
  
  if(status.isFailure())
  {
    if(InputEventNumber == 1) MsgLog << MSG::INFO << "No Level1" << endreq;
    return StatusCode::SUCCESS;
  }
        
  if(isMuon)
  {
    status = this->executeMuon();
    if(status.isFailure()) return StatusCode::FAILURE;
  }
  
  if(isEmTau)
  {
    status = this->executeEmTau();
    if(status.isFailure()) return StatusCode::FAILURE;
  }
  
  if(isJet)
  {
    status = this->executeJet();
    if(status.isFailure()) return StatusCode::FAILURE;
  }
  
  setFilterPassed(true);
  
  OutputEventNumber++;
  
  return StatusCode::SUCCESS;
  
}

StatusCode L1TriggerSelection::executeMuon()
{

  MsgStream MsgLog(msgSvc(), name());
  
  StatusCode status;

  aMuonContainer = new INavigable4MomentumCollection(SG::VIEW_ELEMENTS);
  
  status = aStoreGateSvc->record(aMuonContainer, MuonContainerName);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << " Muon Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  this->SelectMuons();
  
  status = aStoreGateSvc->setConst(aMuonContainer);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << " Muon Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  return StatusCode::SUCCESS;
  
}

StatusCode L1TriggerSelection::executeEmTau()
{

  MsgStream MsgLog(msgSvc(), name());
  
  StatusCode status;

  aEmTauContainer = new INavigable4MomentumCollection(SG::VIEW_ELEMENTS);
  
  status = aStoreGateSvc->record(aEmTauContainer, EmTauContainerName);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << " EmTau Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  this->SelectEmTaus();
  
  status = aStoreGateSvc->setConst(aEmTauContainer);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << " EmTau Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  return StatusCode::SUCCESS;
  
}

StatusCode L1TriggerSelection::executeJet()
{

  MsgStream MsgLog(msgSvc(), name());
  
  StatusCode status;

  aJetContainer = new INavigable4MomentumCollection(SG::VIEW_ELEMENTS);
  
  status = aStoreGateSvc->record(aJetContainer, JetContainerName);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << " Jet Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  this->SelectJets();
  
  status = aStoreGateSvc->setConst(aJetContainer);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << " Jet Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  return StatusCode::SUCCESS;
  
}

void L1TriggerSelection::SelectMuons()
{

  LVL1_ROI::muons_type::const_iterator Itr;
  
  for(Itr=(aLevel1->getMuonROIs()).begin(); Itr!=(aLevel1->getMuonROIs()).end(); Itr++)
  {
  
    Muon_ROI * aMuon = const_cast<Muon_ROI*>(&(*Itr));
    aMuonContainer->push_back(aMuon);
    
  } 

}

void L1TriggerSelection::SelectEmTaus()
{

  LVL1_ROI::emtaus_type::const_iterator Itr;
  
  for(Itr=(aLevel1->getEmTauROIs()).begin(); Itr!=(aLevel1->getEmTauROIs()).end(); Itr++)
  {
  
    EmTau_ROI * aEmTau = const_cast<EmTau_ROI*>(&(*Itr));
    aEmTauContainer->push_back(aEmTau);
    
  }
  
}

void L1TriggerSelection::SelectJets()
{

  LVL1_ROI::jets_type::const_iterator Itr;
  
  for(Itr=(aLevel1->getJetROIs()).begin(); Itr!=(aLevel1->getJetROIs()).end(); Itr++)
  {
  
    Jet_ROI * aJet = const_cast<Jet_ROI*>(&(*Itr));
    aJetContainer->push_back(aJet);
    
  }
  
}
