#include "SingleTopDPDMaker/L2TriggerSelection.h"

L2TriggerSelection::L2TriggerSelection(const std::string& name, ISvcLocator* pSvcLocator) : Algorithm(name, pSvcLocator)
{

  declareProperty("OutputContainerName", OutputContainerName);
            
}

L2TriggerSelection::~L2TriggerSelection(){}

StatusCode L2TriggerSelection::initialize()
{
  
  MsgStream MsgLog(messageService(), name());
  
  MsgLog << MSG::INFO << "Initializing " << name() << endreq;
        
  bool isTestContainerSize = OutputContainerName.size() > 0;
  
  if(!isTestContainerSize)
  {
    MsgLog << MSG::FATAL << "Wrong Container Size Properties" << endreq;
    return StatusCode::FAILURE;
  }
  
  bool isTestContainerType = this->TestContainerType();
      
  if(!isTestContainerType)
  {
    MsgLog << MSG::FATAL << "Wrong Container Type Properties" << endreq;
    return StatusCode::FAILURE;
  }
  
  for(unsigned int i=0; i<OutputContainerName.size(); i++)
  {
    MsgLog << MSG::INFO << "Output Container Name = " << OutputContainerName[i] << endreq;
  }
            
  StatusCode status = service("StoreGateSvc", aStoreGateSvc);

  if(status.isFailure()) 
  {
    MsgLog << MSG::FATAL << "StoreGateSvc Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  InputEventNumber  = 0;
  OutputEventNumber = 0;
        
  return StatusCode::SUCCESS;
  
}

StatusCode L2TriggerSelection::finalize()
{

  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Finalizing " << name() << endreq;
  
  MsgLog << MSG::INFO << "Input  Event Number = " << InputEventNumber  << endreq;
  MsgLog << MSG::INFO << "Output Event Number = " << OutputEventNumber << endreq;
  
  return StatusCode::SUCCESS;
  
}

bool L2TriggerSelection::TestContainerType()
{
  
  for(unsigned int i=0; i<OutputContainerName.size(); i++)
  {
    
    std::string Name = OutputContainerName[i];
    
    if(Name.find("L2", 0) == std::string::npos) return false;
    
    if(Name.find("Electron", 0) != std::string::npos) 
    {
      isElectron = true;
      ElectronContainerName = Name;
    }
    
    else if(Name.find("Photon", 0) != std::string::npos) 
    {
      isPhoton = true;
      PhotonContainerName = Name;
    }
    
    else if(Name.find("Muon", 0) != std::string::npos) 
    {
      isMuon = true;
      MuonContainerName = Name;
    }
    
    else if(Name.find("Tau", 0) != std::string::npos) 
    {
      isTau = true;
      TauContainerName = Name;
    }
    
    else if(Name.find("BJet", 0) != std::string::npos) 
    {
      isBJet = true;
      BJetContainerName = Name;
    }
    else if(Name.find("Jet", 0) != std::string::npos) 
    {
      isJet = true;
      JetContainerName = Name;
    }
    
    else return false;
        
  }
    
  return true;

}

StatusCode L2TriggerSelection::execute()
{
  
  InputEventNumber++;
  
  StatusCode status;
  
  if(isElectron) 
  {
    status = this->executeElectron();
    if(status.isFailure()) return StatusCode::FAILURE;
  }
  
  if(isPhoton)
  {
    status = this->executePhoton();
    if(status.isFailure()) return StatusCode::FAILURE;
  }
  
  if(isMuon)
  {
    status = this->executeMuon();
    if(status.isFailure()) return StatusCode::FAILURE;
  }
  
  if(isTau)
  {
    status = this->executeTau();
    if(status.isFailure()) return StatusCode::FAILURE;
  }
  
  if(isJet)
  {
    status = this->executeJet();
    if(status.isFailure()) return StatusCode::FAILURE;
  }
  
  if(isBJet)
  {
    status = this->executeBJet();
    if(status.isFailure()) return StatusCode::FAILURE;
  }
  
  setFilterPassed(true);
  
  OutputEventNumber++;
  
  return StatusCode::SUCCESS;
  
}

StatusCode L2TriggerSelection::executeElectron()
{

  MsgStream MsgLog(msgSvc(), name());
  
  StatusCode status;
  
  status = aStoreGateSvc->retrieve(ElectronItrB, ElectronItrE);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "No Input Electron Container Available" << endreq;
    return StatusCode::FAILURE;
  }
    
  aElectronContainer = new INavigable4MomentumCollection(SG::VIEW_ELEMENTS);
    
  status = aStoreGateSvc->record(aElectronContainer, ElectronContainerName);
    
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Output Electron Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
    
  this->SelectElectrons();
    
  status = aStoreGateSvc->setConst(aElectronContainer);
    
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Output Electron Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  return StatusCode::SUCCESS;
    
}

StatusCode L2TriggerSelection::executePhoton()
{

  MsgStream MsgLog(msgSvc(), name());
  
  StatusCode status;
  
  status = aStoreGateSvc->retrieve(PhotonItrB, PhotonItrE);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "No Input Photon Container Available" << endreq;
    return StatusCode::FAILURE;
  }
    
  aPhotonContainer = new INavigable4MomentumCollection(SG::VIEW_ELEMENTS);
    
  status = aStoreGateSvc->record(aPhotonContainer, PhotonContainerName);
    
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Output Photon Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
    
  this->SelectPhotons();
    
  status = aStoreGateSvc->setConst(aPhotonContainer);
    
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Output Photon Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  return StatusCode::SUCCESS;
    
}

StatusCode L2TriggerSelection::executeMuon()
{

  MsgStream MsgLog(msgSvc(), name());
  
  StatusCode status;
  
  status = aStoreGateSvc->retrieve(MuonItrB, MuonItrE);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "No Input Muon Container Available" << endreq;
    return StatusCode::FAILURE;
  }
    
  aMuonContainer = new INavigable4MomentumCollection(SG::VIEW_ELEMENTS);
    
  status = aStoreGateSvc->record(aMuonContainer, MuonContainerName);
    
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Output Muon Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
    
  this->SelectMuons();
    
  status = aStoreGateSvc->setConst(aMuonContainer);
    
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Output Muon Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  return StatusCode::SUCCESS;
    
}

StatusCode L2TriggerSelection::executeTau()
{

  MsgStream MsgLog(msgSvc(), name());
  
  StatusCode status;
  
  status = aStoreGateSvc->retrieve(TauItrB, TauItrE);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "No Input Tau Container Available" << endreq;
    return StatusCode::FAILURE;
  }
    
  aTauContainer = new INavigable4MomentumCollection(SG::VIEW_ELEMENTS);
    
  status = aStoreGateSvc->record(aTauContainer, TauContainerName);
    
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Output Tau Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
    
  this->SelectTaus();
    
  status = aStoreGateSvc->setConst(aTauContainer);
    
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Output Tau Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  return StatusCode::SUCCESS;
    
}

StatusCode L2TriggerSelection::executeJet()
{

  MsgStream MsgLog(msgSvc(), name());
  
  StatusCode status;
  
  status = aStoreGateSvc->retrieve(JetItrB, JetItrE);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "No Input Jet Container Available" << endreq;
    return StatusCode::FAILURE;
  }
    
  aJetContainer = new INavigable4MomentumCollection(SG::VIEW_ELEMENTS);
    
  status = aStoreGateSvc->record(aJetContainer, JetContainerName);
    
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Output Jet Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
    
  this->SelectJets();
    
  status = aStoreGateSvc->setConst(aJetContainer);
    
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Output Jet Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  return StatusCode::SUCCESS;
    
}

StatusCode L2TriggerSelection::executeBJet()
{

  MsgStream MsgLog(msgSvc(), name());
  
  StatusCode status;
  
  status = aStoreGateSvc->retrieve(BJetItrB, BJetItrE);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "No Input BJet Container Available" << endreq;
    return StatusCode::FAILURE;
  }
    
  aBJetContainer = new INavigable4MomentumCollection(SG::VIEW_ELEMENTS);
    
  status = aStoreGateSvc->record(aBJetContainer, BJetContainerName);
    
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Output BJet Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
    
  this->SelectBJets();
    
  status = aStoreGateSvc->setConst(aBJetContainer);
    
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Output BJet Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  return StatusCode::SUCCESS;
    
}
  
void L2TriggerSelection::SelectElectrons()
{

  DataHandle<TrigElectronContainer> ContainerItr;
    
  for(ContainerItr=ElectronItrB; ContainerItr!=ElectronItrE; ContainerItr++)
  {
  
    TrigElectronContainer::const_iterator Itr;
            
    for(Itr=(ContainerItr.cptr())->begin(); Itr!=(ContainerItr.cptr())->end(); Itr++)
    {
      TrigElectron * aElectron = const_cast<TrigElectron*>(*Itr);
      aElectronContainer->push_back(aElectron);
    }
      
  }
      
}

void L2TriggerSelection::SelectPhotons()
{

  DataHandle<TrigPhotonContainer> ContainerItr;
    
  for(ContainerItr=PhotonItrB; ContainerItr!=PhotonItrE; ContainerItr++)
  {
      
    TrigPhotonContainer::const_iterator Itr;
            
    for(Itr=(ContainerItr.cptr())->begin(); Itr!=(ContainerItr.cptr())->end(); Itr++)
    {
      TrigPhoton * aPhoton = const_cast<TrigPhoton*>(*Itr);
      aPhotonContainer->push_back(aPhoton);
    }
      
  }
        
}

void L2TriggerSelection::SelectMuons()
{

  DataHandle<CombinedMuonFeature> Itr;
    
  for(Itr=MuonItrB; Itr!=MuonItrE; Itr++)
  {
    CombinedMuonFeature * aMuon = const_cast<CombinedMuonFeature*>(Itr.cptr());
    aMuonContainer->push_back(aMuon);
  }
      
}

void L2TriggerSelection::SelectTaus()
{

  DataHandle<TrigTau> Itr;
    
  for(Itr=TauItrB; Itr!=TauItrE; Itr++)
  {
    TrigTau * aTau = const_cast<TrigTau*>(Itr.cptr());
    aTauContainer->push_back(aTau);    
  }
      
}

void L2TriggerSelection::SelectJets()
{

  DataHandle<TrigT2Jet> Itr;
    
  for(Itr=JetItrB; Itr!=JetItrE; Itr++)
  {
    TrigT2Jet * aJet = const_cast<TrigT2Jet*>(Itr.cptr());
    aJetContainer->push_back(aJet);
  }
      
}

void L2TriggerSelection::SelectBJets()
{

  DataHandle<TrigL2BjetContainer> ContainerItr;
    
  for(ContainerItr=BJetItrB; ContainerItr!=BJetItrE; ContainerItr++)
  {

    TrigL2BjetContainer::const_iterator Itr;
            
    for(Itr=(ContainerItr.cptr())->begin(); Itr!=(ContainerItr.cptr())->end(); Itr++)
    {
      TrigL2Bjet * aBJet = const_cast<TrigL2Bjet*>(*Itr);
      aBJetContainer->push_back(aBJet);
    }
    
  }
    
}
