#include "SingleTopDPDMaker/MuonSelection.h"

MuonSelection::MuonSelection(const std::string& name, ISvcLocator* pSvcLocator) : Algorithm(name, pSvcLocator)
{

  declareProperty("InputContainerName",  InputContainerName  = "StacoMuonCollection"  );
  declareProperty("OutputContainerName", OutputContainerName = "SelectedMuonContainer");
  declareProperty("JetContainerName",    JetContainerName    = "Cone4H1TowerJets"     );
  
  declareProperty("SelectionInfoContainerName", SelectionInfoContainerName = "None"         );
  declareProperty("SelectionLabel",             SelectionLabel             = "MuonSelection");
  
  declareProperty("ApplyIsHighPtCut",  ApplyIsHighPtCut  = false   );
  declareProperty("ApplyBestMatchCut", ApplyBestMatchCut = false   );
  declareProperty("ApplyEtConeCut",    ApplyEtConeCut    = false   );
  declareProperty("ApplyJetDRCut",     ApplyJetDRCut     = false   );
  
  declareProperty("Author",            Author            = "All"   );
  declareProperty("PtCut",             PtCut             = 20.0*GeV);
  declareProperty("EtaCut",            EtaCut            = 2.5     );
  declareProperty("ConeSize",          ConeSize          = 0.2     );
  declareProperty("EtConeCut",         EtConeCut         = 6.0*GeV );
  declareProperty("JetDRCut",          JetDRCut          = 0.4     );
  
}

MuonSelection::~MuonSelection(){}

StatusCode MuonSelection::initialize()
{
  
  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Initializing " << name() << endreq;
  
  MsgLog << MSG::INFO << "Input  Container Name = " << InputContainerName  << endreq;
  MsgLog << MSG::INFO << "Output Container Name = " << OutputContainerName << endreq;
  
  if(SelectionInfoContainerName != "None")
  {
    MsgLog << MSG::INFO << "Selection Info Container Name = " << SelectionInfoContainerName << endreq;
    MsgLog << MSG::INFO << "Selection Label               = " << SelectionLabel             << endreq;
  }
  
  MsgLog << MSG::INFO << "Apply Is High Pt Cut = " << ApplyIsHighPtCut  << endreq;
  MsgLog << MSG::INFO << "Apply Best Match Cut = " << ApplyBestMatchCut << endreq;
  MsgLog << MSG::INFO << "Apply Et Cone Cut    = " << ApplyEtConeCut    << endreq;
  MsgLog << MSG::INFO << "Apply Jet DR Cut     = " << ApplyJetDRCut     << endreq;
      
  MsgLog << MSG::INFO << "Author      = " << Author              << endreq;
  MsgLog << MSG::INFO << "Pt Cut      = " << PtCut/GeV << " GeV" << endreq;
  MsgLog << MSG::INFO << "Eta Cut     = " << EtaCut              << endreq;
  
  if(ApplyEtConeCut)
  {
    MsgLog << MSG::INFO << "Cone Size   = " << ConeSize                << endreq;
    MsgLog << MSG::INFO << "Et Cone Cut = " << EtConeCut/GeV << " GeV" << endreq;
  }
  
  if(ApplyJetDRCut)
  {
    MsgLog << MSG::INFO << "Jet Container Name = " << JetContainerName << endreq;
    MsgLog << MSG::INFO << "Jet DR Cut         = " << JetDRCut         << endreq;
  }
      
  bool isTestAuthor   = this->TestAuthorOption  ();
  bool isTestConeSize = this->TestConeSizeOption();
  
  if(!isTestAuthor)
  {
    MsgLog << MSG::FATAL << "Wrong Author Property" << endreq;
    return StatusCode::FAILURE;
  }
  
  if(!isTestConeSize)
  {
    MsgLog << MSG::FATAL << "Wrong Cone Size Property" << endreq;
    return StatusCode::FAILURE;
  }
  
  StatusCode status = service("StoreGateSvc", aStoreGateSvc);

  if(status.isFailure()) 
  {
    MsgLog << MSG::FATAL << "StoreGateSvc Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  InputEventNumber  = 0;
  OutputEventNumber = 0;
  
  InputObjectNumber  = 0;
  OutputObjectNumber = 0;
      
  return StatusCode::SUCCESS;
  
}

StatusCode MuonSelection::finalize()
{
  
  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Finalizing " << name() << endreq;
  
  MsgLog << MSG::INFO << "Input  Event Number  = " << InputEventNumber  << endreq;
  MsgLog << MSG::INFO << "Output Event Number  = " << OutputEventNumber << endreq;
  
  MsgLog << MSG::INFO << "Input  Object Number = " << InputObjectNumber  << endreq;
  MsgLog << MSG::INFO << "Output Object Number = " << OutputObjectNumber << endreq;
  
  return StatusCode::SUCCESS;
  
}

StatusCode MuonSelection::execute()
{
  
  MsgStream MsgLog(msgSvc(), name());
  
  StatusCode status;
  
  InputEventNumber++;
      
  status = aStoreGateSvc->retrieve(aInputContainer, InputContainerName);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Input Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  if(ApplyJetDRCut)
  {
  
    status = aStoreGateSvc->retrieve(aJetContainer, JetContainerName);
  
    if(status.isFailure())
    {
      MsgLog << MSG::FATAL << "Jet Container Unavailable" << endreq;
      return StatusCode::FAILURE;
    }
    
  }
  
  aOutputContainer = new Analysis::MuonContainer(SG::VIEW_ELEMENTS);
  
  status = aStoreGateSvc->record(aOutputContainer, OutputContainerName);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Output Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  status = aStoreGateSvc->retrieve(aSelectionInfoContainer, SelectionInfoContainerName);
  
  if(status.isFailure())
  {
    
    if(SelectionInfoContainerName != "None")
    {
    
      if(InputEventNumber == 1) MsgLog << MSG::INFO << "Creating Selection Info Container" << endreq;
    
      aSelectionInfoContainer = new SelectionInfoContainer(SG::VIEW_ELEMENTS);
    
      status = aStoreGateSvc->record(aSelectionInfoContainer, SelectionInfoContainerName);
    
      if(status.isFailure())
      {
        MsgLog << MSG::FATAL << "Selection Info Container Unavailable" << endreq;
        return StatusCode::FAILURE;
      }
    
    }
    
    else 
    {
      aSelectionInfoContainer = 0;
      if(InputEventNumber == 1) MsgLog << MSG::INFO << "No Selection Info Container" << endreq;
    }
        
  }
      
  Mask.clear();
  
  this->Select();
  
  if(aSelectionInfoContainer)
  {  
    SelectionInfo * aSelectionInfo = new SelectionInfo();
  
    aSelectionInfo->SetName(InputContainerName);
    aSelectionInfo->SetLabel(SelectionLabel);
    aSelectionInfo->SetMask(Mask);
    
    aSelectionInfoContainer->push_back(aSelectionInfo);
  }
  
  status = aStoreGateSvc->setConst(aOutputContainer);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Output Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  InputObjectNumber  += static_cast<int>(aInputContainer->size());
  OutputObjectNumber += static_cast<int>(aOutputContainer->size());
  
  setFilterPassed(true);
  
  OutputEventNumber++;
    
  return StatusCode::SUCCESS;
  
}

void MuonSelection::Select()
{
  
  Analysis::MuonContainer::const_iterator Itr;
    
  for(Itr=aInputContainer->begin(); Itr!= aInputContainer->end(); Itr++)
  {
    
    const Analysis::Muon * aMuon = *Itr;
        
    bool isSelected = this->SelectMuon(aMuon);
        
    if(isSelected)
    { 
      Analysis::Muon * aSelectedMuon = new Analysis::Muon(*aMuon);
      aOutputContainer->push_back(aSelectedMuon);
      
      Mask.push_back(true);
    }
    
    else 
    {
      Mask.push_back(false);
    }
            
  }
  
}

bool MuonSelection::SelectMuon(const Analysis::Muon * aMuon)
{

  bool isPtCut       = aMuon->pt() > PtCut;
  bool isEtaCut      = fabs(aMuon->eta()) < EtaCut;
    
  bool isAuthor      =  this->TestAuthor(aMuon);
    
  bool isHighPt      = true;
  bool isBestMatch   = true;
  bool isIsolated    = true;
  bool isJetIsolated = true;
    
  if(ApplyIsHighPtCut)  isHighPt      = aMuon->isHighPt();
  if(ApplyBestMatchCut) isBestMatch   = aMuon->bestMatch();
  if(ApplyEtConeCut)    isIsolated    = aMuon->parameter(ConeSizeParameter) < EtConeCut;
  if(ApplyJetDRCut)     isJetIsolated = this->TestJetDeltaR(aMuon);
    
  bool isSelected = isPtCut && isEtaCut && isAuthor && isHighPt && isBestMatch && isIsolated && isJetIsolated;
  
  return isSelected;

}

bool MuonSelection::TestAuthorOption()
{

  bool Test = false;
  
  if     (Author == "All")        Test = true;
  else if(Author == "Combined")   Test = true;
  else if(Author == "Standalone") Test = true;
  else if(Author == "LowPt")      Test = true;
  else if(Author == "Calo")       Test = true;
  
  return Test;

}

bool MuonSelection::TestAuthor(const Analysis::Muon * aMuon)
{

  int author = aMuon->author();

  bool Test = false;
  
  if     (Author == "All")        Test = true;
  else if(Author == "Combined")   Test = (author == MuonParameters::STACO)   || (author == MuonParameters::MuidCo);
  else if(Author == "Standalone") Test = (author == MuonParameters::Muonboy) || (author == MuonParameters::MuidSA);
  else if(Author == "LowPt")      Test = (author == MuonParameters::MuTag)   || (author == MuonParameters::MuGirl);
  else if(Author == "Calo")       Test = (author == MuonParameters::CaloTag) || (author == MuonParameters::CaloLikelihood);;
    
  return Test;  

}

bool MuonSelection::TestConeSizeOption()
{

  bool Test = false;
  
  if(ConeSize == 0.2) 
  {
    Test = true;
    ConeSizeParameter = MuonParameters::etcone20;
  }
  
  else if(ConeSize == 0.4) 
  {
    Test = true;
    ConeSizeParameter = MuonParameters::etcone40;
  }
  
  return Test; 

}

bool MuonSelection::TestJetDeltaR(const Analysis::Muon * aMuon)
{

  bool Test = true;
  
  JetCollection::const_iterator Itr;
  
  for(Itr=aJetContainer->begin(); Itr!=aJetContainer->end(); Itr++)
  {
  
    double dR = P4Help::deltaR(aMuon, *Itr);
    Test = dR > JetDRCut;
    if(!Test) break;
  
  }
      
  return Test;

}
