#include "SingleTopDPDMaker/NtupleMaker.h"

NtupleMaker::NtupleMaker(const std::string& name, ISvcLocator* pSvcLocator) : Algorithm(name, pSvcLocator)
{

  declareProperty("FileName",                 FileName                 = "SingleTopD4PD.root"          );
  declareProperty("NtupleName",               NtupleName               = "SingleTop"                   );
  declareProperty("EventInfoName",            EventInfoName            = "ByteStreamEventInfo"         );
  declareProperty("MissingEtName",            MissingEtName            = "MET_RefFinal"                );
  declareProperty("ElectronContainerName",    ElectronContainerName    = "SelectedElectronContainer"   );
  declareProperty("MuonContainerName",        MuonContainerName        = "SelectedMuonContainer"       );
  declareProperty("TaggedJetContainerName",   TaggedJetContainerName   = "SelectedTaggedJetContainer"  );
  declareProperty("UntaggedJetContainerName", UntaggedJetContainerName = "SelectedUntaggedJetContainer");
        
}

NtupleMaker::~NtupleMaker(){}

StatusCode NtupleMaker::initialize()
{

  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Initializing " << name() << endreq;
  
  MsgLog << MSG::INFO << "File Name                   = " << FileName                 << endreq;
  MsgLog << MSG::INFO << "Ntuple Name                 = " << NtupleName               << endreq;
  MsgLog << MSG::INFO << "Event Info Name             = " << EventInfoName            << endreq;
  MsgLog << MSG::INFO << "Missing Et Name             = " << MissingEtName            << endreq;
  MsgLog << MSG::INFO << "Electron Container Name     = " << ElectronContainerName    << endreq;
  MsgLog << MSG::INFO << "Muon Container Name         = " << MuonContainerName        << endreq;
  MsgLog << MSG::INFO << "Tagged Jet Container Name   = " << TaggedJetContainerName   << endreq;
  MsgLog << MSG::INFO << "Untagged Jet Container Name = " << UntaggedJetContainerName << endreq;
        
  StatusCode status;
  
  status = service("StoreGateSvc", aStoreGateSvc);

  if(status.isFailure()) 
  {
    MsgLog << MSG::FATAL << "StoreGateSvc Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
    
  this->CreateNtuple(); 
      
  NbInputEvents  = 0;
  NbOutputEvents = 0;
  
  return StatusCode::SUCCESS;
  
}

StatusCode NtupleMaker::finalize()
{
  
  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Finalizing " << name() << endreq;
  
  MsgLog << MSG::INFO << "Input  Event Number = " << NbInputEvents  << endreq;
  MsgLog << MSG::INFO << "Output Event Number = " << NbOutputEvents << endreq;
  
  this->WriteNtuple(); 
        
  return StatusCode::SUCCESS;
  
}

StatusCode NtupleMaker::execute()
{

  MsgStream MsgLog(msgSvc(), name());
  
  StatusCode status;

  NbInputEvents++;
  
  status = aStoreGateSvc->retrieve(aEventInfo, EventInfoName);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Event Info Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  status = aStoreGateSvc->retrieve(aMissingEt, MissingEtName);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Missing Et Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
    
  status = aStoreGateSvc->retrieve(aElectronContainer, ElectronContainerName);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Electron Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
    
  status = aStoreGateSvc->retrieve(aMuonContainer, MuonContainerName);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Muon Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  status = aStoreGateSvc->retrieve(aTaggedJetContainer, TaggedJetContainerName);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Tagged Jet Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  status = aStoreGateSvc->retrieve(aUntaggedJetContainer, UntaggedJetContainerName);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Untagged Jet Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  aSortedLeptonContainer      = new IParticleContainer(SG::VIEW_ELEMENTS);
  aSortedJetContainer         = new IParticleContainer(SG::VIEW_ELEMENTS);
  aSortedTaggedJetContainer   = new IParticleContainer(SG::VIEW_ELEMENTS);
  aSortedUntaggedJetContainer = new IParticleContainer(SG::VIEW_ELEMENTS);
  
  this->SortContainers();
  this->ClearNtuple();
  this->FillNtuple();
  
  
  delete aSortedLeptonContainer;
  delete aSortedJetContainer;
  delete aSortedTaggedJetContainer;
  delete aSortedUntaggedJetContainer;
  
  NbOutputEvents++;
      
  return StatusCode::SUCCESS;
  
}

void NtupleMaker::SortContainers()
{

  ElectronContainer::const_iterator ElectronItr;

  for(ElectronItr=aElectronContainer->begin(); ElectronItr!=aElectronContainer->end(); ElectronItr++)
  {
    aSortedLeptonContainer->push_back(*ElectronItr); 
  }
 
  Analysis::MuonContainer::const_iterator MuonItr;
  
  for(MuonItr=aMuonContainer->begin(); MuonItr!=aMuonContainer->end(); MuonItr++)
  {
    aSortedLeptonContainer->push_back(*MuonItr); 
  }
  
  JetCollection::const_iterator JetItr;
  
  for(JetItr=aTaggedJetContainer->begin(); JetItr!=aTaggedJetContainer->end(); JetItr++)
  {
    aSortedTaggedJetContainer->push_back(*JetItr); 
    aSortedJetContainer->push_back(*JetItr);
  }
  
  for(JetItr=aUntaggedJetContainer->begin(); JetItr!=aUntaggedJetContainer->end(); JetItr++)
  {
    aSortedUntaggedJetContainer->push_back(*JetItr); 
    aSortedJetContainer->push_back(*JetItr);
  }
  
  AnalysisUtils::Sort::pT(aSortedLeptonContainer);
  AnalysisUtils::Sort::pT(aSortedJetContainer);
  AnalysisUtils::Sort::pT(aSortedTaggedJetContainer);
  AnalysisUtils::Sort::pT(aSortedUntaggedJetContainer);

}

void NtupleMaker::CreateNtuple()
{

  std::string Info     = "RunNumber:EventNumber:";
  std::string Lepton1  = "NbSelLep:LeptonType:LeptonPt:LeptonEta:LeptonPhi:LeptonCharge:";
  std::string Lepton2  = "Met:MetPhi:NuPz:MetLep:cosDPhi_LepMet:DEta_LepMet:DR_LepMet:MtWlep:MetLepTCut:";
  std::string Jets1    = "NbSelJet:Jphi1:Jphi2:Jphi3:Jphi4:Jeta1:Jeta2:Jeta3:Jeta4:Jpt1:Jpt2:Jpt3:Jpt4:";
  std::string Jets2    = "NbSelBJet:BJ1phi:BJ1eta:BJ1pt:BJ2phi:BJ2eta:BJ2pt:Mass_BJ1BJ2:";
  std::string Jets3    = "NbSelUJet:UJ1phi:UJ1eta:UJ1pt:UJ2phi:UJ2eta:UJ2pt:UJ3phi:UJ3eta:UJ3pt:";
  std::string Jets4    = "cosDPhi_BJ1BJ2:cosDPhi_UJ1UJ2:cosDPhi_BJ1UJ1:cosDPhi_BJ2UJ1:cosDPhi_BJ1UJ2:cosDPhi_BJ2UJ2:cosDPhi_BJ1UJ3:";
  std::string Jets5    = "DR_BJ1BJ2:DR_UJ1UJ2:DR_BJ1UJ1:DR_BJ2UJ1:DR_BJ1UJ2:DR_BJ2UJ2:DR_BJ1UJ3:";
  std::string Jets6    = "DEta_BJ1BJ2:DEta_UJ1UJ2:DEta_BJ1UJ1:DEta_BJ2UJ1:DEta_BJ1UJ2:DEta_BJ2UJ2:DEta_BJ1UJ3:";
  std::string LepJets1 = "cosDPhi_LepBJ1:cosDPhi_LepBJ2:cosDPhi_LepUJ1:cosDPhi_LepUJ2:cosDPhi_LepUJ3:";
  std::string LepJets2 = "DR_LepBJ1:DR_LepBJ2:DR_LepUJ1:DR_LepUJ2:DR_LepUJ3:";
  std::string LepJets3 = "DEta_LepBJ1:DEta_LepBJ2:DEta_LepUJ1:DEta_LepUJ2:DEta_LepUJ3:";
  std::string Wlep     = "Mass_Wlep:Eta_Wlep:Pt_Wlep:";
  std::string Whad     = "Mass_Whad12:Eta_Whad12:Pt_Whad12:Mass_Whad23:Eta_Whad23:Pt_Whad23:Mass_Whad13:Eta_Whad13:Pt_Whad13:";
  std::string TopLep   = "Mass_TlepBJ1:Eta_TlepBJ1:Pt_TlepBJ1:Mass_TlepBJ2:Eta_TlepBJ2:Pt_TlepBJ2:";
  std::string TopHad1  = "Mass_Thad12BJ1:Eta_Thad12BJ1:Pt_Thad12BJ1:Mass_Thad12BJ2:Eta_Thad12BJ2:Pt_Thad12BJ2:";
  std::string TopHad2  = "Mass_Thad23BJ1:Eta_Thad23BJ1:Pt_Thad23BJ1:Mass_Thad23BJ2:Eta_Thad23BJ2:Pt_Thad23BJ2:";
  std::string TopHad3  = "Mass_Thad13BJ1:Eta_Thad13BJ1:Pt_Thad13BJ1:Mass_Thad13BJ2:Eta_Thad13BJ2:Pt_Thad13BJ2:";
  std::string Global   = "HtBJets:HtJets:HtTot:Shat:Mass_Jets:Mass_tot:MetHtJets:";
  std::string Topo     = "Sphericity:Aplanarity:Centrality";
  
  std::string Event;
  
  Event  = Info;
  Event += Lepton1+Lepton2;
  Event += Jets1+Jets2+Jets3+Jets4+Jets5+Jets6;
  Event += LepJets1+LepJets2+LepJets3;
  Event += Wlep+Whad;
  Event += TopLep+TopHad1+TopHad2+TopHad3;
  Event += Global;
  Event += Topo;
  
  std::string NtupleTitle = NtupleName;
  aNtuple = new TNtupleD(NtupleName.c_str(), NtupleTitle.c_str(), Event.c_str());
  
}

void NtupleMaker::ClearNtuple()
{

  this->ClearLeptons();
  this->ClearJets();
  this->ClearLeptonsJets();
  this->ClearWLeptonic();
  this->ClearWHadronic();
  this->ClearTopLeptonic();
  this->ClearTopHadronic();
  this->ClearGlobal();
  this->ClearTopological();

}

void NtupleMaker::FillNtuple()
{

  aNeutrinoContainer = new NeutrinoContainer();
  
  this->FillInfo();
  this->FillLeptons();
  this->FillJets();
  this->FillLeptonsJets();
  this->FillWLeptonic();
  this->FillWHadronic();
  this->FillTopLeptonic();
  this->FillTopHadronic();
  this->FillGlobal();
  this->FillTopological();
  
  double Event[] =
  {
    RunNumber,EventNumber,
    NbSelLep,LeptonType,LeptonPt,LeptonEta,LeptonPhi,LeptonCharge,
    Met,MetPhi,NuPz,MetLep,cosDPhi_LepMet,DEta_LepMet,DR_LepMet,MtWlep,MetLepTCut,
    NbSelJet,Jphi[0],Jphi[1],Jphi[2],Jphi[3],Jeta[0],Jeta[1],Jeta[2],Jeta[3],Jpt[0],Jpt[1],Jpt[2],Jpt[3],
    NbSelBJet,BJ1phi,BJ1eta,BJ1pt,BJ2phi,BJ2eta,BJ2pt,Mass_BJ1BJ2,
    NbSelUJet,UJ1phi,UJ1eta,UJ1pt,UJ2phi,UJ2eta,UJ2pt,UJ3phi,UJ3eta,UJ3pt,
    cosDPhi_BJ1BJ2,cosDPhi_UJ1UJ2,cosDPhi_BJ1UJ1,cosDPhi_BJ2UJ1,cosDPhi_BJ1UJ2,cosDPhi_BJ2UJ2,cosDPhi_BJ1UJ3,
    DR_BJ1BJ2,DR_UJ1UJ2,DR_BJ1UJ1,DR_BJ2UJ1,DR_BJ1UJ2,DR_BJ2UJ2,DR_BJ1UJ3,
    DEta_BJ1BJ2,DEta_UJ1UJ2,DEta_BJ1UJ1,DEta_BJ2UJ1,DEta_BJ1UJ2,DEta_BJ2UJ2,DEta_BJ1UJ3,
    cosDPhi_LepBJ1,cosDPhi_LepBJ2,cosDPhi_LepUJ1,cosDPhi_LepUJ2,cosDPhi_LepUJ3,
    DR_LepBJ1,DR_LepBJ2,DR_LepUJ1,DR_LepUJ2,DR_LepUJ3,
    DEta_LepBJ1,DEta_LepBJ2,DEta_LepUJ1,DEta_LepUJ2,DEta_LepUJ3,
    Mass_Wlep,Eta_Wlep,Pt_Wlep,
    Mass_Whad12,Eta_Whad12,Pt_Whad12,Mass_Whad23,Eta_Whad23,Pt_Whad23,Mass_Whad13,Eta_Whad13,Pt_Whad13,
    Mass_TlepBJ1,Eta_TlepBJ1,Pt_TlepBJ1,Mass_TlepBJ2,Eta_TlepBJ2,Pt_TlepBJ2,
    Mass_Thad12BJ1,Eta_Thad12BJ1,Pt_Thad12BJ1,Mass_Thad12BJ2,Eta_Thad12BJ2,Pt_Thad12BJ2,
    Mass_Thad23BJ1,Eta_Thad23BJ1,Pt_Thad23BJ1,Mass_Thad23BJ2,Eta_Thad23BJ2,Pt_Thad23BJ2,
    Mass_Thad13BJ1,Eta_Thad13BJ1,Pt_Thad13BJ1,Mass_Thad13BJ2,Eta_Thad13BJ2,Pt_Thad13BJ2,
    HtBJets,HtJets,HtTot,Shat,Mass_Jets,Mass_tot,MetHtJets,
    Sphericity,Aplanarity,Centrality  
  };
    
  aNtuple->Fill(Event);
  
  delete aNeutrinoContainer;

}

void NtupleMaker::WriteNtuple()
{
  
  aFile = new TFile(FileName.c_str(), "recreate");
  
  aNtuple->Write();
  aFile->Close();
  
}

void NtupleMaker::FillInfo()
{

  EventID * aEventID = aEventInfo->event_ID();
        
  RunNumber   = (double) aEventID->run_number();
  EventNumber = (double) aEventID->event_number();
  
}

void NtupleMaker::ClearLeptons()
{

  NbSelLep       = 0; 
  LeptonType     = -1;
  LeptonPt       = -1;
  LeptonEta      = -1;
  LeptonPhi      = -1;
  LeptonCharge   = -1;
  Met            = -1;
  MetPhi         = -1;
  NuPz           = -1;
  MetLep         = -1;
  cosDPhi_LepMet = -1;
  DEta_LepMet    = -1;
  DR_LepMet      = -1;
  MtWlep         = -1;
  MetLepTCut     = -1;

}

void NtupleMaker::FillLeptons()
{

  Met    = aMissingEt->et()/GeV;
  MetPhi = aMissingEt->phi();
    
  int LeptonNumber = aSortedLeptonContainer->size();
  
  NbSelLep = (double)LeptonNumber;
  
  aLepton = 0;
      
  double LeptonPx = 0;
  double LeptonPy = 0;
                  
  if(LeptonNumber > 0)
  {
    
    aLepton      = aSortedLeptonContainer->at(0);
    hlvLepton    = aLepton->hlv();
        
    LeptonPx     = aLepton->px()/GeV;
    LeptonPy     = aLepton->py()/GeV;
    LeptonPt     = aLepton->pt()/GeV;
    LeptonEta    = aLepton->eta();
    LeptonPhi    = aLepton->phi();
    LeptonCharge = aLepton->charge();
    LeptonType   = aLepton->pdgId();
            
  }
    
  MetLep = Met+LeptonPt;
  
  double Metx    = aMissingEt->etx()/GeV;
  double Mety    = aMissingEt->ety()/GeV;
  double MetLepx = Metx+LeptonPx;
  double MetLepy = Mety+LeptonPy;
  
  MtWlep = sqrt(MetLep*MetLep-MetLepx*MetLepx-MetLepy*MetLepy);
          
  bool FindNeutrino = false;
    
  if (aLepton) FindNeutrino = NeutrinoUtils::candidatesFromWMass(aLepton, Metx, Mety, (*aNeutrinoContainer));

  aNeutrino = 0;
    
  double MetEta = 0;

  if(FindNeutrino)
  {
    
    Neutrino * aNeutrino1 = aNeutrinoContainer->at(0);
    Neutrino * aNeutrino2 = aNeutrinoContainer->at(1);
    
    double Pz1 = aNeutrino1->pz();
    double Pz2 = aNeutrino2->pz();
    
    if(fabs(Pz1) < fabs(Pz2)) aNeutrino = aNeutrinoContainer->at(0);
    else                      aNeutrino = aNeutrinoContainer->at(1);
    
    hlvNeutrino = aNeutrino->hlv();
    
    NuPz   = aNeutrino->pz()/GeV;
    MetEta = aNeutrino->eta(); 
    
  }
    
  double DPhi_LepMet = 0;
  
  if(aNeutrino)
  {
    
    if(aLepton)
    {
      DR_LepMet   = P4Help::deltaR(  aLepton, aNeutrino);
      DPhi_LepMet = P4Help::deltaPhi(aLepton, aNeutrino);
    }
          
    cosDPhi_LepMet = cos(DPhi_LepMet);
    MetLepTCut     = fabs(cosDPhi_LepMet)-(1.-Met/80.);
    DEta_LepMet    = fabs(LeptonEta - MetEta);
          
  }
    
}

void NtupleMaker::ClearJets()
{

  NbSelJet = -1;
  
  for(unsigned int i=0; i<4; i++)
  {
    Jphi[i] = -1;
    Jeta[i] = -1;
    Jpt[i]  = -1;
  }
  
  NbSelBJet   = -1;
  
  BJ1phi      = -1;
  BJ1eta      = -1;
  BJ1pt       = -1;
  BJ2phi      = -1;
  BJ2eta      = -1;
  BJ2pt       = -1;
  Mass_BJ1BJ2 = -1;
  
  NbSelUJet = -1;
  UJ1phi    = -1;
  UJ1eta    = -1;
  UJ1pt     = -1;
  UJ2phi    = -1;
  UJ2eta    = -1;
  UJ2pt     = -1;
  UJ3phi    = -1;
  UJ3eta    = -1;
  UJ3pt     = -1;
  
  cosDPhi_BJ1BJ2 = -1;
  cosDPhi_UJ1UJ2 = -1;
  cosDPhi_BJ1UJ1 = -1;
  cosDPhi_BJ2UJ1 = -1;
  cosDPhi_BJ1UJ2 = -1;
  cosDPhi_BJ2UJ2 = -1;
  cosDPhi_BJ1UJ3 = -1;
  
  DR_BJ1BJ2 = -1;
  DR_UJ1UJ2 = -1;
  DR_BJ1UJ1 = -1;
  DR_BJ2UJ1 = -1;
  DR_BJ1UJ2 = -1;
  DR_BJ2UJ2 = -1;
  DR_BJ1UJ3 = -1;
  
  DEta_BJ1BJ2 = -1;
  DEta_UJ1UJ2 = -1;
  DEta_BJ1UJ1 = -1;
  DEta_BJ2UJ1 = -1;
  DEta_BJ1UJ2 = -1;
  DEta_BJ2UJ2 = -1;
  DEta_BJ1UJ3 = -1;
    
}

void NtupleMaker::FillJets()
{

  int UJetNumber = aSortedUntaggedJetContainer->size();
  int BJetNumber = aSortedTaggedJetContainer->size();
  int JetNumber  = UJetNumber+BJetNumber;
        
  NbSelJet  = (double)(JetNumber);
  NbSelUJet = (double)(UJetNumber);
  NbSelBJet = (double)(BJetNumber);
  
  int JetIndex = 0;
  
  IParticleContainer::const_iterator Itr;
  
  for(Itr=aSortedJetContainer->begin(); Itr!=aSortedJetContainer->end(); Itr++)
  {
    
    if(JetIndex > 3) continue;
    
    IParticle * aJet = *Itr;
    
    Jphi[JetIndex] = aJet->phi();
    Jeta[JetIndex] = aJet->eta();
    Jpt[JetIndex]  = aJet->pt()/GeV;
    
    JetIndex++;
    
  }
  
  aBJet1 = 0;
  aBJet2 = 0;
  aUJet1  = 0;
  aUJet2  = 0;
  aUJet3  = 0;
          
  if(BJetNumber > 0)
  {
    aBJet1   = aSortedTaggedJetContainer->at(0);
    hlvBJet1 = aBJet1->hlv();
  
    BJ1phi = aBJet1->phi();
    BJ1eta = aBJet1->eta();
    BJ1pt  = aBJet1->pt()/GeV;
        
    if(BJetNumber > 1)
    {
      aBJet2   = aSortedTaggedJetContainer->at(1);
      hlvBJet2 = aBJet2->hlv();
    
      BJ2phi = aBJet2->phi();
      BJ2eta = aBJet2->eta();
      BJ2pt  = aBJet2->pt()/GeV;
        
      cosDPhi_BJ1BJ2 = cos(P4Help::deltaPhi(aBJet1, aBJet2));
      DR_BJ1BJ2      = P4Help::deltaR(aBJet1, aBJet2);
      DEta_BJ1BJ2    = fabs(BJ1eta-BJ2eta);
        
      Mass_BJ1BJ2 = (hlvBJet1+hlvBJet2).restMass();
      
    }  
    
  }
  
  if(UJetNumber > 0)
  {
    aUJet1   = aSortedUntaggedJetContainer->at(0);
    hlvUJet1 = aUJet1->hlv();
    
    UJ1phi = aUJet1->phi();
    UJ1eta = aUJet1->eta();
    UJ1pt  = aUJet1->pt()/GeV;
            
    if(BJetNumber > 0)
    {
      cosDPhi_BJ1UJ1 = cos(P4Help::deltaPhi(aBJet1, aUJet1));
      DR_BJ1UJ1      = P4Help::deltaR(aBJet1, aUJet1);
      DEta_BJ1UJ1    = fabs(UJ1eta-BJ1eta);

      if (BJetNumber > 1)
      {
        cosDPhi_BJ2UJ1 = cos(P4Help::deltaPhi(aBJet2, aUJet1));
        DR_BJ2UJ1      = P4Help::deltaR(aBJet2, aUJet1);
        DEta_BJ2UJ1    = fabs(UJ2eta-BJ1eta);
      }
    
    }
    
    if(UJetNumber > 1)
    {
      aUJet2   = aSortedUntaggedJetContainer->at(1);
      hlvUJet2 = aUJet2->hlv();

      UJ2phi = aUJet2->phi();
      UJ2eta = aUJet2->eta();
      UJ2pt  = aUJet2->pt()/GeV;
        
      cosDPhi_UJ1UJ2 = cos(P4Help::deltaPhi(aUJet1, aUJet2));
      DR_UJ1UJ2      = P4Help::deltaR(aUJet1, aUJet2);
      DEta_UJ1UJ2    = fabs(UJ2eta-UJ1eta);
        
      if (BJetNumber > 0)
      {
        cosDPhi_BJ1UJ2 = cos(P4Help::deltaPhi(aBJet1, aUJet2));
        DR_BJ1UJ2      = P4Help::deltaR(aBJet1, aUJet2);
        DEta_BJ1UJ2    = fabs(UJ2eta-BJ1eta);
     
        if (BJetNumber > 1)
        {
          cosDPhi_BJ2UJ2 = cos(P4Help::deltaPhi(aBJet2, aUJet2));
          DR_BJ2UJ2      = P4Help::deltaR(aBJet2, aUJet2);
          DEta_BJ2UJ2    = fabs(UJ2eta-BJ2eta);
        }
	
      }
    
      if(UJetNumber > 2)
      {
        aUJet3   = aSortedUntaggedJetContainer->at(2);
        hlvUJet3 = aUJet3->hlv();

        UJ3phi = aUJet3->phi();
        UJ3eta = aUJet3->eta();
        UJ3pt  = aUJet3->pt()/GeV;
            
        if (BJetNumber > 0)
        {
          cosDPhi_BJ1UJ3 = cos(P4Help::deltaPhi(aBJet1, aUJet3));
          DR_BJ1UJ3      = P4Help::deltaR(aBJet1, aUJet3);
          DEta_BJ1UJ3    = fabs(UJ3eta-BJ1eta);
        }
       
      }
      
    }
    
  } 
    
}

void NtupleMaker::ClearLeptonsJets()
{

  cosDPhi_LepBJ1 = -1;
  cosDPhi_LepBJ2 = -1;
  cosDPhi_LepUJ1 = -1;
  cosDPhi_LepUJ2 = -1;
  cosDPhi_LepUJ3 = -1;
  
  DR_LepBJ1 = -1;
  DR_LepBJ2 = -1;
  DR_LepUJ1 = -1;
  DR_LepUJ2 = -1;
  DR_LepUJ3 = -1;
  
  DEta_LepBJ1 = -1;
  DEta_LepBJ2 = -1;
  DEta_LepUJ1 = -1;
  DEta_LepUJ2 = -1;
  DEta_LepUJ3 = -1;

  
}

void NtupleMaker::FillLeptonsJets()
{

  if(aBJet1)
  {
    
    if(aLepton)
    {
      cosDPhi_LepBJ1 = cos(P4Help::deltaPhi(aLepton, aBJet1));
      DR_LepBJ1      = P4Help::deltaR(aLepton, aBJet1);
      DEta_LepBJ1    = fabs(aLepton->eta()-aBJet1->eta());
    }
           
    if(aBJet2)
    {
      
      if(aLepton)
      {
        cosDPhi_LepBJ2 = cos(P4Help::deltaPhi(aLepton, aBJet2));
        DR_LepBJ2      = P4Help::deltaR(aLepton, aBJet2);
        DEta_LepBJ2    = fabs(aLepton->eta()-aBJet2->eta());
      }
            
    }
    
  }
  
  if(aUJet1)
  {
    
    if(aLepton)
    {
      cosDPhi_LepUJ1 = cos(P4Help::deltaPhi(aLepton, aUJet1));
      DR_LepUJ1      = P4Help::deltaR(aLepton, aUJet1);
      DEta_LepUJ1    = fabs(aLepton->eta()-aUJet1->eta());
    }
                     
    if(aUJet2)
    {
    
      if(aLepton)
      {
        cosDPhi_LepUJ2 = cos(P4Help::deltaPhi(aLepton, aUJet2));
        DR_LepUJ2      = P4Help::deltaR(aLepton, aUJet2);
        DEta_LepUJ2    = fabs(aLepton->eta()-aUJet2->eta());
      }
              
      if(aUJet3)
      {
      
        if(aLepton)
        {
          cosDPhi_LepUJ3 = cos(P4Help::deltaPhi(aLepton, aUJet3));
          DR_LepUJ3      = P4Help::deltaR(aLepton, aUJet3);
          DEta_LepUJ3    = fabs(aLepton->eta()-aUJet3->eta());
        }
                
      } 
      
    }
    
  }  
  
}

void NtupleMaker::ClearWLeptonic()
{

  Mass_Wlep = -1;
  Eta_Wlep  = -1;
  Pt_Wlep   = -1;
  
}

void NtupleMaker::FillWLeptonic()
{

  if(aLepton && aNeutrino)
  {
    
    hlvWlep = hlvLepton+hlvNeutrino;
    Hep3Vector betaWlep = hlvWlep.boostVector();
    
    hlvLeptonRWlep   = hlvLepton;
    hlvNeutrinoRWlep = hlvNeutrino;
  
    hlvLeptonRWlep.boost(  -betaWlep);
    hlvNeutrinoRWlep.boost(-betaWlep);
    
    Mass_Wlep = hlvWlep.restMass()/GeV;
    Pt_Wlep   = hlvWlep.perp()/GeV;
    Eta_Wlep  = hlvWlep.pseudoRapidity();

  }
  
}

void NtupleMaker::ClearWHadronic()
{

  Mass_Whad12 = -1;
  Eta_Whad12  = -1;
  Pt_Whad12   = -1;
  Mass_Whad23 = -1;
  Eta_Whad23  = -1;
  Pt_Whad23   = -1;
  Mass_Whad13 = -1;
  Eta_Whad13  = -1;
  Pt_Whad13   = -1;
  
}

void NtupleMaker::FillWHadronic()
{

  if(aUJet1 && aUJet2)
  {
    
    hlvWhad12 = hlvUJet1+hlvUJet2;
    Hep3Vector betaWhad12 = hlvWhad12.boostVector();
    
    hlvUJet1RWhad12 = hlvUJet1;
    hlvUJet2RWhad12 = hlvUJet2;
    
    hlvUJet1RWhad12.boost(-betaWhad12); 
    hlvUJet2RWhad12.boost(-betaWhad12);
    
    Mass_Whad12 = hlvWhad12.restMass()/GeV;
    Pt_Whad12   = hlvWhad12.perp()/GeV;
    Eta_Whad12  = hlvWhad12.pseudoRapidity();
        
    if(aUJet3)
    {
      
      hlvWhad13 = hlvUJet1+hlvUJet3;
      Hep3Vector betaWhad13 = hlvWhad13.boostVector();
      
      hlvUJet1RWhad13 = hlvUJet1;
      hlvUJet3RWhad13 = hlvUJet3;
    
      hlvUJet1RWhad13.boost(-betaWhad13); 
      hlvUJet3RWhad13.boost(-betaWhad13);
  
      Mass_Whad13 = hlvWhad13.restMass()/GeV;
      Pt_Whad13   = hlvWhad13.perp()/GeV;
      Eta_Whad13  = hlvWhad13.pseudoRapidity();
          
      hlvWhad23 = hlvUJet2+hlvUJet3;
      Hep3Vector betaWhad23 = hlvWhad23.boostVector();
      
      hlvUJet2RWhad23 = hlvUJet2;
      hlvUJet3RWhad23 = hlvUJet3;
    
      hlvUJet2RWhad23.boost(-betaWhad23); 
      hlvUJet3RWhad23.boost(-betaWhad23);
    
      Mass_Whad23 = hlvWhad23.restMass()/GeV;
      Pt_Whad23   = hlvWhad23.perp()/GeV;
      Eta_Whad23  = hlvWhad23.pseudoRapidity();
    
    }
  
  }
  
}

void NtupleMaker::ClearTopLeptonic()
{

  Mass_TlepBJ1 = -1;
  Eta_TlepBJ1  = -1;
  Pt_TlepBJ1   = -1;
  Mass_TlepBJ2 = -1;
  Eta_TlepBJ2  = -1;
  Pt_TlepBJ2   = -1;
  
}

void NtupleMaker::FillTopLeptonic()
{

  if(aLepton && aNeutrino && aBJet1)
  {
  
    hlvTlepBJ1 = hlvWlep+hlvBJet1;
    Hep3Vector betaTlepBJ1 = hlvTlepBJ1.boostVector();
      
    hlvLeptonRTlepBJ1   = hlvLepton;
    hlvNeutrinoRTlepBJ1 = hlvNeutrino;
    hlvBJet1RTlepBJ1    = hlvBJet1;
    hlvWlepRTlepBJ1     = hlvWlep;
      
    hlvLeptonRTlepBJ1.boost(  -betaTlepBJ1);
    hlvNeutrinoRTlepBJ1.boost(-betaTlepBJ1);
    hlvBJet1RTlepBJ1.boost(   -betaTlepBJ1);
    hlvWlepRTlepBJ1.boost(    -betaTlepBJ1);
    
    Mass_TlepBJ1 = hlvTlepBJ1.restMass()/GeV;
    Pt_TlepBJ1   = hlvTlepBJ1.perp()/GeV;
    Eta_TlepBJ1  = hlvTlepBJ1.pseudoRapidity();
  
    if(aBJet2)
    {
      
      hlvTlepBJ2 = hlvWlep+hlvBJet2;
      Hep3Vector betaTlepBJ2 = hlvTlepBJ2.boostVector();
	
      hlvLeptonRTlepBJ2   = hlvLepton;
      hlvNeutrinoRTlepBJ2 = hlvNeutrino;
      hlvBJet2RTlepBJ2    = hlvBJet2;
      hlvWlepRTlepBJ2     = hlvWlep;
      
      hlvLeptonRTlepBJ2.boost(  -betaTlepBJ2);
      hlvNeutrinoRTlepBJ2.boost(-betaTlepBJ2);
      hlvBJet2RTlepBJ2.boost(   -betaTlepBJ2);
      hlvWlepRTlepBJ2.boost(    -betaTlepBJ2);
  
      Mass_TlepBJ2 = hlvTlepBJ2.restMass()/GeV;
      Pt_TlepBJ2   = hlvTlepBJ2.perp()/GeV;
      Eta_TlepBJ2  = hlvTlepBJ2.pseudoRapidity();
      
    }
    
  }
  
}

void NtupleMaker::ClearTopHadronic()
{

  Mass_Thad12BJ1 = -1;
  Eta_Thad12BJ1  = -1;
  Pt_Thad12BJ1   = -1;
  Mass_Thad12BJ2 = -1;
  Eta_Thad12BJ2  = -1;
  Pt_Thad12BJ2   = -1;
  Mass_Thad23BJ1 = -1;
  Eta_Thad23BJ1  = -1;
  Pt_Thad23BJ1   = -1;
  Mass_Thad23BJ2 = -1;
  Eta_Thad23BJ2  = -1;
  Pt_Thad23BJ2   = -1;
  Mass_Thad13BJ1 = -1;
  Eta_Thad13BJ1  = -1;
  Pt_Thad13BJ1   = -1;
  Mass_Thad13BJ2 = -1;
  Eta_Thad13BJ2  = -1;
  Pt_Thad13BJ2   = -1;
  
}

void NtupleMaker::FillTopHadronic()
{

  if(aUJet1 && aUJet2 && aBJet1)
  {
    
    hlvThad12BJ1 = hlvWhad12+hlvBJet1;
    Hep3Vector betaThad12BJ1 = hlvThad12BJ1.boostVector();
      
    hlvUJet1RThad12BJ1  = hlvUJet1;
    hlvUJet2RThad12BJ1  = hlvUJet2;
    hlvBJet1RThad12BJ1  = hlvBJet1;
    hlvWhad12RThad12BJ1 = hlvWhad12;
      
    hlvUJet1RThad12BJ1.boost( -betaThad12BJ1); 
    hlvUJet2RThad12BJ1.boost( -betaThad12BJ1);
    hlvBJet1RThad12BJ1.boost( -betaThad12BJ1);
    hlvWhad12RThad12BJ1.boost(-betaThad12BJ1);
      
    Mass_Thad12BJ1 = hlvThad12BJ1.restMass()/GeV;
    Pt_Thad12BJ1   = hlvThad12BJ1.perp()/GeV;
    Eta_Thad12BJ1  = hlvThad12BJ1.pseudoRapidity();
    
    if(aBJet2)
    {
        
      hlvThad12BJ2 = hlvWhad12+hlvBJet2;
      Hep3Vector betaThad12BJ2 = hlvThad12BJ2.boostVector();
	
      hlvUJet1RThad12BJ2  = hlvUJet1;
      hlvUJet2RThad12BJ2  = hlvUJet2;
      hlvBJet2RThad12BJ2  = hlvBJet2;
      hlvWhad12RThad12BJ2 = hlvWhad12;
      
      hlvUJet1RThad12BJ2.boost( -betaThad12BJ2); 
      hlvUJet2RThad12BJ2.boost( -betaThad12BJ2);
      hlvBJet2RThad12BJ2.boost( -betaThad12BJ2);
      hlvWhad12RThad12BJ2.boost(-betaThad12BJ2);
    
      Mass_Thad12BJ2 = hlvThad12BJ2.restMass()/GeV;
      Pt_Thad12BJ2   = hlvThad12BJ2.perp()/GeV;
      Eta_Thad12BJ2  = hlvThad12BJ2.pseudoRapidity();
	
    }
      
    if(aUJet3)
    {
    
      hlvThad13BJ1 = hlvWhad13+hlvBJet1;
      Hep3Vector betaThad13BJ1 = hlvThad13BJ1.boostVector();
	
      hlvUJet1RThad13BJ1  = hlvUJet1;
      hlvUJet3RThad13BJ1  = hlvUJet3;
      hlvBJet1RThad13BJ1  = hlvBJet1;
      hlvWhad13RThad13BJ1 = hlvWhad13;
      
      hlvUJet1RThad13BJ1.boost( -betaThad13BJ1); 
      hlvUJet3RThad13BJ1.boost( -betaThad13BJ1);
      hlvBJet1RThad13BJ1.boost( -betaThad13BJ1);
      hlvWhad13RThad13BJ1.boost(-betaThad13BJ1);
      
      Mass_Thad13BJ1 = hlvThad13BJ1.restMass()/GeV;
      Pt_Thad13BJ1   = hlvThad13BJ1.perp()/GeV;
      Eta_Thad13BJ1  = hlvThad13BJ1.pseudoRapidity();
      
      hlvThad23BJ1 = hlvWhad23+hlvBJet1;
      Hep3Vector betaThad23BJ1 = hlvThad23BJ1.boostVector();
	
      hlvUJet2RThad23BJ1  = hlvUJet2;
      hlvUJet3RThad23BJ1  = hlvUJet3;
      hlvBJet1RThad23BJ1  = hlvBJet1;
      hlvWhad23RThad23BJ1 = hlvWhad23;
      
      hlvUJet2RThad23BJ1.boost( -betaThad23BJ1); 
      hlvUJet3RThad23BJ1.boost( -betaThad23BJ1);
      hlvBJet1RThad23BJ1.boost( -betaThad23BJ1);
      hlvWhad23RThad23BJ1.boost(-betaThad23BJ1);
      
      Mass_Thad23BJ1 = hlvThad23BJ1.restMass()/GeV;
      Pt_Thad23BJ1   = hlvThad23BJ1.perp()/GeV;
      Eta_Thad23BJ1  = hlvThad23BJ1.pseudoRapidity();
    
      if(aBJet2)
      {
          
	hlvThad13BJ2 = hlvWhad13+hlvBJet2;
	Hep3Vector betaThad13BJ2 = hlvThad13BJ2.boostVector();
	  
	hlvUJet1RThad13BJ2  = hlvUJet1;
        hlvUJet3RThad13BJ2  = hlvUJet3;
        hlvBJet2RThad13BJ2  = hlvBJet2;
        hlvWhad13RThad13BJ2 = hlvWhad13;
      
        hlvUJet1RThad13BJ2.boost( -betaThad13BJ2); 
        hlvUJet3RThad13BJ2.boost( -betaThad13BJ2);
        hlvBJet2RThad13BJ2.boost( -betaThad13BJ2);
        hlvWhad13RThad13BJ2.boost(-betaThad13BJ2);
      
        Mass_Thad13BJ2 = hlvThad13BJ2.restMass()/GeV;
        Pt_Thad13BJ2   = hlvThad13BJ2.perp()/GeV;
        Eta_Thad13BJ2  = hlvThad13BJ2.pseudoRapidity();
      
        hlvThad23BJ2 = hlvWhad23+hlvBJet2;
	Hep3Vector betaThad23BJ2 = hlvThad23BJ2.boostVector();
	  
	hlvUJet2RThad23BJ2  = hlvUJet2;
        hlvUJet3RThad23BJ2  = hlvUJet3;
        hlvBJet2RThad23BJ2  = hlvBJet2;
        hlvWhad23RThad23BJ2 = hlvWhad23;
      
        hlvUJet2RThad23BJ2.boost( -betaThad23BJ2); 
        hlvUJet3RThad23BJ2.boost( -betaThad23BJ2);
        hlvBJet2RThad23BJ2.boost( -betaThad23BJ2);
        hlvWhad23RThad23BJ2.boost(-betaThad23BJ2);
      
        Mass_Thad23BJ2 = hlvThad13BJ2.restMass()/GeV;
        Pt_Thad23BJ2   = hlvThad13BJ2.perp()/GeV;
        Eta_Thad23BJ2  = hlvThad13BJ2.pseudoRapidity();
	
      }
      
    }
    
  }
  
}

void NtupleMaker::ClearGlobal()
{

  HtBJets   = -1;
  HtJets    = -1;
  HtTot     = -1;
  Shat      = -1;
  Mass_Jets = -1;
  Mass_tot  = -1;
  MetHtJets = -1;
  
}

void NtupleMaker::FillGlobal()
{

    
}

void NtupleMaker::ClearTopological()
{

  Sphericity = -1;
  Aplanarity = -1;
  Centrality = -1;

}

void NtupleMaker::FillTopological()
{

}

void NtupleMaker::ClearWLeptonicPolarization()
{

  cosLepWlepBJ1 = -1;
  cosNuWlepBJ1 = -1;
  cosLepWlepBJ2 = -1;
  cosNuWlepBJ2 = -1;
  
}

void NtupleMaker::FillWLeptonicPolarization()
{

  
}

void NtupleMaker::ClearWHadronicPolarization()
{

  cosUJ1Whad12BJ1 = -1;
  cosUJ2Whad12BJ1 = -1;
  cosUJ1Whad12BJ2 = -1;
  cosUJ2Whad12BJ2 = -1;
  cosUJ1Whad13BJ1 = -1;
  cosUJ3Whad13BJ1 = -1;
  cosUJ1Whad13BJ2 = -1;
  cosUJ3Whad13BJ2 = -1;
  cosUJ2Whad23BJ1 = -1;
  cosUJ3Whad23BJ1 = -1;
  cosUJ2Whad23BJ2 = -1;
  cosUJ3Whad23BJ2 = -1;
  
}

void NtupleMaker::FillWHadronicPolarization()
{

  
}

void NtupleMaker::ClearTopPolarization()
{

  cosLepTlepBJ1UJ1          = -1;
  cosNuTlepBJ1UJ1           = -1;
  cosWlepTlepBJ1UJ1         = -1;
  cosBJTlepBJ1UJ1           = -1;
  cosUJ1Thad12BJ1Wlep       = -1;
  cosUJ2Thad12BJ1Wlep       = -1;
  cosWhad12Thad12BJ1Wlep    = -1;
  cosBJThad12BJ1Wlep        = -1;
  cosLepTlepBJ1Whad12       = -1;
  cosNuTlepBJ1Whad12        = -1;
  cosWlepTlepBJ1Whad12      = -1;
  cosBJTlepBJ1Whad12        = -1;
  cosLepTlepBJ1Thad12BJ2    = -1;
  cosNuTlepBJ1Thad12BJ2     = -1;
  cosWlepTlepBJ1Thad12BJ2   = -1;
  cosBJTlepBJ1Thad12BJ2     = -1;
  cosLepTlepBJ2Thad12BJ1    = -1;
  cosNuTlepBJ2Thad12BJ1     = -1;
  cosWlepTlepBJ2Thad12BJ1   = -1;
  cosBJTlepBJ2Thad12BJ1     = -1;
  cosUJ1Thad12BJ1TlepBJ2    = -1;
  cosUJ2Thad12BJ1TlepBJ2    = -1;
  cosWhad12Thad12BJ1TlepBJ2 = -1;
  cosBJThad12BJ1TlepBJ2     = -1;
  cosUJ1Thad12BJ2TlepBJ1    = -1;
  cosUJ2Thad12BJ2TlepBJ1    = -1;
  cosWhad12Thad12BJ2TlepBJ1 = -1;
  cosBJThad12BJ2TlepBJ1     = -1;
  cosLepTlepBJ1BJ2          = -1;
  cosNuTlepBJ1BJ2           = -1;
  cosWlepTlepBJ1BJ2         = -1;
  cosBJTlepBJ1BJ2           = -1;
  cosLepTlepBJ2BJ1          = -1;
  cosNuTlepBJ2BJ1           = -1;
  cosWlepTlepBJ2BJ1         = -1;
  cosBJTlepBJ2BJ1           = -1;
  
}

void NtupleMaker::FillTopPolarization()
{

  
}

double NtupleMaker::RelativeCosine(HepLorentzVector& FourVector1, HepLorentzVector& FourVector2)
{

  Hep3Vector ThreeVector1 = FourVector1.vect();
  Hep3Vector ThreeVector2 = FourVector2.vect();
  
  double cosinus = ThreeVector1.dot(ThreeVector2)/(ThreeVector1.mag()*ThreeVector2.mag());
  return cosinus;

}
