#include "SingleTopDPDMaker/OverlapFilter.h"

OverlapFilter::OverlapFilter(const std::string& name, ISvcLocator* pSvcLocator) : Algorithm(name, pSvcLocator)
{

  declareProperty("RootContainerName", RootContainerName = "ElectronAODCollection");
  
  declareProperty("InputContainerName",  InputContainerName );
  declareProperty("OutputContainerName", OutputContainerName);
  
  declareProperty("SelectionInfoContainerName", SelectionInfoContainerName);
  declareProperty("SelectionLabel",             SelectionLabel            );
  
  declareProperty("FilterSameContainerType", FilterSameContainerType);
  declareProperty("DeltaRCut",               DeltaRCut              );
          
}

OverlapFilter::~OverlapFilter(){}

StatusCode OverlapFilter::initialize()
{
  
  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Initializing " << name() << endreq;
                    
  bool Test1 = InputContainerName.size() > 0;
  bool Test2 = InputContainerName.size() == OutputContainerName.size();
  bool Test3 = InputContainerName.size() == SelectionInfoContainerName.size();
  bool Test4 = InputContainerName.size() == SelectionLabel.size();
  bool Test5 = InputContainerName.size() == FilterSameContainerType.size();
  bool Test6 = InputContainerName.size() == DeltaRCut.size();
  
  bool isTestContainerSize = Test1 && Test2 && Test3 && Test4 && Test5 && Test6;
  
  if(!isTestContainerSize)
  {
  
    MsgLog << MSG::FATAL << "Wrong Container Size Properties :" << endreq;
    
    if(!Test1) MsgLog << MSG::FATAL << "Wrong Input Container Name Size"          << endreq;
    if(!Test2) MsgLog << MSG::FATAL << "Wrong Output Container Name Size"         << endreq;
    if(!Test3) MsgLog << MSG::FATAL << "Wrong Selection Info Container Name Size" << endreq;
    if(!Test4) MsgLog << MSG::FATAL << "Wrong Selection Label Size"               << endreq;
    if(!Test5) MsgLog << MSG::FATAL << "Wrong Filter Same Container Type Size"    << endreq;
    if(!Test6) MsgLog << MSG::FATAL << "Wrong DeltaR Cut Size"                    << endreq;
    
    return StatusCode::FAILURE;
    
  }
  
  bool isTestContainerType = this->TestContainerType();
      
  if(!isTestContainerType)
  {
    MsgLog << MSG::FATAL << "Wrong Container Type Properties" << endreq;
    return StatusCode::FAILURE;
  }
  
  MsgLog << MSG::INFO << "Root Container Name = " << RootContainerName   << endreq;
  
  for(unsigned int i=0; i<InputContainerName.size(); i++)
  {
    
    MsgLog << MSG::INFO << "Input Container Name       = " << InputContainerName[i]      << endreq;
    MsgLog << MSG::INFO << "Output Container Name      = " << OutputContainerName[i]     << endreq;
    MsgLog << MSG::INFO << "Filter Same Container Type = " << FilterSameContainerType[i] << endreq;
    MsgLog << MSG::INFO << "DeltaR Cut                 = " << DeltaRCut[i]               << endreq;
    
    if(SelectionInfoContainerName[i] != "None")
    {
      MsgLog << MSG::INFO << "Selection Info Container Name = " << SelectionInfoContainerName[i] << endreq;
      MsgLog << MSG::INFO << "Selection Label               = " << SelectionLabel[i]             << endreq;
    }
    
  }
  
  StatusCode status = service("StoreGateSvc", aStoreGateSvc);

  if(status.isFailure()) 
  {
    MsgLog << MSG::FATAL << "StoreGateSvc Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  InputEventNumber  = 0;
  OutputEventNumber = 0;
                
  return StatusCode::SUCCESS;
  
}

StatusCode OverlapFilter::finalize()
{

  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Finalizing " << name() << endreq;
  
  MsgLog << MSG::INFO << "Input  Event Number = " << InputEventNumber  << endreq;
  MsgLog << MSG::INFO << "Output Event Number = " << OutputEventNumber << endreq;
  
  for(unsigned int i=0; i<InputContainerName.size(); i++)
  {
    MsgLog << MSG::INFO << "Container " << InputContainerName[i] << " :" << endreq;
    
    MsgLog << MSG::INFO << "Input  Object Number = " << InputObjectNumber[i]  << endreq;
    MsgLog << MSG::INFO << "Output Object Number = " << OutputObjectNumber[i] << endreq;
  }
  
  return StatusCode::SUCCESS;
  
}

StatusCode OverlapFilter::execute()
{

  MsgStream MsgLog(msgSvc(), name());
  
  StatusCode status;
  
  InputEventNumber++;
      
  ParticleContainer.clear();
  
  status = this->InitializeOverlapRemoval();
  if(status.isFailure()) return StatusCode::FAILURE;
      
  for(unsigned int i=0; i<InputContainerName.size(); i++)
  {
  
    status = aStoreGateSvc->retrieve(aSelectionInfoContainer, SelectionInfoContainerName[i]);
  
    if(status.isFailure())
    {
    
      if(SelectionInfoContainerName[i] != "None")
      {
    
        if(InputEventNumber == 1) MsgLog << MSG::INFO << "Creating Container " << SelectionInfoContainerName[i] << endreq;
    
        aSelectionInfoContainer = new SelectionInfoContainer(SG::VIEW_ELEMENTS);
    
        status = aStoreGateSvc->record(aSelectionInfoContainer, SelectionInfoContainerName[i]);
    
        if(status.isFailure())
        {
          MsgLog << MSG::FATAL << SelectionInfoContainerName[i] << " Container Unavailable" << endreq;
          return StatusCode::FAILURE;
        }
    
      }
    
      else 
      {
        aSelectionInfoContainer = 0;
        if(InputEventNumber == 1) MsgLog << MSG::INFO << "No Selection Info Container" << endreq;
      }
        
    }
    
    Mask.clear();
    
    status = this->ExecuteOverlapRemoval(i);
    if(status.isFailure()) return StatusCode::FAILURE;
                 
    if(aSelectionInfoContainer)
    {  
      SelectionInfo * aSelectionInfo = new SelectionInfo();
  
      aSelectionInfo->SetName(InputContainerName[i]);
      aSelectionInfo->SetLabel(SelectionLabel[i]);
      aSelectionInfo->SetMask(Mask);
    
      aSelectionInfoContainer->push_back(aSelectionInfo);
    } 
        
  }
  
  setFilterPassed(true);
  
  OutputEventNumber++;
        
  return StatusCode::SUCCESS;
  
}

StatusCode OverlapFilter::InitializeOverlapRemoval()
{

  MsgStream MsgLog(msgSvc(), name());
    
  bool isInitialized = false;
  
  if     (RootContainerType == "Photon")    isInitialized = this->StoreRootContainer(aPhotonContainer);
  else if(RootContainerType == "Electron")  isInitialized = this->StoreRootContainer(aElectronContainer);
  else if(RootContainerType == "Muon")      isInitialized = this->StoreRootContainer(aMuonContainer);
  else if(RootContainerType == "TauJet")    isInitialized = this->StoreRootContainer(aTauJetContainer);
  else if(RootContainerType == "Jet")       isInitialized = this->StoreRootContainer(aJetContainer);
  else if(RootContainerType == "Truth")     isInitialized = this->StoreRootContainer(aTruthParticleContainer);
  else if(RootContainerType == "Composite") isInitialized = this->StoreRootContainer(aCompositeParticleContainer);
  
  if(!isInitialized) 
  {
    MsgLog << MSG::FATAL << RootContainerName << " Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  return StatusCode::SUCCESS;
  
}

StatusCode OverlapFilter::ExecuteOverlapRemoval(int i)
{

  MsgStream MsgLog(msgSvc(), name());
  
  std::pair<int, int> Sizes;
  
  if     (ContainerType[i] == "Photon")    Sizes = this->RemoveOverlap(aPhotonContainer, aPhoton, InputContainerName[i], 
                                                                       OutputContainerName[i], ContainerType[i],
								       FilterSameContainerType[i], DeltaRCut[i]);
  else if(ContainerType[i] == "Electron")  Sizes = this->RemoveOverlap(aElectronContainer, aElectron, InputContainerName[i], 
                                                                       OutputContainerName[i], ContainerType[i], 
								       FilterSameContainerType[i], DeltaRCut[i]);
  else if(ContainerType[i] == "Muon")      Sizes = this->RemoveOverlap(aMuonContainer, aMuon, InputContainerName[i], 
                                                                       OutputContainerName[i], ContainerType[i],
								       FilterSameContainerType[i], DeltaRCut[i]);
  else if(ContainerType[i] == "TauJet")    Sizes = this->RemoveOverlap(aTauJetContainer, aTauJet, InputContainerName[i], 
                                                                       OutputContainerName[i], ContainerType[i], 
								       FilterSameContainerType[i], DeltaRCut[i]);
  else if(ContainerType[i] == "Jet")       Sizes = this->RemoveOverlap(aJetContainer, aJet, InputContainerName[i], 
                                                                       OutputContainerName[i], ContainerType[i],
								       FilterSameContainerType[i], DeltaRCut[i]);
  else if(ContainerType[i] == "Truth")     Sizes = this->RemoveOverlap(aTruthParticleContainer, aTruthParticle, InputContainerName[i], 
                                                                       OutputContainerName[i], ContainerType[i], 
								       FilterSameContainerType[i], DeltaRCut[i]);
  else if(ContainerType[i] == "Composite") Sizes = this->RemoveOverlap(aCompositeParticleContainer, aCompositeParticle, InputContainerName[i], 
                                                                       OutputContainerName[i], ContainerType[i], 
								       FilterSameContainerType[i], DeltaRCut[i]);
    
  if(Sizes.first == -1) 
  {
    MsgLog << MSG::FATAL << InputContainerName[i] << " Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  InputObjectNumber[i]  += Sizes.first;
  OutputObjectNumber[i] += Sizes.second;
  
  return StatusCode::SUCCESS;
  
}

template <class CT> bool OverlapFilter::StoreRootContainer(const CT * aRootContainer)
{

  StatusCode status = aStoreGateSvc->retrieve(aRootContainer, RootContainerName);
  if(status.isFailure()) return false;
  
  typename CT::const_iterator Itr;
  
  for(Itr=aRootContainer->begin(); Itr!=aRootContainer->end(); Itr++)
  {
    
    const INavigable4Momentum * aParticle = dynamic_cast<INavigable4Momentum*>(*Itr);
	
    std::pair<std::string, const INavigable4Momentum*> Particle;
    
    Particle.first  = RootContainerType;
    Particle.second = aParticle;
	
    ParticleContainer.push_back(Particle);
    
  }
  
  return true;

}

template <class CT, class T> std::pair<int, int> OverlapFilter::RemoveOverlap(const CT * aContainer, T * aObject, 
std::string InputName, std::string OutputName, std::string Type, bool SameType, double DRCut)
{

  StatusCode status;
  
  std::pair<int, int> Sizes;
  
  Sizes.first  = -1;
  Sizes.second = -1;

  status = aStoreGateSvc->retrieve(aContainer, InputName);
  if(status.isFailure()) return Sizes;
        
  CT * aSelectedContainer = new CT(SG::VIEW_ELEMENTS);
  status = aStoreGateSvc->record(aSelectedContainer, OutputName);
        
  typename CT::const_iterator Itr;
            
  for(Itr=aContainer->begin(); Itr!=aContainer->end(); Itr++)
  {
      
    const INavigable4Momentum * aParticle = dynamic_cast<INavigable4Momentum*>(*Itr);
    		    
    bool isSelected = true;
    
    std::vector<std::pair <std::string, const INavigable4Momentum*> >::const_iterator ParticleItr;
		
    for(ParticleItr=ParticleContainer.begin(); ParticleItr!=ParticleContainer.end(); ParticleItr++)
    {
    
      bool isTest = (!SameType && Type != (*ParticleItr).first) || SameType;
	   
      if(isTest)
      {
        double dR = P4Help::deltaR(aParticle, (*ParticleItr).second);
	isSelected = dR > DRCut;
	if(!isSelected) break;
      }
	     
    }
		
    if(isSelected)
    {
      aObject = new T(**Itr);
      aSelectedContainer->push_back(aObject);
      
      Mask.push_back(true);
    }
    
    else 
    {
      Mask.push_back(false);
    }
    				
  }
              
  for(Itr=aSelectedContainer->begin(); Itr!=aSelectedContainer->end(); Itr++)
  {
    const INavigable4Momentum * aParticle = dynamic_cast<INavigable4Momentum*>(*Itr);
	
    std::pair<std::string, const INavigable4Momentum*> Particle;
    Particle.first  = Type;
    Particle.second = aParticle;
	
    ParticleContainer.push_back(Particle);
  }
          
  status = aStoreGateSvc->setConst(aSelectedContainer);
  
  Sizes.first  = static_cast<int>(aContainer->size());
  Sizes.second = static_cast<int>(aSelectedContainer->size());
    
  return Sizes;
      
}

bool OverlapFilter::TestContainerType()
{

  std::string Name = RootContainerName;
  
  bool Veto = Name.find("Truth", 0) == std::string::npos && Name.find("Composite", 0) == std::string::npos;
  
  if     (Name.find("Photon",    0) != std::string::npos && Veto) RootContainerType = "Photon";
  else if(Name.find("Electron",  0) != std::string::npos && Veto) RootContainerType = "Electron";
  else if(Name.find("Muon",      0) != std::string::npos && Veto) RootContainerType = "Muon";
  else if(Name.find("Tau",       0) != std::string::npos && Veto) RootContainerType = "TauJet";
  else if(Name.find("Jet",       0) != std::string::npos && Veto) RootContainerType = "Jet";
  else if(Name.find("Truth",     0) != std::string::npos)         RootContainerType = "Truth";
  else if(Name.find("Composite", 0) != std::string::npos)         RootContainerType = "Composite";
  else return false;
  
  for(unsigned int i=0; i<InputContainerName.size(); i++)
  {
    
    std::string Name = InputContainerName[i];
    
    bool Veto = Name.find("Truth", 0) == std::string::npos && Name.find("Composite", 0) == std::string::npos;
  
    if     (Name.find("Photon",    0) != std::string::npos && Veto) ContainerType.push_back("Photon");
    else if(Name.find("Electron",  0) != std::string::npos && Veto) ContainerType.push_back("Electron");
    else if(Name.find("Muon",      0) != std::string::npos && Veto) ContainerType.push_back("Muon");
    else if(Name.find("Tau",       0) != std::string::npos && Veto) ContainerType.push_back("TauJet");
    else if(Name.find("Jet",       0) != std::string::npos && Veto) ContainerType.push_back("Jet");
    else if(Name.find("Truth",     0) != std::string::npos)         ContainerType.push_back("Truth");
    else if(Name.find("Composite", 0) != std::string::npos)         ContainerType.push_back("Composite");
    else return false;
    
    InputObjectNumber.push_back (0);
    OutputObjectNumber.push_back(0);
    
  }
    
  return true;

}
