#include "SingleTopDPDMaker/PhotonSelection.h"

PhotonSelection::PhotonSelection(const std::string& name, ISvcLocator* pSvcLocator) : Algorithm(name, pSvcLocator)
{

  declareProperty("InputContainerName",  InputContainerName  = "PhotonAODCollection"    );
  declareProperty("OutputContainerName", OutputContainerName = "SelectedPhotonContainer");
  
  declareProperty("SelectionInfoContainerName", SelectionInfoContainerName = "None"           );
  declareProperty("SelectionLabel",             SelectionLabel             = "PhotonSelection"); 
  
  declareProperty("ApplyEtConeCut", ApplyEtConeCut = false   );
  
  declareProperty("Author",         Author         = "Photon");
  declareProperty("EmMask",         EmMask         = "None"  );
  declareProperty("PtCut",          PtCut          = 20.0*GeV);
  declareProperty("EtaCut",         EtaCut         = 2.5     );
  declareProperty("ConeSize",       ConeSize       = 0.2     );
  declareProperty("EtConeCut",      EtConeCut      = 6.0*GeV );
  
}

PhotonSelection::~PhotonSelection(){}

StatusCode PhotonSelection::initialize()
{
  
  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Initializing " << name() << endreq;
  
  MsgLog << MSG::INFO << "Input  Container Name = " << InputContainerName  << endreq;
  MsgLog << MSG::INFO << "Output Container Name = " << OutputContainerName << endreq;
  
  if(SelectionInfoContainerName != "None")
  {
    MsgLog << MSG::INFO << "Selection Info Container Name = " << SelectionInfoContainerName << endreq;
    MsgLog << MSG::INFO << "Selection Label               = " << SelectionLabel             << endreq;
  }
  
  MsgLog << MSG::INFO << "Apply Et Cone Cut = " << ApplyEtConeCut << endreq;
  
  MsgLog << MSG::INFO << "Author      = " << Author              << endreq;
  MsgLog << MSG::INFO << "Em Mask     = " << EmMask              << endreq;
  MsgLog << MSG::INFO << "Pt Cut      = " << PtCut/GeV << " GeV" << endreq;
  MsgLog << MSG::INFO << "Eta Cut     = " << EtaCut              << endreq;
  
  if(ApplyEtConeCut)
  {
    MsgLog << MSG::INFO << "Cone Size   = " << ConeSize                << endreq;
    MsgLog << MSG::INFO << "Et Cone Cut = " << EtConeCut/GeV << " GeV" << endreq;
  }
        
  bool isTestAuthor   = this->TestAuthorOption  ();
  bool isTestEmMask   = this->TestEmMaskOption  ();
  bool isTestConeSize = this->TestConeSizeOption();
  
  if(!isTestAuthor)
  {
    MsgLog << MSG::FATAL << "Wrong Author Property" << endreq;
    return StatusCode::FAILURE;
  }
  
  if(!isTestEmMask)
  {
    MsgLog << MSG::FATAL << "Wrong Em Mask Property" << endreq;
    return StatusCode::FAILURE;
  }
  
  if(!isTestConeSize)
  {
    MsgLog << MSG::FATAL << "Wrong Cone Size Property" << endreq;
    return StatusCode::FAILURE;
  }
  
  StatusCode status = service("StoreGateSvc", aStoreGateSvc);

  if(status.isFailure()) 
  {
    MsgLog << MSG::FATAL << "StoreGateSvc Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  InputEventNumber  = 0;
  OutputEventNumber = 0;
  
  InputObjectNumber  = 0;
  OutputObjectNumber = 0;
      
  return StatusCode::SUCCESS;
  
}

StatusCode PhotonSelection::finalize()
{
  
  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Finalizing " << name() << endreq;
  
  MsgLog << MSG::INFO << "Input  Event Number  = " << InputEventNumber  << endreq;
  MsgLog << MSG::INFO << "Output Event Number  = " << OutputEventNumber << endreq;
  
  MsgLog << MSG::INFO << "Input  Object Number = " << InputObjectNumber  << endreq;
  MsgLog << MSG::INFO << "Output Object Number = " << OutputObjectNumber << endreq;
  
  return StatusCode::SUCCESS;
  
}

StatusCode PhotonSelection::execute()
{
  
  MsgStream MsgLog(msgSvc(), name());
  
  StatusCode status;
  
  InputEventNumber++;
      
  status = aStoreGateSvc->retrieve(aInputContainer, InputContainerName);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Input Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
    
  aOutputContainer = new PhotonContainer(SG::VIEW_ELEMENTS);
  
  status = aStoreGateSvc->record(aOutputContainer, OutputContainerName);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Output Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  status = aStoreGateSvc->retrieve(aSelectionInfoContainer, SelectionInfoContainerName);
  
  if(status.isFailure())
  {
    
    if(SelectionInfoContainerName != "None")
    {
    
      if(InputEventNumber == 1) MsgLog << MSG::INFO << "Creating Selection Info Container" << endreq;
    
      aSelectionInfoContainer = new SelectionInfoContainer(SG::VIEW_ELEMENTS);
    
      status = aStoreGateSvc->record(aSelectionInfoContainer, SelectionInfoContainerName);
    
      if(status.isFailure())
      {
        MsgLog << MSG::FATAL << "Selection Info Container Unavailable" << endreq;
        return StatusCode::FAILURE;
      }
    
    }
    
    else 
    {
      aSelectionInfoContainer = 0;
      if(InputEventNumber == 1) MsgLog << MSG::INFO << "No Selection Info Container" << endreq;
    }
        
  }
      
  Mask.clear();
  
  this->Select();
  
  if(aSelectionInfoContainer)
  {  
    SelectionInfo * aSelectionInfo = new SelectionInfo();
  
    aSelectionInfo->SetName(InputContainerName);
    aSelectionInfo->SetLabel(SelectionLabel);
    aSelectionInfo->SetMask(Mask);
    
    aSelectionInfoContainer->push_back(aSelectionInfo);
  }
  
  status = aStoreGateSvc->setConst(aOutputContainer);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Output Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  InputObjectNumber  += static_cast<int>(aInputContainer->size());
  OutputObjectNumber += static_cast<int>(aOutputContainer->size());
  
  setFilterPassed(true);
  
  OutputEventNumber++;
    
  return StatusCode::SUCCESS;
  
}

void PhotonSelection::Select()
{

  PhotonContainer::const_iterator Itr;
  
  for(Itr=aInputContainer->begin(); Itr!= aInputContainer->end(); Itr++)
  {
    
    const Analysis::Photon * aPhoton = *Itr;
    
    bool isSelected = this->SelectPhoton(aPhoton);
        
    if(isSelected)
    {           
      Analysis::Photon * aSelectedPhoton = new Analysis::Photon(*aPhoton);
      aOutputContainer->push_back(aSelectedPhoton);
      
      Mask.push_back(true);
    }
    
    else 
    {
      Mask.push_back(false);
    }
                
  }
    
}

bool PhotonSelection::SelectPhoton(const Analysis::Photon * aPhoton)
{

  bool isPtCut  = aPhoton->pt() > PtCut;
  bool isEtaCut = fabs(aPhoton->eta()) < EtaCut;
    
  bool isAuthor = this->TestAuthor(aPhoton);
  bool isEmCut  = this->TestEmMask(aPhoton);
    
  bool isIsolated = true;
  if(ApplyEtConeCut) isIsolated = this->TestEtCone(aPhoton);
     
  bool isSelected = isPtCut && isEtaCut && isAuthor && isEmCut && isIsolated;
  
  return isSelected;

}

bool PhotonSelection::TestAuthorOption()
{

  bool Test = false;
  
  if     (Author == "All")    Test = true;
  else if(Author == "Photon") Test = true;
  
  return Test;

}

bool PhotonSelection::TestAuthor(const Analysis::Photon * aPhoton)
{

  int author = aPhoton->author();

  bool Test = false;
  
  if     (Author == "All")    Test = true;
  else if(Author == "Photon") Test = (author == 4);
    
  return Test;    

}

bool PhotonSelection::TestEmMaskOption()
{

  bool Test = false;
  
  if     (EmMask == "None")      Test = true;
  else if(EmMask == "Tight")     Test = true;
  else if(EmMask == "CALONOISO") Test = true;
  
  return Test; 
  
}

bool PhotonSelection::TestEmMask(const Analysis::Photon * aPhoton)
{

  bool Test = false;
  
  if     (EmMask == "None")      Test = true;
  else if(EmMask == "Tight")     Test = aPhoton->isem(egammaPID::PhotonTight) == 0;
  else if(EmMask == "CALONOISO") Test = aPhoton->isem(egammaPID::CALONOISO)   == 0;
  
  return Test; 

}

bool PhotonSelection::TestConeSizeOption()
{

  bool Test = false;
  
  if(ConeSize == 0.2) 
  {
    Test = true;
    ConeSizeParameter = egammaParameters::etcone20;
  }
  
  return Test; 

}

bool PhotonSelection::TestEtCone(const Analysis::Photon * aPhoton)
{

  bool Test = false;

  const EMShower * aEMShower = aPhoton->detail<EMShower>("egDetailAOD");
    
  if(aEMShower) 
  {
    Test = aEMShower->parameter(ConeSizeParameter) < EtConeCut;
  }
  
  return Test; 

}
