#include "SingleTopDPDMaker/TaggedJetSelection.h"

TaggedJetSelection::TaggedJetSelection(const std::string& name, ISvcLocator* pSvcLocator) : Algorithm(name, pSvcLocator)
{

  declareProperty("InputContainerName",  InputContainerName  = "Cone4H1TowerJets"          );
  declareProperty("OutputContainerName", OutputContainerName = "SelectedTaggedJetContainer");
  
  declareProperty("SelectionInfoContainerName", SelectionInfoContainerName = "None"              );
  declareProperty("SelectionLabel",             SelectionLabel             = "SelectionTaggedJet");         
  
  declareProperty("PtCut",            PtCut            = 20.0*GeV);
  declareProperty("EtaCut",           EtaCut           = 2.5     );
  declareProperty("TaggingAlgorithm", TaggingAlgorithm = ""      );
  declareProperty("TaggingWeightCut", TaggingWeightCut = 7.05    );
  
}

TaggedJetSelection::~TaggedJetSelection(){}

StatusCode TaggedJetSelection::initialize()
{
  
  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Initializing " << name() << endreq;
  
  MsgLog << MSG::INFO << "Input  Container Name = " << InputContainerName  << endreq;
  MsgLog << MSG::INFO << "Output Container Name = " << OutputContainerName << endreq;
  
  if(SelectionInfoContainerName != "None")
  {
    MsgLog << MSG::INFO << "Selection Info Container Name = " << SelectionInfoContainerName << endreq;
    MsgLog << MSG::INFO << "Selection Label               = " << SelectionLabel             << endreq;
  }
  
  MsgLog << MSG::INFO << "Pt Cut  = " << PtCut/GeV << " GeV" << endreq;
  MsgLog << MSG::INFO << "Eta Cut = " << EtaCut              << endreq;
  
  std::string Algorithm = "Standard";
  if(TaggingAlgorithm != "") Algorithm = TaggingAlgorithm;
  
  MsgLog << MSG::INFO << "Tagging Algorithm  = " << Algorithm        << endreq;
  MsgLog << MSG::INFO << "Tagging Weight Cut = " << TaggingWeightCut << endreq;
  
  StatusCode status = service("StoreGateSvc", aStoreGateSvc);

  if(status.isFailure()) 
  {
    MsgLog << MSG::FATAL << "StoreGateSvc Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  InputEventNumber  = 0;
  OutputEventNumber = 0;
  
  InputObjectNumber  = 0;
  OutputObjectNumber = 0;
      
  return StatusCode::SUCCESS;
  
}

StatusCode TaggedJetSelection::finalize()
{
  
  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Finalizing " << name() << endreq;
  
  MsgLog << MSG::INFO << "Input  Event Number  = " << InputEventNumber  << endreq;
  MsgLog << MSG::INFO << "Output Event Number  = " << OutputEventNumber << endreq;
  
  MsgLog << MSG::INFO << "Input  Object Number = " << InputObjectNumber  << endreq;
  MsgLog << MSG::INFO << "Output Object Number = " << OutputObjectNumber << endreq;
  
  return StatusCode::SUCCESS;
  
}

StatusCode TaggedJetSelection::execute()
{
  
  MsgStream MsgLog(msgSvc(), name());
  
  StatusCode status;
  
  InputEventNumber++;
      
  status = aStoreGateSvc->retrieve(aInputContainer, InputContainerName);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Input Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
    
  aOutputContainer = new JetCollection(SG::VIEW_ELEMENTS);
  
  status = aStoreGateSvc->record(aOutputContainer, OutputContainerName);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Output Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  status = aStoreGateSvc->retrieve(aSelectionInfoContainer, SelectionInfoContainerName);
  
  if(status.isFailure())
  {
    
    if(SelectionInfoContainerName != "None")
    {
    
      if(InputEventNumber == 1) MsgLog << MSG::INFO << "Creating Selection Info Container" << endreq;
    
      aSelectionInfoContainer = new SelectionInfoContainer(SG::VIEW_ELEMENTS);
    
      status = aStoreGateSvc->record(aSelectionInfoContainer, SelectionInfoContainerName);
    
      if(status.isFailure())
      {
        MsgLog << MSG::FATAL << "Selection Info Container Unavailable" << endreq;
        return StatusCode::FAILURE;
      }
    
    }
    
    else 
    {
      aSelectionInfoContainer = 0;
      if(InputEventNumber == 1) MsgLog << MSG::INFO << "No Selection Info Container" << endreq;
    }
        
  }
      
  Mask.clear();
  
  this->Select();
  
  if(aSelectionInfoContainer)
  {  
    SelectionInfo * aSelectionInfo = new SelectionInfo();
  
    aSelectionInfo->SetName(InputContainerName);
    aSelectionInfo->SetLabel(SelectionLabel);
    aSelectionInfo->SetMask(Mask);
    
    aSelectionInfoContainer->push_back(aSelectionInfo);
  }
  
  status = aStoreGateSvc->setConst(aOutputContainer);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Output Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  bool Test = this->TestTaggingAlgorithmOption();
  
  if(!Test)
  {
    MsgLog << MSG::FATAL << "Wrong Properties" << endreq;
    return StatusCode::FAILURE;
  }
  
  InputObjectNumber  += static_cast<int>(aInputContainer->size());
  OutputObjectNumber += static_cast<int>(aOutputContainer->size());
  
  setFilterPassed(true);
  
  OutputEventNumber++;
    
  return StatusCode::SUCCESS;
  
}


void TaggedJetSelection::Select()
{
    
  JetCollection::const_iterator Itr;
    
  for(Itr=aInputContainer->begin(); Itr!= aInputContainer->end(); Itr++)
  {
    
    const Jet * aJet = *Itr;
            
    bool isSelected = this->SelectTaggedJet(aJet);
            
    if(isSelected)
    {
      Jet * aSelectedJet = new Jet(*aJet);
      aOutputContainer->push_back(aSelectedJet);
      
      Mask.push_back(true);
    }
    
    else 
    {
      Mask.push_back(false);
    }
        
  }
        
}

bool TaggedJetSelection::SelectTaggedJet(const Jet * aJet)
{

  bool isPtCut  = aJet->pt() > PtCut;
  bool isEtaCut = fabs(aJet->eta()) < EtaCut;
    
  double TaggingWeight = aJet->getFlavourTagWeight(TaggingAlgorithm);
  bool   isTagged      = TaggingWeight > TaggingWeightCut;
    
  bool isSelected = isPtCut && isEtaCut  && isTagged;
  
  return isSelected;

}

bool TaggedJetSelection::TestTaggingAlgorithmOption()
{

  bool Test = false;
  
  if     (TaggingAlgorithm == "")                Test = true;
  else if(TaggingAlgorithm == "TrackCounting2D") Test = true;
  else if(TaggingAlgorithm == "JetProb")         Test = true;
  else if(TaggingAlgorithm == "IP1D")            Test = true;
  else if(TaggingAlgorithm == "IP2D")            Test = true;
  else if(TaggingAlgorithm == "IP3D")            Test = true;
  else if(TaggingAlgorithm == "SV1")             Test = true;
  else if(TaggingAlgorithm == "SV2")             Test = true;
  else if(TaggingAlgorithm == "BaselineTagger")  Test = true;
  else if(TaggingAlgorithm == "JetFitterTag")    Test = true;
  else if(TaggingAlgorithm == "JetFitterCOMB")   Test = true;
  else if(TaggingAlgorithm == "JetFitterTagNN")  Test = true;
  else if(TaggingAlgorithm == "JetFitterCOMBNN") Test = true;
  else if(TaggingAlgorithm == "SoftMuonTag")     Test = true;
  else if(TaggingAlgorithm == "SoftLeptonTag")   Test = true;
    
  return Test;

}

