#include "SingleTopDPDMaker/TauJetSelection.h"

TauJetSelection::TauJetSelection(const std::string& name, ISvcLocator* pSvcLocator) : Algorithm(name, pSvcLocator)
{

  declareProperty("InputContainerName",  InputContainerName  = "TauRecContainer"        );
  declareProperty("OutputContainerName", OutputContainerName = "SelectedTauJetContainer");
  
  declareProperty("SelectionInfoContainerName", SelectionInfoContainerName = "None"           );
  declareProperty("SelectionLabel",             SelectionLabel             = "TauJetSelection");
      
  declareProperty("Author",                Author                = "All"   );
  declareProperty("TauFlag",               TauFlag               = "None"  );
  declareProperty("PtCut",                 PtCut                 = 20.0*GeV);
  declareProperty("EtaCut",                EtaCut                = 2.5     );
  declareProperty("MinTrackNumber",        MinTrackNumber        = 0       );
  declareProperty("MaxTrackNumber",        MaxTrackNumber        = 10      );
  declareProperty("HadronicEtFractionCut", HadronicEtFractionCut = 0.1     );
  declareProperty("DiscriminantName",      DiscriminantName                );
  declareProperty("DiscriminantCut",       DiscriminantCut                 );
  declareProperty("DiscriminantType",      DiscriminantType                );
  
}

TauJetSelection::~TauJetSelection(){}

StatusCode TauJetSelection::initialize()
{
  
  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Initializing " << name() << endreq;
  
  MsgLog << MSG::INFO << "Input  Container Name = " << InputContainerName  << endreq;
  MsgLog << MSG::INFO << "Output Container Name = " << OutputContainerName << endreq;
  
  if(SelectionInfoContainerName != "None")
  {
    MsgLog << MSG::INFO << "Selection Info Container Name = " << SelectionInfoContainerName << endreq;
    MsgLog << MSG::INFO << "Selection Label               = " << SelectionLabel             << endreq;
  }
    
  MsgLog << MSG::INFO << "Author                   = " << Author                << endreq;
  MsgLog << MSG::INFO << "TauFlag                  = " << TauFlag               << endreq;
  MsgLog << MSG::INFO << "Pt Cut                   = " << PtCut/GeV << " GeV"   << endreq;
  MsgLog << MSG::INFO << "Eta Cut                  = " << EtaCut                << endreq;
  MsgLog << MSG::INFO << "Min Track Number         = " << MinTrackNumber        << endreq;
  MsgLog << MSG::INFO << "Max Track Number         = " << MaxTrackNumber        << endreq;
  MsgLog << MSG::INFO << "Hadronic Et Fraction Cut = " << HadronicEtFractionCut << endreq;
        
  bool isTestAuthor  = this->TestAuthorOption ();
  bool isTestTauFlag = this->TestTauFlagOption();
  
  if(!isTestAuthor)
  {
    MsgLog << MSG::FATAL << "Wrong Author Property" << endreq;
    return StatusCode::FAILURE;
  }
  
  if(!isTestTauFlag)
  {
    MsgLog << MSG::FATAL << "Wrong Tau Flag Property" << endreq;
    return StatusCode::FAILURE;
  }
  
  bool Test1 = DiscriminantName.size() == DiscriminantCut.size();
  bool Test2 = DiscriminantName.size() == DiscriminantType.size();
  
  bool isTestDiscriminantSize = Test1 && Test2;
  
  if(!isTestDiscriminantSize)
  {
    MsgLog << MSG::FATAL << "Wrong Discriminant Size Property" << endreq;
    return StatusCode::FAILURE;
  }
  
  bool isTestDiscriminantName = this->TestDiscriminantName();
        
  if(!isTestDiscriminantName)
  {
    MsgLog << MSG::FATAL << "Wrong Discriminant Name or Type Property" << endreq;
    return StatusCode::FAILURE;
  }
  
  for(unsigned int i=0; i<DiscriminantName.size(); i++)
  {
    MsgLog << MSG::INFO << "DiscriminantName = " << DiscriminantName[i] << endreq;
    MsgLog << MSG::INFO << "DiscriminantCut  = " << DiscriminantCut[i]  << endreq;
    MsgLog << MSG::INFO << "DiscriminantType = " << DiscriminantType[i] << endreq;
  }
  
  StatusCode status = service("StoreGateSvc", aStoreGateSvc);

  if(status.isFailure()) 
  {
    MsgLog << MSG::FATAL << "StoreGateSvc Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
    
  InputEventNumber  = 0;
  OutputEventNumber = 0;
  
  InputObjectNumber  = 0;
  OutputObjectNumber = 0;
      
  return StatusCode::SUCCESS;
  
}

StatusCode TauJetSelection::finalize()
{
  
  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Finalizing " << name() << endreq;
  
  MsgLog << MSG::INFO << "Input  Event Number  = " << InputEventNumber  << endreq;
  MsgLog << MSG::INFO << "Output Event Number  = " << OutputEventNumber << endreq;
  
  MsgLog << MSG::INFO << "Input  Object Number = " << InputObjectNumber  << endreq;
  MsgLog << MSG::INFO << "Output Object Number = " << OutputObjectNumber << endreq;
  
  return StatusCode::SUCCESS;
  
}

StatusCode TauJetSelection::execute()
{
  
  MsgStream MsgLog(msgSvc(), name());
  
  StatusCode status;
  
  InputEventNumber++;
      
  status = aStoreGateSvc->retrieve(aInputContainer, InputContainerName);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Input Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
    
  aOutputContainer = new Analysis::TauJetContainer(SG::VIEW_ELEMENTS);
  
  status = aStoreGateSvc->record(aOutputContainer, OutputContainerName);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Output Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  status = aStoreGateSvc->retrieve(aSelectionInfoContainer, SelectionInfoContainerName);
  
  if(status.isFailure())
  {
    
    if(SelectionInfoContainerName != "None")
    {
    
      if(InputEventNumber == 1) MsgLog << MSG::INFO << "Creating Selection Info Container" << endreq;
    
      aSelectionInfoContainer = new SelectionInfoContainer(SG::VIEW_ELEMENTS);
    
      status = aStoreGateSvc->record(aSelectionInfoContainer, SelectionInfoContainerName);
    
      if(status.isFailure())
      {
        MsgLog << MSG::FATAL << "Selection Info Container Unavailable" << endreq;
        return StatusCode::FAILURE;
      }
    
    }
    
    else 
    {
      aSelectionInfoContainer = 0;
      if(InputEventNumber == 1) MsgLog << MSG::INFO << "No Selection Info Container" << endreq;
    }
        
  }
     
  Mask.clear();
  
  this->Select();
  
  if(aSelectionInfoContainer)
  {  
    SelectionInfo * aSelectionInfo = new SelectionInfo();
  
    aSelectionInfo->SetName(InputContainerName);
    aSelectionInfo->SetLabel(SelectionLabel);
    aSelectionInfo->SetMask(Mask);
    
    aSelectionInfoContainer->push_back(aSelectionInfo);
  }
  
  status = aStoreGateSvc->setConst(aOutputContainer);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Output Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  InputObjectNumber  += static_cast<int>(aInputContainer->size());
  OutputObjectNumber += static_cast<int>(aOutputContainer->size());
  
  setFilterPassed(true);
  
  OutputEventNumber++;
    
  return StatusCode::SUCCESS;
  
}

void TauJetSelection::Select()
{

  Analysis::TauJetContainer::const_iterator Itr;
  
  for(Itr=aInputContainer->begin(); Itr!= aInputContainer->end(); Itr++)
  {
    
    const Analysis::TauJet * aTauJet = *Itr;
         
    bool isSelected = this->SelectTauJet(aTauJet);
        
    if(isSelected)
    { 
      Analysis::TauJet * aSelectedTauJet = new Analysis::TauJet(*aTauJet);
      aOutputContainer->push_back(aSelectedTauJet);
      
      Mask.push_back(true);
    }
    
    else 
    {
      Mask.push_back(false);
    }
        
  }
    
}

bool TauJetSelection::SelectTauJet(const Analysis::TauJet * aTauJet)
{

  bool isPtCut       = aTauJet->pt() > PtCut;
  bool isEtaCut      = fabs(aTauJet->eta()) < EtaCut;
  bool isTrackNumber = (aTauJet->numTrack() >= MinTrackNumber) && (aTauJet->numTrack() <= MaxTrackNumber);
  bool isHadronicCut = (aTauJet->parameter(TauJetParameters::etHadCalib))/(aTauJet->et()) > HadronicEtFractionCut;
    
  bool isAuthor      = this->TestAuthor(aTauJet);
  bool isTauFlag     = this->TestTauFlag(aTauJet);
  bool isDiscriCut   = this->TestDiscriminant(aTauJet);
            
  bool isSelected = isPtCut && isEtaCut && isTrackNumber && isHadronicCut && isAuthor && isTauFlag && isDiscriCut;
  
  return isSelected;

}

bool TauJetSelection::TestAuthorOption()
{

  bool Test = false;
  
  if     (Author == "All")         Test = true;
  else if(Author == "OnlyTauRec")  Test = true;
  else if(Author == "OnlyTau1P3P") Test = true;
  
  return Test;

}

bool TauJetSelection::TestAuthor(const Analysis::TauJet * aTauJet)
{

  int author = aTauJet->author();

  bool Test = false;
  
  if     (Author == "All")         Test = true;
  else if(Author == "OnlyTauRec")  Test = (author == TauJetParameters::tauRec);
  else if(Author == "OnlyTau1P3P") Test = (author == TauJetParameters::tau1P3P);
      
  return Test; 
  
}

bool TauJetSelection::TestTauFlagOption()
{

  bool Test = false;
  
  if     (TauFlag == "None")   Test = true;
  else if(TauFlag == "Medium") Test = true;
  else if(TauFlag == "Loose")  Test = true;
  else if(TauFlag == "Tight")  Test = true;
  
  return Test;  

}

bool TauJetSelection::TestTauFlag(const Analysis::TauJet * aTauJet)
{

  const Analysis::TauPID * aTauPID = aTauJet->tauID();

  bool Test = false;  
  
  if     (TauFlag == "None")   Test = true;
  else if(TauFlag == "Medium") Test = aTauPID->isTau(TauJetParameters::TauCutMedium);
  else if(TauFlag == "Loose")  Test = aTauPID->isTau(TauJetParameters::TauCutLoose);
  else if(TauFlag == "Tight")  Test = aTauPID->isTau(TauJetParameters::TauCutTight);
  
  return Test;  

}

bool TauJetSelection::TestDiscriminantName()
{
  
  for(unsigned int i=0; i<DiscriminantName.size(); i++)
  {
  
    std::string Name = DiscriminantName[i];
  
    if     (Name == "Likelihood")         DiscriminantIdent.push_back(TauJetParameters::Likelihood);
    else if(Name == "TauElTauLikelihood") DiscriminantIdent.push_back(TauJetParameters::TauElTauLikelihood);
    else return false;
    
    std::string Type = DiscriminantType[i];
    
    if(Type != "Min" || Type != "Max") return false;
  
  }
  
  return true;  

}

bool TauJetSelection::TestDiscriminant(const Analysis::TauJet * aTauJet)
{

  const Analysis::TauPID * aTauPID = aTauJet->tauID();
  
  bool Test = true;

  for(unsigned int i=0; i<DiscriminantIdent.size(); i++)
  {
    TauJetParameters::TauID Ident = TauJetParameters::TauID(DiscriminantIdent[i]);
     
    if     (DiscriminantType[i] == "Min") Test = Test && (aTauPID->discriminant(Ident) > DiscriminantCut[i]);
    else if(DiscriminantType[i] == "Max") Test = Test && (aTauPID->discriminant(Ident) < DiscriminantCut[i]);
  }
  
  return Test;  

}
