#include "SingleTopDPDMaker/TreeMaker.h"

TreeMaker::TreeMaker(const std::string& name, ISvcLocator* pSvcLocator) : Algorithm(name, pSvcLocator)
{
  
  declareProperty("McEventContainerName",       McEventContainerName       = "GEN_AOD");
  declareProperty("EventInfoName",              EventInfoName              = "None"   );
  declareProperty("TrackParticleContainerName", TrackParticleContainerName = "None"   );
  
  declareProperty("MissingEtName",  MissingEtName );
  declareProperty("MissingEtLabel", MissingEtLabel);
  
  declareProperty("ContainerName",  ContainerName );
  declareProperty("ContainerLabel", ContainerLabel);
  
  declareProperty("PtCut", PtCut);
  
  declareProperty("TreeName", TreeName= "CollectionTree");
  
  declareProperty("TrackMuonDRCutMin", TrackMuonDRCutMin = 0.01   );
  declareProperty("TrackMuonDRCutMax", TrackMuonDRCutMax = 0.45   );
  declareProperty("TrackMuonPtCut",    TrackMuonPtCut    = 1.0*GeV);
        
}

TreeMaker::~TreeMaker(){}

StatusCode TreeMaker::initialize()
{

  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Initializing " << name() << endreq;
    
  MsgLog << MSG::INFO << "MC Event Container Name       = " << McEventContainerName       << endreq;
  MsgLog << MSG::INFO << "Event Info Name               = " << EventInfoName              << endreq;
  MsgLog << MSG::INFO << "Track Particle Container Name = " << TrackParticleContainerName << endreq;
    
  MsgLog << MSG::INFO << "Tree Name = " << TreeName << endreq;
  
  MsgLog << MSG::INFO << "Track Muon DR Cut Min = " << TrackMuonDRCutMin            << endreq;
  MsgLog << MSG::INFO << "Track Muon DR Cut Max = " << TrackMuonDRCutMax            << endreq;
  MsgLog << MSG::INFO << "Track Muon Pt Cut     = " << TrackMuonPtCut/GeV << " GeV" << endreq;
  
  bool isTestMissingEtSize = MissingEtName.size() == MissingEtLabel.size();
  
  if(!isTestMissingEtSize)
  {
    MsgLog << MSG::FATAL << "Wrong Missing Et Size Properties" << endreq;
    return StatusCode::FAILURE;
  }
    
  bool Test1 = ContainerName.size() == ContainerLabel.size();
  bool Test2 = ContainerName.size() == PtCut.size();
    
  bool isTestContainerSize = Test1 && Test2;
  
  if(!isTestContainerSize)
  {
    
    MsgLog << MSG::FATAL << "Wrong Container Size Properties :" << endreq;
    
    if(!Test1) MsgLog << MSG::FATAL << "Wrong Container Label Size" << endreq;
    if(!Test2) MsgLog << MSG::FATAL << "Wrong PtCut Size"           << endreq;
    
    return StatusCode::FAILURE;
    
  }
  
  bool isTestContainerType = this->TestContainerType();
    
  if(!isTestContainerType)
  {
    MsgLog << MSG::FATAL << "Wrong Container Type Properties" << endreq;
    return StatusCode::FAILURE;
  }
  
  bool isTestContainerLabel = this->TestContainerLabel();
    
  if(!isTestContainerLabel)
  {
    MsgLog << MSG::FATAL << "Wrong Container Label Properties" << endreq;
    return StatusCode::FAILURE;
  }
  
  for(unsigned int i=0; i<MissingEtName.size(); i++)
  {
    MsgLog << MSG::INFO << "Missing Et Name  = " << MissingEtName[i]  << endreq;
    MsgLog << MSG::INFO << "Missing Et Label = " << MissingEtLabel[i] << endreq;
  }
  
  for(unsigned int i=0; i<PhotonContainerName.size(); i++)
  {
    MsgLog << MSG::INFO << "Photon Container Name  = " << PhotonContainerName[i]  << endreq;
    MsgLog << MSG::INFO << "Photon Container Label = " << PhotonContainerLabel[i] << endreq;
        
    MsgLog << MSG::INFO << "Photon Pt Cut = " << PhotonPtCut[PhotonContainerLabel[i]]/GeV << " GeV" << endreq;
  }
  
  for(unsigned int i=0; i<ElectronContainerName.size(); i++)
  {
    MsgLog << MSG::INFO << "Electron Container Name  = " << ElectronContainerName[i]  << endreq;
    MsgLog << MSG::INFO << "Electron Container Label = " << ElectronContainerLabel[i] << endreq;
     
    MsgLog << MSG::INFO << "Electron Pt Cut = " << ElectronPtCut[ElectronContainerLabel[i]]/GeV << " GeV" << endreq;
  }
  
  for(unsigned int i=0; i<MuonContainerName.size(); i++)
  {
    MsgLog << MSG::INFO << "Muon Container Name  = " << MuonContainerName[i]  << endreq;
    MsgLog << MSG::INFO << "Muon Container Label = " << MuonContainerLabel[i] << endreq;
   
    MsgLog << MSG::INFO << "Muon Pt Cut = " << MuonPtCut[MuonContainerLabel[i]]/GeV << " GeV" << endreq;
  }
  
  for(unsigned int i=0; i<TauJetContainerName.size(); i++)
  {
    MsgLog << MSG::INFO << "TauJet Container Name  = " << TauJetContainerName[i]  << endreq;
    MsgLog << MSG::INFO << "TauJet Container Label = " << TauJetContainerLabel[i] << endreq;
    
    MsgLog << MSG::INFO << "TauJet Pt Cut = " << TauJetPtCut[TauJetContainerLabel[i]]/GeV << " GeV" << endreq;
  }
  
  for(unsigned int i=0; i<JetContainerName.size(); i++)
  {
    MsgLog << MSG::INFO << "Jet Container Name  = " << JetContainerName[i]  << endreq;
    MsgLog << MSG::INFO << "Jet Container Label = " << JetContainerLabel[i] << endreq;
    
    MsgLog << MSG::INFO << "Jet Pt Cut = " << JetPtCut[JetContainerLabel[i]]/GeV << " GeV" << endreq;
  }
        
  StatusCode status;
  
  status = service("StoreGateSvc", aStoreGateSvc);

  if(status.isFailure()) 
  {
    MsgLog << MSG::FATAL << "StoreGateSvc Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  status = service("THistSvc", aTHistSvc);

  if(status.isFailure()) 
  {
    MsgLog << MSG::FATAL << "THistSvc Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  std::string TreeTitle = TreeName;
  aTree = new TTree(TreeName.c_str(), TreeTitle.c_str());
  
  std::string FullTreeName = "/AANT/"+TreeName;
  status = aTHistSvc->regTree(FullTreeName, aTree);

  if(status.isFailure()) 
  {
    MsgLog << MSG::FATAL << "regTree Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
      
  this->SetBranches(); 
      
  InputEventNumber  = 0;
  OutputEventNumber = 0;
  
  return StatusCode::SUCCESS;
  
}

StatusCode TreeMaker::finalize()
{
  
  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Finalizing " << name() << endreq;
  
  MsgLog << MSG::INFO << "Input  Event Number = " << InputEventNumber  << endreq;
  MsgLog << MSG::INFO << "Output Event Number = " << OutputEventNumber << endreq;
  
  this->DeleteBranches();
    
  return StatusCode::SUCCESS;
  
}

StatusCode TreeMaker::execute()
{

  MsgStream MsgLog(msgSvc(), name());
  
  StatusCode status;

  InputEventNumber++;
  
  if(!(InputEventNumber%1000)) 
  {
    MsgLog << MSG::INFO << "Current Event Number = " << InputEventNumber << endreq;
  }
  
  status = aStoreGateSvc->retrieve(aMcEventContainer, McEventContainerName);
  
  if(status.isFailure())
  {
    aMcEventContainer = 0;
    if(InputEventNumber == 1) MsgLog << MSG::INFO << "No MC Event Container" << endreq;
  }
  
  status = aStoreGateSvc->retrieve(aEventInfo, EventInfoName);
  
  if(status.isFailure())
  {
    aEventInfo = 0;
    if(InputEventNumber == 1) MsgLog << MSG::INFO << "No Event Info" << endreq;
  }
    
  status = aStoreGateSvc->retrieve(aTrackParticleContainer, TrackParticleContainerName);
  
  if(status.isFailure())
  {
    aTrackParticleContainer = 0;
    if(InputEventNumber == 1) MsgLog << MSG::INFO << "No Track Particle Container" << endreq;
  }
  
  for(unsigned int i=0; i<MissingEtName.size(); i++)
  {
    
    std::string Label = MissingEtLabel[i];
    
    status = aStoreGateSvc->retrieve(aMissingEt[Label], MissingEtName[i]);
  
    if(status.isFailure())
    {
      MsgLog << MSG::FATAL << MissingEtName[i] << " Unavailable" << endreq;
      return StatusCode::FAILURE;
    }
  
  }
  
  for(unsigned int i=0; i<PhotonContainerName.size(); i++)
  {
      
    std::string Label = PhotonContainerLabel[i];
    
    status = aStoreGateSvc->retrieve(aPhotonContainer[Label], PhotonContainerName[i]);
  
    if(status.isFailure())
    {
      MsgLog << MSG::FATAL << PhotonContainerName[i] << " Unavailable" << endreq;
      return StatusCode::FAILURE;
    }
              
  }
  
  for(unsigned int i=0; i<ElectronContainerName.size(); i++)
  {
      
    std::string Label = ElectronContainerLabel[i];
    
    status = aStoreGateSvc->retrieve(aElectronContainer[Label], ElectronContainerName[i]);
  
    if(status.isFailure())
    {
      MsgLog << MSG::FATAL << ElectronContainerName[i] << " Unavailable" << endreq;
      return StatusCode::FAILURE;
    }
    
  }
  
  for(unsigned int i=0; i<MuonContainerName.size(); i++)
  {
      
    std::string Label = MuonContainerLabel[i];
    
    status = aStoreGateSvc->retrieve(aMuonContainer[Label], MuonContainerName[i]);
  
    if(status.isFailure())
    {
      MsgLog << MSG::FATAL << MuonContainerName[i] << " Unavailable" << endreq;
      return StatusCode::FAILURE;
    }
    
  }
  
  for(unsigned int i=0; i<TauJetContainerName.size(); i++)
  {
      
    std::string Label = TauJetContainerLabel[i];
    
    status = aStoreGateSvc->retrieve(aTauJetContainer[Label], TauJetContainerName[i]);
  
    if(status.isFailure())
    {
      MsgLog << MSG::FATAL << TauJetContainerName[i] << " Unavailable" << endreq;
      return StatusCode::FAILURE;
    }
            
  }
  
  for(unsigned int i=0; i<JetContainerName.size(); i++)
  {
      
    std::string Label = JetContainerLabel[i];
    
    status = aStoreGateSvc->retrieve(aJetContainer[Label], JetContainerName[i]);
  
    if(status.isFailure())
    {
      MsgLog << MSG::FATAL << JetContainerName[i] << " Unavailable" << endreq;
      return StatusCode::FAILURE;
    }
        
  }
                  
  this->ClearBranches();
  this->FillBranches();
  
  OutputEventNumber++;
        
  return StatusCode::SUCCESS;
  
}

void TreeMaker::SetBranches()
{

  if(EventInfoName != "None") this->SetEventInfo();

  this->SetMissingEt();
  this->SetPhotons  ();
  this->SetElectrons();
  this->SetMuons    ();
  this->SetTauJets  ();
  this->SetJets     ();
  
}

void TreeMaker::DeleteBranches()
{

  this->DeletePhotons  ();
  this->DeleteElectrons();
  this->DeleteMuons    ();
  this->DeleteTauJets  ();
  this->DeleteJets     ();

}

void TreeMaker::ClearBranches()
{

  this->ClearEventInfo();
  this->ClearMissingEt();
  this->ClearPhotons  ();
  this->ClearElectrons();
  this->ClearMuons    ();
  this->ClearTauJets  ();
  this->ClearJets     ();
  
}

void TreeMaker::FillBranches()
{

  this->FillEventInfo();
  this->FillMissingEt();
  this->FillPhotons  ();
  this->FillElectrons();
  this->FillMuons    ();
  this->FillTauJets  ();
  this->FillJets     ();
  
  aTree->Fill();

}

bool TreeMaker::TestContainerType()
{

  for(unsigned int i=0; i<ContainerName.size(); i++)
  {
    
    std::string Name  = ContainerName[i];
    std::string Label = ContainerLabel[i];
    
    double MinPt = PtCut[i];
    
    if(Name.find("Photon", 0) != std::string::npos) 
    {
      PhotonContainerName.push_back(Name);
      PhotonContainerLabel.push_back(Label);
            
      PhotonPtCut[Label] = MinPt;
    }
    
    else if(Name.find("Electron", 0) != std::string::npos) 
    {
      ElectronContainerName.push_back(Name);
      ElectronContainerLabel.push_back(Label);
      
      ElectronPtCut[Label] = MinPt;
    }
    
    else if(Name.find("Muon", 0) != std::string::npos) 
    {
      MuonContainerName.push_back(Name);
      MuonContainerLabel.push_back(Label);
            
      MuonPtCut[Label] = MinPt;
    }
    
    else if(Name.find("Tau", 0) != std::string::npos) 
    {
      TauJetContainerName.push_back(Name);
      TauJetContainerLabel.push_back(Label);
            
      TauJetPtCut[Label] = MinPt;
    }
    
    else if(Name.find("Jet", 0) != std::string::npos) 
    {
      JetContainerName.push_back(Name);
      JetContainerLabel.push_back(Label);
      
      JetPtCut[Label] = MinPt;
    }
    
    else return false;
    
  }
    
  return true;
  
}

bool TreeMaker::TestContainerLabel()
{

  bool Test = true;

  for(unsigned int i=0; i<ContainerLabel.size()-1; i++)
  {
  
    std::string Label1 = ContainerLabel[i];
  
    for(unsigned int j=i+1; j<ContainerLabel.size(); j++)
    {
      std::string Label2 = ContainerLabel[j];
      if(Label1 == Label2) Test = false;
    }
  
  }
  
  return Test;

}
