#include "SingleTopDPDMaker/TriggerDecisionFilter.h"

TriggerDecisionFilter::TriggerDecisionFilter(const std::string& name, ISvcLocator* pSvcLocator) : Algorithm(name, pSvcLocator), 
aTriggerDecisionTool("TrigDec::TrigDecisionTool")
{
 
  declareProperty("AcceptedChainName",       AcceptedChainName                              );
  declareProperty("SelectedChainName",       SelectedChainName                              );
  declareProperty("TriggerDecisionInfoName", TriggerDecisionInfoName = "TriggerDecisionInfo");
  declareProperty("TriggerDecisionTool",     aTriggerDecisionTool                           );

}

TriggerDecisionFilter::~TriggerDecisionFilter(){}

StatusCode TriggerDecisionFilter::initialize()
{

  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Initializing " << name() << endreq;
    
  for(unsigned int i=0; i<AcceptedChainName.size(); i++)
  {
    MsgLog << MSG::INFO << "Accepted Chain Name = " << AcceptedChainName[i] << endreq;
  }
  
  MsgLog << MSG::INFO << "Trigger Decision Info Name = " << TriggerDecisionInfoName << endreq;
  
  for(unsigned int i=0; i<SelectedChainName.size(); i++)
  {
    MsgLog << MSG::INFO << "Selected Chain Name = " << SelectedChainName[i] << endreq;
  }
  
  StatusCode status;

  status = service("StoreGateSvc", aStoreGateSvc);

  if(status.isFailure()) 
  {
    MsgLog << MSG::FATAL << "StoreGateSvc Unavailable" << endreq;
    return StatusCode::FAILURE;
  }

  status = aTriggerDecisionTool.retrieve();
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Trigger Decision Tool Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  InputEventNumber  = 0;
  OutputEventNumber = 0;

  return StatusCode::SUCCESS;

}

StatusCode TriggerDecisionFilter::finalize()
{

  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Finalizing " << name() << endreq;
  
  MsgLog << MSG::INFO << "Input  Event Number = " << InputEventNumber  << endreq;
  MsgLog << MSG::INFO << "Output Event Number = " << OutputEventNumber << endreq;
    
  return StatusCode::SUCCESS;

}

StatusCode TriggerDecisionFilter::execute()
{

  InputEventNumber++;
              
  this->GetDecision();
  this->RecordDecision();
  
  bool isSelected = this->SelectDecision();
             
  setFilterPassed(isSelected);
  
  if(isSelected) OutputEventNumber++;
      
  return StatusCode::SUCCESS;

}

void TriggerDecisionFilter::GetDecision()
{

  PassedTrigger     = aTriggerDecisionTool->isTriggerPassed();
  PassedLevel1      = aTriggerDecisionTool->isPassed(TrigDec::L1);
  PassedLevel2      = aTriggerDecisionTool->isPassed(TrigDec::L2);
  PassedEventFilter = aTriggerDecisionTool->isPassed(TrigDec::EF);
      
  Level1Names      = aTriggerDecisionTool->getConfiguredChainNames(TrigDec::L1);
  Level2Names      = aTriggerDecisionTool->getConfiguredChainNames(TrigDec::L2);
  EventFilterNames = aTriggerDecisionTool->getConfiguredChainNames(TrigDec::EF);
    
  std::vector<std::string>::const_iterator Itr;
    
  for(Itr=Level1Names.begin(); Itr!= Level1Names.end(); Itr++)
  {
    std::string Name = *Itr;
    if(Name != "") PassedChain[Name] = aTriggerDecisionTool->isPassed(Name);
  }
    
  for(Itr=Level2Names.begin(); Itr!= Level2Names.end(); Itr++)
  {
    std::string Name = *Itr;
    if(Name != "") PassedChain[Name] = aTriggerDecisionTool->isPassed(Name);
  }
    
  for(Itr=EventFilterNames.begin(); Itr!= EventFilterNames.end(); Itr++)
  {
    std::string Name = *Itr;
    if(Name != "") PassedChain[Name] = aTriggerDecisionTool->isPassed(Name);
  }

}
  
void TriggerDecisionFilter::RecordDecision()
{

  // DBG
  // MsgStream MsgLog(msgSvc(), name());


  if(InputEventNumber == 1)
  {
  
    std::map<std::string, bool>::const_iterator Itr;
      
    for(Itr=PassedChain.begin(); Itr!=PassedChain.end(); Itr++)
    {
      
      std::string Name = Itr->first;

      // DBG
      // MsgLog << MSG::INFO << "DBG: Trigger list: " << Name << ": " << Itr->second << endreq;
      
      if(SelectedChainName.size() > 0)
      {
      
        std::vector<std::string>::const_iterator SelectedItr;
  
        for(SelectedItr=SelectedChainName.begin(); SelectedItr!=SelectedChainName.end(); SelectedItr++)
        {
    
          std::string SelectedName = *SelectedItr;
	
	  if(Name.find(SelectedName, 0) != std::string::npos)
	  {
	    PassedSelectedChain[Name] = (int)PassedChain[Name];
	    break;
	  }
	
        }
      
      }
      
      else PassedSelectedChain[Name] = (int)PassedChain[Name];
      
    }
    
  }
  
  else
  {
  
    std::map<std::string, int>::const_iterator Itr;
  
    for(Itr=PassedSelectedChain.begin(); Itr!=PassedSelectedChain.end(); Itr++)
    {
      std::string Name = Itr->first;
      PassedSelectedChain[Name] = (int)PassedChain[Name];
    }
  
  }
  
  aTriggerDecisionInfo = new TriggerDecisionInfo();
  
  aStoreGateSvc->record(aTriggerDecisionInfo, TriggerDecisionInfoName);
  
  aTriggerDecisionInfo->SetPassedTrigger(    (int)PassedTrigger    );
  aTriggerDecisionInfo->SetPassedLevel1(     (int)PassedLevel1     );
  aTriggerDecisionInfo->SetPassedLevel2(     (int)PassedLevel2     );
  aTriggerDecisionInfo->SetPassedEventFilter((int)PassedEventFilter); 
  
  aTriggerDecisionInfo->SetPassedChain(PassedSelectedChain);    
  
  aStoreGateSvc->setConst(aTriggerDecisionInfo);

}

bool TriggerDecisionFilter::SelectDecision()
{

  // DBG
  // MsgStream MsgLog(msgSvc(), name());


  if(AcceptedChainName.size() == 0) return true;
  
  bool isSelected = false;
  
  std::vector<std::string>::const_iterator AcceptedItr;
    
  for(AcceptedItr=AcceptedChainName.begin(); AcceptedItr!=AcceptedChainName.end(); AcceptedItr++)
  {
  
    std::string AcceptedName = *AcceptedItr;
    
    bool isPassed = false;
            
    std::map<std::string, bool>::const_iterator Itr;
    
    for(Itr=PassedChain.begin(); Itr!=PassedChain.end(); Itr++)
    {
      
      std::string Name = Itr->first;

      // DBG
      // MsgLog << MSG::INFO << "DBG: evt #" << InputEventNumber << " " << Name << ": " << Itr->second << endreq;
                  
      if(Name.find(AcceptedName, 0) != std::string::npos)
      {
	isPassed = Itr->second;
	// DBG 
	// if (isPassed)  MsgLog << MSG::INFO << "DBG: Passed!" << endreq;

	if(isPassed) break;
      }
      
    }
    
    if(isPassed)
    {
      isSelected = true;
      break;
    }
        
  }
  
  return isSelected;

}
