#include "SingleTopDPDMaker/TruthParticleSelection.h"

TruthParticleSelection::TruthParticleSelection(const std::string& name, ISvcLocator* pSvcLocator) : Algorithm(name, pSvcLocator)
{

  declareProperty("McEventContainerName", McEventContainerName = "GEN_AOD"               );
  declareProperty("InputContainerName",   InputContainerName   = "SpclMC"                );
  declareProperty("OutputContainerName",  OutputContainerName  = "SelectedTruthContainer");
  
  declareProperty("SelectionInfoContainerName", SelectionInfoContainerName = "None"                   );
  declareProperty("SelectionLabel",             SelectionLabel             = "TruthParticleSSelection"); 
  
  declareProperty("ApplyIsGeneratorCut", ApplyIsGeneratorCut = false   );
  declareProperty("ApplyIsStableCut",    ApplyIsStableCut    = false   );
  declareProperty("ApplySelfDecayCut",   ApplySelfDecayCut   = false   );
  declareProperty("PtCut",               PtCut               = 0.0*GeV );
  declareProperty("EtaCut",              EtaCut              = 10.0    );
  
  declareProperty("PdgId",               PdgId                         );
  declareProperty("Status",              Status                        );
  declareProperty("VetoStatus",          VetoStatus                    );
      
}

TruthParticleSelection::~TruthParticleSelection(){}

StatusCode TruthParticleSelection::initialize()
{
  
  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Initializing " << name() << endreq;
  
  MsgLog << MSG::INFO << "MC Event Container Name = " << McEventContainerName << endreq;
  MsgLog << MSG::INFO << "Input    Container Name = " << InputContainerName   << endreq;
  MsgLog << MSG::INFO << "Output   Container Name = " << OutputContainerName  << endreq;
  
  if(SelectionInfoContainerName != "None")
  {
    MsgLog << MSG::INFO << "Selection Info Container Name = " << SelectionInfoContainerName << endreq;
    MsgLog << MSG::INFO << "Selection Label               = " << SelectionLabel             << endreq;
  }
  
  MsgLog << MSG::INFO << "Apply Is Generator Cut  = " << ApplyIsGeneratorCut << endreq;
  MsgLog << MSG::INFO << "Apply Is Stable Cut     = " << ApplyIsStableCut    << endreq;
  MsgLog << MSG::INFO << "Apply Self Decay Cut    = " << ApplySelfDecayCut   << endreq;
  
  MsgLog << MSG::INFO << "Pt Cut  = " << PtCut/GeV << " GeV" << endreq;
  MsgLog << MSG::INFO << "Eta Cut = " << EtaCut              << endreq;
    
  if(PdgId.size() > 0)
  {
    MsgLog << MSG::INFO << "PdgId = " << PdgId << endreq;
  }
  
  if(Status.size() > 0)
  {
    MsgLog << MSG::INFO << "Status = " << Status << endreq;
  }
  
  if(VetoStatus.size() > 0)
  {
    MsgLog << MSG::INFO << "Veto Status = " << Status << endreq;
  }
        
  StatusCode status = service("StoreGateSvc", aStoreGateSvc);

  if(status.isFailure()) 
  {
    MsgLog << MSG::FATAL << "StoreGateSvc Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
    
  InputEventNumber  = 0;
  OutputEventNumber = 0;
  
  InputObjectNumber  = 0;
  OutputObjectNumber = 0;
      
  return StatusCode::SUCCESS;
  
}

StatusCode TruthParticleSelection::finalize()
{
  
  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Finalizing " << name() << endreq;
  
  MsgLog << MSG::INFO << "Input  Event Number  = " << InputEventNumber  << endreq;
  MsgLog << MSG::INFO << "Output Event Number  = " << OutputEventNumber << endreq;
  
  MsgLog << MSG::INFO << "Input  Object Number = " << InputObjectNumber  << endreq;
  MsgLog << MSG::INFO << "Output Object Number = " << OutputObjectNumber << endreq;
  
  return StatusCode::SUCCESS;
  
}

StatusCode TruthParticleSelection::execute()
{
  
  MsgStream MsgLog(msgSvc(), name());
  
  StatusCode status;
  
  InputEventNumber++;
  
  if(InputEventNumber == 1)
  {
  
    const McEventCollection * aMcEventContainer;
  
    status = aStoreGateSvc->retrieve(aMcEventContainer, McEventContainerName);
  
    if(status.isFailure())
    {
      MsgLog << MSG::FATAL << "Mandatory MC Event Container Unavailable" << endreq;
      return StatusCode::FAILURE;
    }
  
  }
  
  status = aStoreGateSvc->retrieve(aInputContainer, InputContainerName);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Input Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  aOutputContainer = new TruthParticleContainer(SG::VIEW_ELEMENTS);
  
  status = aStoreGateSvc->record(aOutputContainer, OutputContainerName);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Output Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  status = aStoreGateSvc->retrieve(aSelectionInfoContainer, SelectionInfoContainerName);
  
  if(status.isFailure())
  {
    
    if(SelectionInfoContainerName != "None")
    {
    
      if(InputEventNumber == 1) MsgLog << MSG::INFO << "Creating Selection Info Container" << endreq;
    
      aSelectionInfoContainer = new SelectionInfoContainer(SG::VIEW_ELEMENTS);
    
      status = aStoreGateSvc->record(aSelectionInfoContainer, SelectionInfoContainerName);
    
      if(status.isFailure())
      {
        MsgLog << MSG::FATAL << "Selection Info Container Unavailable" << endreq;
        return StatusCode::FAILURE;
      }
    
    }
    
    else 
    {
      aSelectionInfoContainer = 0;
      if(InputEventNumber == 1) MsgLog << MSG::INFO << "No Selection Info Container" << endreq;
    }
        
  }
      
  Mask.clear();
    
  this->Select();
  
  if(aSelectionInfoContainer)
  {  
    SelectionInfo * aSelectionInfo = new SelectionInfo();
  
    aSelectionInfo->SetName(InputContainerName);
    aSelectionInfo->SetLabel(SelectionLabel);
    aSelectionInfo->SetMask(Mask);
    
    aSelectionInfoContainer->push_back(aSelectionInfo);
  }
  
  status = aStoreGateSvc->setConst(aOutputContainer);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Output Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  InputObjectNumber  += static_cast<int>(aInputContainer->size());
  OutputObjectNumber += static_cast<int>(aOutputContainer->size());
  
  setFilterPassed(true);
  
  OutputEventNumber++;
    
  return StatusCode::SUCCESS;
  
}

void TruthParticleSelection::Select()
{

  TruthParticleContainer::const_iterator Itr;
  
  for(Itr=aInputContainer->begin(); Itr!= aInputContainer->end(); Itr++)
  {
    
    const TruthParticle * aTruthParticle = *Itr;
    
    bool isSelected = this->SelectTruthParticle(aTruthParticle);
    
    if(isSelected)
    {           
      TruthParticle * aSelectedTruthParticle = new TruthParticle(*aTruthParticle);
      aOutputContainer->push_back(aSelectedTruthParticle);
      
      Mask.push_back(true);
    }
    
    else 
    {
      Mask.push_back(false);
    }
                
  } 
       
}

bool TruthParticleSelection::SelectTruthParticle(const TruthParticle * aTruthParticle)
{

  bool isPtCut  = aTruthParticle->pt() > PtCut;
  bool isEtaCut = fabs(aTruthParticle->pt()) < EtaCut;
        
  bool isPdgId      = true;
  bool isStatus     = true;
  bool isVetoStatus = true;
        
  int pdgId  = aTruthParticle->pdgId();
  int status = aTruthParticle->status();
    
  for(unsigned int i=0; i<PdgId.size(); i++)
  {
    isPdgId = isPdgId && (PdgId[i] == pdgId);
  }
        
  for(unsigned int i=0; i<Status.size(); i++)
  {
    isStatus = isStatus && (Status[i] == status);
  }

// status = 0 : null entry 
// status = 1 : not decayed or fragmented entry (final state)
// status = 2 : decayed or fragmented entry (intermediate state)
// status = 3 : documentation line
            
  for(unsigned int i=0; i<VetoStatus.size(); i++)
  {
      isVetoStatus = isVetoStatus && (VetoStatus[i] != status);
  }
        
  bool isGenerator = true;
  bool isStable    = true;
  bool isSelfDecay = false;
    
  const HepMC::GenParticle * aGenParticle = aTruthParticle->genParticle();
    
  if(ApplyIsGeneratorCut) 
  {
    TruthHelper::IsGenerator TestGenerator;
    isGenerator = TestGenerator(aGenParticle);
  }
        
  if(ApplyIsStableCut) 
  {
    TruthHelper::IsGenStable TestGenStable;
    isStable = TestGenStable(aGenParticle);
  }
            
  if(ApplySelfDecayCut)
  {
      
    HepMC::GenVertex * aGenVertex = aGenParticle->end_vertex();
            
    if(aGenVertex)
    {
    
      int ParticleIdent   = aGenParticle->pdg_id();
      int ParticleBarcode = aGenParticle->barcode();
      
      HepMC::GenVertex::particle_iterator Itr  = aGenVertex->particles_begin(HepMC::children);
      HepMC::GenVertex::particle_iterator ItrE = aGenVertex->particles_end  (HepMC::children);
      
      for(; Itr!=ItrE; Itr++)
      {
      
        int DecayIdent   = (*Itr)->pdg_id();
        int DecayBarcode = (*Itr)->barcode();
	
	isSelfDecay = (DecayIdent == ParticleIdent) && (DecayBarcode != ParticleBarcode) && (DecayBarcode < 100000);
      
      }
    
    }
    
  }
    
  bool isSelected = isPtCut && isEtaCut && isPdgId && isStatus && isVetoStatus && isGenerator && isStable && !isSelfDecay;
  
  return isSelected;

}
