#include "SingleTopDPDMaker/TruthTreeMaker.h"

TruthTreeMaker::TruthTreeMaker(const std::string& name, ISvcLocator* pSvcLocator) : Algorithm(name, pSvcLocator)
{
  
  declareProperty("MissingEtName",        MissingEtName                   );
  declareProperty("MissingEtLabel",       MissingEtLabel                  );
  declareProperty("MissingEtSource",      MissingEtSource                 );
  
  declareProperty("McEventContainerName", McEventContainerName = "GEN_AOD");
  declareProperty("ContainerName",        ContainerName                   );
  declareProperty("ContainerLabel",       ContainerLabel                  );
  
  declareProperty("TreeName",             TreeName             = "Truth"  );
          
}

TruthTreeMaker::~TruthTreeMaker(){}

StatusCode TruthTreeMaker::initialize()
{

  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Initializing " << name() << endreq;
      
  MsgLog << MSG::INFO << "Tree Name = " << TreeName << endreq;
  
  bool isTestMissingEtSize = MissingEtName.size() == MissingEtLabel.size();
  
  if(!isTestMissingEtSize)
  {
    MsgLog << MSG::FATAL << "Wrong Missing Et Size Properties" << endreq;
    return StatusCode::FAILURE;
  }
  
  bool isTestMissingEtSource = true;
  if(MissingEtName.size() > 0) isTestMissingEtSource = this->TestMissingEtSource();
  
  if(!isTestMissingEtSource)
  {
    MsgLog << MSG::FATAL << "Wrong Missing Et Source Properties" << endreq;
    return StatusCode::FAILURE;
  }
  
  bool isTestContainerSize = ContainerName.size() == ContainerLabel.size();
  
  if(!isTestContainerSize)
  {
    MsgLog << MSG::FATAL << "Wrong Container Size Properties" << endreq;
    return StatusCode::FAILURE;
  }
  
  bool isTestContainerType = this->TestContainerType();
  
  if(!isTestContainerType)
  {
    MsgLog << MSG::FATAL << "Wrong Container Type Properties" << endreq;
    return StatusCode::FAILURE;
  }
  
  bool isTestContainerLabel  = this->TestContainerLabel();
    
  if(!isTestContainerLabel)
  {
    MsgLog << MSG::FATAL << "Wrong Container Label Properties" << endreq;
    return StatusCode::FAILURE;
  }
    
  if(MissingEtName.size() > 0)
  {
    
    for(unsigned int i=0; i<MissingEtName.size(); i++)
    {
      MsgLog << MSG::INFO << "Missing Et Name   = " << MissingEtName[i]   << endreq;
      MsgLog << MSG::INFO << "Missing Et Label  = " << MissingEtLabel[i]  << endreq;
    }
    
    for(unsigned int i=0; i<MissingEtSource.size(); i++)
    {  
      MsgLog << MSG::INFO << "Missing Et Source = " << MissingEtSource[i] << endreq;
    }
    
  }
  
  if(ParticleContainerName.size() > 0)
  {
  
    MsgLog << MSG::INFO << "MC Event Container Name  = " << McEventContainerName << endreq;
  
    for(unsigned int i=0; i<ParticleContainerName.size(); i++)
    {
      MsgLog << MSG::INFO << "Particle Container Name  = " << ParticleContainerName[i]  << endreq;
      MsgLog << MSG::INFO << "Particle Container Label = " << ParticleContainerLabel[i] << endreq;
    }
        
  }
        
  StatusCode status;
  
  status = service("StoreGateSvc", aStoreGateSvc);

  if(status.isFailure()) 
  {
    MsgLog << MSG::FATAL << "StoreGateSvc Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  status = service("THistSvc", aTHistSvc);

  if(status.isFailure()) 
  {
    MsgLog << MSG::FATAL << "THistSvc Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  std::string TreeTitle = TreeName;
  aTree = new TTree(TreeName.c_str(), TreeTitle.c_str());
  
  std::string FullTreeName = "/AANT/"+TreeName;
  status = aTHistSvc->regTree(FullTreeName, aTree);

  if(status.isFailure()) 
  {
    MsgLog << MSG::FATAL << "regTree Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
        
  this->SetBranches(); 
      
  InputEventNumber  = 0;
  OutputEventNumber = 0;
  
  return StatusCode::SUCCESS;
  
}

StatusCode TruthTreeMaker::finalize()
{
  
  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Finalizing " << name() << endreq;
  
  MsgLog << MSG::INFO << "Input  Event Number = " << InputEventNumber  << endreq;
  MsgLog << MSG::INFO << "Output Event Number = " << OutputEventNumber << endreq;
  
  this->DeleteBranches();
    
  return StatusCode::SUCCESS;
  
}

StatusCode TruthTreeMaker::execute()
{

  MsgStream MsgLog(msgSvc(), name());
  
  StatusCode status;

  InputEventNumber++;
  
  if(!(InputEventNumber%1000)) 
  {
    MsgLog << MSG::INFO << "Current Event Number = " << InputEventNumber << endreq;
  }
  
  if(InputEventNumber == 1)
  {
  
    if(ParticleContainerName.size() > 0)
    {
  
      const McEventCollection * aMcEventContainer;
  
      status = aStoreGateSvc->retrieve(aMcEventContainer, McEventContainerName);
  
      if(status.isFailure())
      {
        MsgLog << MSG::FATAL << "Mandatory MC Event Container Unavailable" << endreq;
        return StatusCode::FAILURE;
      }
  
    }
  
  }
  
  for(unsigned int i=0; i<MissingEtName.size(); i++)
  {
    
    std::string Label = MissingEtLabel[i];
    
    status = aStoreGateSvc->retrieve(aMissingEt[Label], MissingEtName[i]);
  
    if(status.isFailure())
    {
      MsgLog << MSG::FATAL << MissingEtName[i] << " Unavailable" << endreq;
      return StatusCode::FAILURE;
    }
  
  }
  
  for(unsigned int i=0; i<ParticleContainerName.size(); i++)
  {
      
    std::string Label = ParticleContainerLabel[i];
    
    status = aStoreGateSvc->retrieve(aParticleContainer[Label], ParticleContainerName[i]);
  
    if(status.isFailure())
    {
      MsgLog << MSG::FATAL << ParticleContainerName[i] << " Unavailable" << endreq;
      return StatusCode::FAILURE;
    }
    
  }
                    
  this->ClearBranches();
  this->FillBranches();
  
  OutputEventNumber++;
        
  return StatusCode::SUCCESS;
  
}

void TruthTreeMaker::SetBranches()
{

  this->SetMissingEt();
  this->SetParticles();
  
}

void TruthTreeMaker::DeleteBranches()
{

  this->DeleteParticles();

}

void TruthTreeMaker::ClearBranches()
{

  this->ClearMissingEt();
  this->ClearParticles();
  
}

void TruthTreeMaker::FillBranches()
{

  this->FillMissingEt();
  this->FillParticles();
  
  aTree->Fill();

}

bool TruthTreeMaker::TestMissingEtSource()
{
  
  if(MissingEtSource.size() == 0) return false;
  
  bool Test = false;
  
  for(unsigned int i=0; i<MissingEtSource.size(); i++)
  {
  
    std::string Source = MissingEtSource[i];
    
    if(Source == "Int")
    {
      Test = true;
      MissingEtIndex.push_back(MissingEtTruth::Int);
    }
    
    else if(Source == "NonInt")
    {
      Test = true;
      MissingEtIndex.push_back(MissingEtTruth::NonInt);
    }
    
    else if(Source == "IntCentral")
    {
      Test = true;
      MissingEtIndex.push_back(MissingEtTruth::IntCentral);
    }
    
    else if(Source == "IntFwd")
    {
      Test = true;
      MissingEtIndex.push_back(MissingEtTruth::IntFwd);
    }
    
    else if(Source == "IntOutCover")
    {
      Test = true;
      MissingEtIndex.push_back(MissingEtTruth::IntOutCover);
    }
    
    else if(Source == "Muons")
    {
      Test = true;
      MissingEtIndex.push_back(MissingEtTruth::Muons);
    }
      
  }
  
  return Test;

}

bool TruthTreeMaker::TestContainerType()
{

  for(unsigned int i=0; i<ContainerName.size(); i++)
  {
    
    std::string Name  = ContainerName[i];
    std::string Label = ContainerLabel[i];
  
    if(Name.find("Truth", 0) != std::string::npos || Name == "SpclMC") 
    {
      ParticleContainerName.push_back(Name);
      ParticleContainerLabel.push_back(Label);
    }
        
    else return false;
    
  }
  
  return true;
  
}

bool TruthTreeMaker::TestContainerLabel()
{

  bool Test = true;

  for(unsigned int i=0; i<ContainerLabel.size()-1; i++)
  {
  
    std::string Label1 = ContainerLabel[i];
  
    for(unsigned int j=i+1; j<ContainerLabel.size(); j++)
    {
      std::string Label2 = ContainerLabel[j];
      if(Label1 == Label2) Test = false;
    }
  
  }
  
  return Test;

}
