#include "SingleTopDPDMaker/UntaggedJetSelection.h"

UntaggedJetSelection::UntaggedJetSelection(const std::string& name, ISvcLocator* pSvcLocator) : Algorithm(name, pSvcLocator)
{

  declareProperty("InputContainerName",  InputContainerName  = "Cone4H1TowerJets"            );
  declareProperty("OutputContainerName", OutputContainerName = "SelectedUntaggedJetContainer");
    
  declareProperty("PtCut",            PtCut            = 20.0*GeV);
  declareProperty("EtaCut",           EtaCut           = 2.5     );
  declareProperty("TaggingAlgorithm", TaggingAlgorithm = ""      );
  declareProperty("TaggingWeightCut", TaggingWeightCut = 7.05    );
  
}

UntaggedJetSelection::~UntaggedJetSelection(){}

StatusCode UntaggedJetSelection::initialize()
{
  
  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Initializing " << name() << endreq;
  
  MsgLog << MSG::INFO << "Input  Container Name = " << InputContainerName  << endreq;
  MsgLog << MSG::INFO << "Output Container Name = " << OutputContainerName << endreq;
  
  MsgLog << MSG::INFO << "Pt Cut  = " << PtCut/GeV << " GeV" << endreq;
  MsgLog << MSG::INFO << "Eta Cut = " << EtaCut              << endreq;
  
  std::string Algorithm = "Standard";
  if(TaggingAlgorithm != "") Algorithm = TaggingAlgorithm;
  
  MsgLog << MSG::INFO << "Tagging Algorithm  = " << Algorithm        << endreq;
  MsgLog << MSG::INFO << "Tagging Weight Cut = " << TaggingWeightCut << endreq;
  
  StatusCode status = service("StoreGateSvc", aStoreGateSvc);

  if(status.isFailure()) 
  {
    MsgLog << MSG::FATAL << "StoreGateSvc Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  InputEventNumber  = 0;
  OutputEventNumber = 0;
  
  InputObjectNumber  = 0;
  OutputObjectNumber = 0;
      
  return StatusCode::SUCCESS;
  
}

StatusCode UntaggedJetSelection::finalize()
{
  
  MsgStream MsgLog(msgSvc(), name());
  
  MsgLog << MSG::INFO << "Finalizing " << name() << endreq;
  
  MsgLog << MSG::INFO << "Input  Event Number  = " << InputEventNumber  << endreq;
  MsgLog << MSG::INFO << "Output Event Number  = " << OutputEventNumber << endreq;
  
  MsgLog << MSG::INFO << "Input  Object Number = " << InputObjectNumber  << endreq;
  MsgLog << MSG::INFO << "Output Object Number = " << OutputObjectNumber << endreq;
  
  return StatusCode::SUCCESS;
  
}

StatusCode UntaggedJetSelection::execute()
{
  
  MsgStream MsgLog(msgSvc(), name());
  
  StatusCode status;
  
  InputEventNumber++;
      
  status = aStoreGateSvc->retrieve(aInputContainer, InputContainerName);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Input Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
    
  aOutputContainer = new JetCollection(SG::VIEW_ELEMENTS);
  
  status = aStoreGateSvc->record(aOutputContainer, OutputContainerName);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Output Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  this->Select();
    
  status = aStoreGateSvc->setConst(aOutputContainer);
  
  if(status.isFailure())
  {
    MsgLog << MSG::FATAL << "Output Container Unavailable" << endreq;
    return StatusCode::FAILURE;
  }
  
  bool Test = this->TestTaggingAlgorithmOption();
  
  if(!Test)
  {
    MsgLog << MSG::FATAL << "Wrong Properties" << endreq;
    return StatusCode::FAILURE;
  }
  
  InputObjectNumber  += static_cast<int>(aInputContainer->size());
  OutputObjectNumber += static_cast<int>(aOutputContainer->size());
  
  setFilterPassed(true);
  
  OutputEventNumber++;
    
  return StatusCode::SUCCESS;
  
}


void UntaggedJetSelection::Select()
{
    
  JetCollection::const_iterator Itr;
    
  for(Itr=aInputContainer->begin(); Itr!= aInputContainer->end(); Itr++)
  {
    
    const Jet * aJet = *Itr;
            
    bool isSelected = this->SelectUntaggedJet(aJet);
            
    if(isSelected)
    {
      Jet * aSelectedJet = new Jet(*aJet);
      aOutputContainer->push_back(aSelectedJet);
    }
    
  }
        
}

bool UntaggedJetSelection::SelectUntaggedJet(const Jet * aJet)
{

  bool isPtCut  = aJet->pt() > PtCut;
  bool isEtaCut = fabs(aJet->eta()) < EtaCut;
    
  double TaggingWeight = aJet->getFlavourTagWeight(TaggingAlgorithm);
  bool   isTagged      = TaggingWeight > TaggingWeightCut;
    
  bool isSelected = isPtCut && isEtaCut  && !isTagged;
  
  return isSelected;

}

bool UntaggedJetSelection::TestTaggingAlgorithmOption()
{

  bool Test = false;
  
  if     (TaggingAlgorithm == "")                Test = true;
  else if(TaggingAlgorithm == "TrackCounting2D") Test = true;
  else if(TaggingAlgorithm == "JetProb")         Test = true;
  else if(TaggingAlgorithm == "IP1D")            Test = true;
  else if(TaggingAlgorithm == "IP2D")            Test = true;
  else if(TaggingAlgorithm == "IP3D")            Test = true;
  else if(TaggingAlgorithm == "SV1")             Test = true;
  else if(TaggingAlgorithm == "SV2")             Test = true;
  else if(TaggingAlgorithm == "BaselineTagger")  Test = true;
  else if(TaggingAlgorithm == "JetFitterTag")    Test = true;
  else if(TaggingAlgorithm == "JetFitterCOMB")   Test = true;
  else if(TaggingAlgorithm == "JetFitterTagNN")  Test = true;
  else if(TaggingAlgorithm == "JetFitterCOMBNN") Test = true;
  else if(TaggingAlgorithm == "SoftMuonTag")     Test = true;
  else if(TaggingAlgorithm == "SoftLeptonTag")   Test = true;
    
  return Test;

}

